#include "BarCurve.h"

#include <qwt_plot.h>
#include <qwt_curve.h>
#include <qwt_painter.h>
#include <qwt_double_rect.h>
#include <qpainter.h>

#include <stdlib.h>

QwtBarCurve::QwtBarCurve(BarStyle style, QwtPlot *parent, const char *name):
    QwtPlotCurve(parent,name)
{
bar_gap=0;
bar_style=style;
bar_pen=QPen(Qt::black,1,Qt::SolidLine);
bar_brush=QBrush(Qt::red);
QwtPlotCurve::setBrush(bar_brush);
}

void QwtBarCurve::draw(QPainter *painter,
    const QwtDiMap &xMap, const QwtDiMap &yMap, int from, int to)
{
   if ( !painter || dataSize() <= 0 )
        return;

    if (to < 0)
        to = dataSize() - 1;

    if ( verifyRange(from, to) > 0 )
		{
        painter->save();
        painter->setPen(bar_pen);
        painter->setBrush(bar_brush);
			
		int dx,dy,ref,bar_width;
			
		if (bar_style == Vertical)
			ref= yMap.transform(1e-100); //smalest positive value for log scales
		else
			ref= xMap.transform(1e-100);	
				
		int i;
		if (bar_style == Vertical)
			{
			dx = abs(xMap.transform(x(from+1))-xMap.transform(x(from)));
			for (i=from+2; i<to; i++)
				{
				int min = abs(xMap.transform(x(i+1))-xMap.transform(x(i)));
				if (min <= dx)
					dx=min;
				}
			bar_width=int(dx*(1-bar_gap*0.01));
			}
		else
			{
			dy = abs(yMap.transform(y(from+1))-yMap.transform(y(from)));
			for (i=from+2; i<to; i++)
				{
				int min = abs(yMap.transform(y(i+1))-yMap.transform(y(i)));
				if (min <= dy)
					dy=min;
				}
			bar_width=int(dy*(1-bar_gap*0.01));
			}
			
        for (i=from; i<=to; i++)
			{
            const int px = xMap.transform(x(i));
            const int py = yMap.transform(y(i));
			
			if (bar_style == Vertical)
				{
				if (y(i) < 0)
					painter->drawRect(px-int(0.5*bar_width), ref, bar_width+1, (py-ref));
				else
					painter->drawRect(px-int(0.5*bar_width), py, bar_width+1, (ref-py+1));	
				}
			else
				{
				if (x(i) < 0)
					painter->drawRect(px, py-int(0.5*bar_width), (ref-px), bar_width+1);
				else
					painter->drawRect(ref, py-int(0.5*bar_width),(px-ref), bar_width+1);
				}	
			}
		painter->restore();
		}
}

QwtDoubleRect QwtBarCurve::boundingRect() const
{
QwtDoubleRect rect = QwtCurve::boundingRect();
double n= (double)dataSize();

if (bar_style == Vertical)
	{	
	double dx=(rect.x2()-rect.x1())/n;
	rect.setX1(rect.x1()-dx);
	rect.setX2(rect.x2()+dx);
	}
else
	{	
	double dy=(rect.y2()-rect.y1())/n;
	rect.setY1(rect.y1()-dy);
	rect.setY2(rect.y2()+dy);
	}
	
return rect;
}

void QwtBarCurve::setBrush (const QBrush &brush)   
{
if (bar_brush == brush)
	return;

QwtPlotCurve::setBrush(brush);
bar_brush=brush;
}

void QwtBarCurve::setPen (const QPen &pen)   
{
if (bar_pen == pen)
	return;

QwtPlotCurve::setPen(pen);
bar_pen =pen;
}

void QwtBarCurve::setGap (int gap)   
{
if (bar_gap == gap)
	return;

bar_gap =gap;
}
