#include "Histogram.h"

#include <qwt_plot.h>
#include <qwt_curve.h>
#include <qwt_painter.h>
#include <qwt_double_rect.h>
#include <qpainter.h>

#include <stdlib.h>

QwtHistogram::QwtHistogram(QwtPlot *parent, const char *name):
    QwtPlotCurve(parent,name)
{
bar_pen=QPen(Qt::black,1,Qt::SolidLine);
bar_brush=QBrush(Qt::red);
QwtPlotCurve::setBrush(bar_brush);

bar_gap=0;
}

void QwtHistogram::draw(QPainter *painter,
    const QwtDiMap &xMap, const QwtDiMap &yMap, int from, int to)
{
   if ( !painter || dataSize() <= 0 )
        return;

    if (to < 0)
        to = dataSize() - 1;

    if ( verifyRange(from, to) > 0 )
		{
        painter->save();
        painter->setPen(bar_pen);
        painter->setBrush(bar_brush);

		int i,dx;
		const int ref= yMap.transform(baseline());

        for (i=from; i<=to; i++)
			{
            const int px1 = xMap.transform(x(i));
            const int py1 = yMap.transform(y(i));
            const int px2 = xMap.transform(x(i+1));

			if (i != to)
				dx=abs(px2 - px1);
			
			const int bar_width=int(dx*(1-bar_gap*0.01));

			painter->drawRect(px1+int(0.5*(dx-bar_width)), py1, bar_width+1, (ref-py1+1));
			}

		painter->restore();
		}
}

QwtDoubleRect QwtHistogram::boundingRect() const
{
QwtDoubleRect rect = QwtCurve::boundingRect();

int n=dataSize();

rect.setX1(rect.x1()-x(1));
rect.setX2(rect.x2()+x(n-1));
rect.setY1(0);

rect.setY2(1.2*rect.y2());
return rect;
}

void QwtHistogram::setBrush (const QBrush &brush)   
{
if (bar_brush == brush)
	return;

QwtPlotCurve::setBrush(brush);
bar_brush=brush;
}

void QwtHistogram::setPen (const QPen &pen)   
{
if (bar_pen == pen)
	return;

QwtPlotCurve::setPen(pen);
bar_pen =pen;
}

void QwtHistogram::setGap (int gap)   
{
if (bar_gap == gap)
	return;

bar_gap =gap;
}
