#include "VectXYXYCurve.h"

#include <qwt_plot.h>
#include <qwt_curve.h>
#include <qwt_painter.h>
#include <qwt_double_rect.h>
#include <qpainter.h>
#include <qmessagebox.h>

VectXYXYCurve::VectXYXYCurve(QwtPlot *parent, const char *name):
    QwtPlotCurve(parent,name)
{
pen=QPen(Qt::black, 1, Qt::SolidLine);
filledArrow=true;
d_headLength=4;
d_headAngle=45;
}

void VectXYXYCurve::draw(QPainter *painter,
    const QwtDiMap &xMap, const QwtDiMap &yMap, int from, int to)
{
    if ( !painter || dataSize() <= 0 )
        return;
	
    if (to < 0)
        to = dataSize() - 1;
	
	QwtPlotCurve::draw(painter, xMap, yMap, from, to);

    if ( verifyRange(from, to) > 0 )
    {
        painter->save();
        painter->setPen(pen);
		drawVector(painter, xMap, yMap, from, to);
        painter->restore();
    }
}

void VectXYXYCurve::drawVector(QPainter *painter,
    const QwtDiMap &xMap, const QwtDiMap &yMap, int from, int to)
{   
 for (int i = from; i <= to; i++)
		{
		const int xs = xMap.transform(x(i));
		const int ys = yMap.transform(y(i));			
		const int xe = xMap.transform(vectorEnd->x(i));
		const int ye = yMap.transform(vectorEnd->y(i));
			
		QwtPainter::drawLine(painter, xs, ys, xe, ye);
		drawArrowHead(painter, xs, ys, xe, ye);
		}
}

void VectXYXYCurve::drawArrowHead(QPainter *p, int xs, int ys, int xe, int ye)
{
p->save();
p->translate(xe, ye);
double t=teta(xs, ys, xe, ye);
p->rotate(-t);
	
double pi=4*atan(-1);	
int d=qRound(d_headLength*tan(pi*(double)d_headAngle/180.0));	
	
const QPointArray endArray(3);	
endArray[0] = QPoint(0, 0);
endArray[1] = QPoint(-d_headLength, d);
endArray[2] = QPoint(-d_headLength, -d);

if (filledArrow)
	p->setBrush(QBrush(pen.color(), QBrush::SolidPattern));

QwtPainter::drawPolygon(p,endArray);
p->restore();
}

double VectXYXYCurve::teta(int x0, int y0, int x1, int y1)
{		
double t,pi=4*atan(-1);
if (x1==x0)
	{
	if (y0>y1)
		t=90;
	else
		t=270;
	}
else
	{
	t=atan2((y1-y0)*1.0,(x1-x0)*1.0)*180/pi;
	if (t<0)
		t=360+t;
	}
return t;
}

void VectXYXYCurve::setVectorEnd(const QwtArray<double>&x, const QwtArray<double>&y)
{
vectorEnd=new QwtArrayData(x, y);
}

int VectXYXYCurve::width()
{
return pen.width ();
}

void VectXYXYCurve::setWidth(int w)
{
pen.setWidth (w);
}

QColor VectXYXYCurve::color()
{
return pen.color();
}

void VectXYXYCurve::setColor(const QColor& c)
{
if (pen.color() != c)
	pen.setColor(c);
}

void VectXYXYCurve::setHeadLength(int l)
{
if (d_headLength != l)
	d_headLength = l;
}
	
void VectXYXYCurve::setHeadAngle(int a)
{
if (d_headAngle != a)
	d_headAngle = a;
}
		
void VectXYXYCurve::fillArrowHead(bool fill)
{
if (filledArrow != fill)
	filledArrow = fill;	
}

QwtDoubleRect VectXYXYCurve::boundingRect() const
{
QwtDoubleRect rect = QwtCurve::boundingRect();
QwtDoubleRect vrect = vectorEnd->boundingRect();
	
rect.setY1(QMIN(rect.y1(), vrect.y1()));
rect.setY2(QMAX(rect.y2(), vrect.y2()));
rect.setX1(QMIN(rect.x1(), vrect.x1()));
rect.setX2(QMAX(rect.x2(), vrect.x2()));
	
return rect;
}

VectXYXYCurve::~VectXYXYCurve()
{
delete vectorEnd;
}
