#ifndef APPLICATION_H
#define APPLICATION_H

#include <qmainwindow.h>
#include <qlist.h>
#include <qlistview.h> 
#include <qwt_plot.h>

#include "matrix.h"
#include "worksheet.h"
#include "graph3D.h"
#include "plot3DDialog.h"

class QAction;
class QActionGroup;
class QTextEdit;
class QToolBar;
class QToolButton;
class QPopupMenu;
class QWorkspace;
class QPopupMenu;
class QListView;
class QToolButton;
class QListBoxItem;
class QTextBrowser;
class QLineEdit;
class Graph;
class ScalePicker;
class Graph3D;
class plot3DDialog;
class MultiLayer;
class fitDialog;

class ApplicationWindow: public QMainWindow
{
    Q_OBJECT
public:
    ApplicationWindow();
	ApplicationWindow(const QString& filename);

	~ApplicationWindow();

	QTextEdit *results;
	QWorkspace* ws;
    QToolBar *fileTools, *plotTools, *tableTools, *plot3DTools, *displayBar;
    QPopupMenu *windowsMenu,*view,*graph,*file,*format,*calcul,*edit,*dataMenu,*recent, *exportPlot;
	QPopupMenu *help,*type,*import,*plot2D,*plot3D, *specialPlot, *panels,*stat, *fft,*decay;
	QPopupMenu *matrixMenu, *plot3DMenu, *plotDataMenu, *tableMenu;
	QListView *lv;
	QToolButton *btnResults;
	QWidgetList *hiddenWindows;
	QTextBrowser *browser;
	QLineEdit *info;
	QWidget *lastModified;

	QActionGroup* dataTools;
	QAction *btnCursor, *btnSelect, *btnPicker, *btnRemovePoints, *btnMovePoints;
	QAction  *btnZoom, *btnPointer, *btnLine;
	
	QActionGroup* coord;
	QAction* Box;
    QAction* Frame;
    QAction* None;

	QActionGroup* grids;
    QAction* front;
    QAction* back;
    QAction* right;
    QAction* left;
    QAction* ceil;
    QAction* floor;

	QActionGroup* floorstyle;
    QAction* floordata;
    QAction* flooriso;
    QAction* floornone;

	QActionGroup* plotstyle;
    QAction* wireframe;
    QAction* hiddenline;
    QAction* polygon;
    QAction* filledmesh;
    QAction* pointstyle;
	QAction* barstyle;
	QAction *conestyle, *crossHairStyle;
	Graph *activeGraph,  *lastCopiedLayer;
	fitDialog *fd;

public slots:
	void open();
	ApplicationWindow* open(const QString& fn);
	ApplicationWindow* openQtiFile_03(const QString& fn);
	ApplicationWindow* openProject(const QString& fn);
	ApplicationWindow* importOPJ(const QString& filename);
	ApplicationWindow * plotFile(const QString& fn);
	void updatePlotsTransparency();

	void setSaveSettings(bool autoSaving, int min);
	void initAppStyle(const QString& s);
	void changeAppStyle(const QString& s);
	void changeAppFont(const QFont& f);
	void updateAppFonts();
	void setAppColors(const QColor& wc,const QColor& pc,const QColor& tpc);

	MultiLayer* plotMultiLayers(int c, int r, int style);
	MultiLayer* multilayerPlot(Table* w,const QStringList& colList, int style);
	MultiLayer* multilayerPlot(const QString& caption);
	MultiLayer* multilayerPlot(const QStringList& colList); //for plot wizard
	void connectMultilayerPlot(MultiLayer *g);

	void plot2VerticalLayers();
	void plot2HorizontalLayers();
	void plot4Layers();
	void plotStackedLayers();
	void plotStackedHistograms();

	//3D data plots
	Graph3D* openMatrixPlot3D(const QString& caption, const QString& matrix_name,
							 					int xl, int xr, int yl, int yr, double zl, double zr);
	Graph3D* dataPlot3D(Table* table,const QString& colName);
	Graph3D* dataPlotXYZ(Table* table,const QString& zColName, int type);
		//when reading from .qti file
	Graph3D* dataPlot3D(const QString& caption,const QString& formula,
											 double xl, double xr, double yl, double yr, double zl, double zr);
	Graph3D* dataPlotXYZ(const QString& caption,const QString& formula,
 												double xl, double xr, double yl, double yr, double zl, double zr);
	/*!
	* used when plotting from the wizard
	*/
	Graph3D* dataPlotXYZ(const QString& formula);
	Graph3D* dataPlot3D(const QString& formula);

	//surface plots
	Graph3D* newPlot3D(const QString& formula, double xl, double xr,
					   double yl, double yr, double zl, double zr);

	Graph3D* newPlot3D(const QString& caption,const QString& formula, 
					   double xl, double xr,double yl, double yr, 
					   double zl, double zr);
	Graph3D* copySurfacePlot();
	Graph3D* surfacePlot(const QString& name);
	void connectSurfacePlot(Graph3D *plot);
	void newSurfacePlot();
	void addSurfacePlot();
	void remove3DPlot();
	void remove3DMatrixPlots(Matrix *m);
	void update3DMatrixPlots(QWidget *w);
	void add3DData();
	void insertNew3DData(const QString& colName);
	void add3DMatrixPlot();
	void insert3DMatrixPlot(const QString& matrix_name);
	void initPlot3D(Graph3D *plot);
	void customPlot3D(Graph3D *plot);
	void setPlot3DOptions(const QStringList& lst, bool legend, 
						  bool projection, int resolution, const QFont& titleFnt,
						  const QFont& numbersFnt, const QFont& axesFnt);

	//multilayer plots
	MultiLayer* plot(const QString& name);
	MultiLayer* copyGraph();
	void cloneGraph(Graph* g, Graph* g2);

	void addLayer();
	void deleteLayer();

	//user-defined functions
	void newFunctionPlot();
	void newFunctionPlot(QString& type,QStringList &formulas,QStringList &vars,QValueList<double> &ranges,QValueList<int> &points);

	void addFunctionCurve();
	void addFunctionCurve(QString&,QStringList &,QStringList &,QValueList<double> &,QValueList<int> &);
	void clearFunctionsList();
	void clearFitFunctionsList();
	void saveFitFunctionsList(const QStringList& l);
	void clearSurfaceFunctionsList();
	void clearLogInfo();
	void clearParamFunctionsList();
	void clearPolarFunctionsList();

	Matrix* cloneMatrix();
	Matrix* newMatrix();
	Matrix* newMatrix(const QString& caption, int r, int c);
	Matrix* matrix(const QString& name);
	Matrix* createIntensityMatrix(const QPixmap& pic);
	Matrix* convertTableToMatrix();
	void initMatrix(Matrix* m, const QString& caption);
	void transposeMatrix();
	void invertMatrix();
	void matrixDeterminant();

	Table* newTable();
	Table* newTable(const QString& fname,const QString &sep, int lines, bool renameCols);
	Table* newTable(const QString& caption,int r, int c);
	Table* newTable(const QString& caption, int r, int c, const QString& text);
	Table* table(const QString& name);
	Table* copyTable();
	Table* convertMatrixToTable();
	QWidgetList* tableList();

	void connectTable(Table* w);
	void newWrksheetPlot(const QString& caption,int r, int c, const QString& text);
	void showHistogramTable(const QString& caption, int r, int c, const QString& text);
	void initTable(Table* w, const QString& caption);
	void customTable(Table* w);
	void customizeTables(const QColor& bgColor,const QColor& textColor,
										const QColor& headerColor,const QFont& textFont, const QFont& headerFont);
	
	void customGraph(Graph* g);
	void setGraphDefaultSettings(int ticks, int legendFrame, bool title, bool allAxes, bool canvasFrame,
								 int frameWidth, bool backbones, int width, int majLength, int minLength, int margin,
								 const QFont& axesFnt,const QFont& numFnt, 
								 const QFont& legendFnt, const QFont& titleFnt);
	
	void setCurvesDefaultSettings(int style, int width, int size);

	void plot3DWireframe();
	void plot3DHiddenLine();
	void plot3DPolygons();
	void plot3DWireSurface();

	void plot3DMatrix(int style);

	void plot3DRibbon();
	void plot3DScatter();
	void plot3DTrajectory();
	void plot3DBars();

	void plotL();
	void plotP();
	void plotLP();
	void plotPie();
	void plotVerticalBars();
	void plotHorizontalBars();
	void plotArea();
	void plotSteps();
	void plotSpline();
	void plotVerticalDropLines();
	void plotHistogram();
	void plotVectXYXY();
	
	//image analysis
	void intensityTable();
	void pixelLineProfile();
	void loadImage();
	void importImage();
	
    void load();
	void loadMultiple();
	void exportAllTables(const QString& sep, bool colNames, bool expSelection);
	void exportASCII(const QString& tableName, const QString& sep, bool colNames, bool expSelection);

    void exportGraph();
	void exportAllGraphs();
	void exportAllGraphs(const QString& dir, const QString& format, 
									int quality, bool transparency);
	void export2DPlotToFile(MultiLayer *plot, const QString& fileName, const QString& format, 
										int quality, bool transparency);
	void export3DPlotToFile(Graph3D *plot, const QString& fileName, 
										const QString& format);

	void saveProjectAs();
	void saveProject();
	void readSettings();
	void saveSettings();
	void modifiedProject();
	void modifiedProject(QWidget *w);
    void print();
	void printAllPlots();
	void printHelp();
	void setImportOptions(const QString& sep, int lines, bool rename);
    
	void showList();
	QStringList columnsList();
	QStringList columnsListXY();
	
	void undo();
	void redo();
	
	myWidget* copyWindow();
	QWidget* window(QListViewItem *it);
	void rename();
	void renameWindow();
	void maximizeWindow(QListViewItem * lbi);
	bool minimized(QWidget* window);
	void checkMinimizedWindows();
	void minimizeWindow();
	void maximizeWindow();
	bool maximized(QWidget* window);
	bool hidden(QWidget* window);
	void resizedWindow(QWidget* window);
	void closeActiveWindow();
	void closeWindow(QWidget* window);
	void hideWindow(QWidget* window);
	void hideWindow();
	void hideActiveWindow();
	void activateWindow();
	void printWindow();
	void updateTable(const QString& caption,int row,const QString& text);
    void about();
    void windowsMenuAboutToShow();
    void windowsMenuActivated( int id );
	void removeCurves(const QString& name);
	QStringList dependingPlots(const QString& caption);
	QStringList depending3DPlots(Matrix *m);
	QStringList multilayerDependencies(QWidget *w);

	QString windowGeometryInfo(QWidget *w);
	void restoreWindowGeometry(ApplicationWindow *app, QWidget *w, const QString s);

	void resizeActiveWindow();
	void resizeWindow();
	
	// list view in project explorer
	void updateListView(const QString& caption);
	void setListView(const QString& caption,const QString& view);
	void renameListViewItem(const QString& oldName,const QString& newName);
	void setListViewDate(const QString& caption,const QString& date);
	QString listViewDate(const QString& caption);
	void setListViewSize(const QString& caption,const QString& size);
	void setListViewLabel(const QString& caption,const QString& label);
	
	void updateColNames(const QString& oldName, const QString& newName);
	void updateTableNames(const QString& oldName, const QString& newName);
	void updateCurves(const QString& name);
	
	void showWorksheet(const QString& curve);
	void addColToTable();
	void cutSelection();
	void copySelection();
	void copyMarker();
	void pasteSelection();
	void clearSelection();
	void clearCellFromTable(const QString& name,double value);
	void copyActiveLayer();
	
	void newProject();

	Matrix* openMatrix(ApplicationWindow* app, const QStringList &flist);
	Table* openTable(ApplicationWindow* app, const QStringList &flist);
	void openSurfacePlot(ApplicationWindow* app, const QStringList &list);
	void openGraph(ApplicationWindow* app, MultiLayer *plot, const QStringList &list);

	void openRecentProject(int index);
	void init();
	void initActions();
	void initMainMenu();
	void initTableMenu();
	void initTableDataMenu();
	void initPlotDataMenu();
	void initToolBars();
	void initPlot3DToolBar();
	void disableMenus();
	void hideMenus();
	void hideToolbars();
	void customToolBars(QWidget* w);
	void customMenu(QWidget* w);
	void windowActivated(QWidget *w);

	// plot tools 
	void newLegend();
	void drawLine();
	void addText();
	void disableAddText();
	void addImage();
	void zoom();
	void unzoom();
	void showRangeSelectors();	
	void showCursor();
	void showScreenReader();
	void pickPointerCursor();
	void disableTools();
	void pickDataTool( QAction* action );

	//interpolation
	void interlinear();
	void akima();
	void intercubic();
	
	//fitting
	void deleteFitTables();
	void fitLinear();
	void fitExpDecay(const QString&, double, double, double);
	void fitExpDecay2(const QString&,double,double,double,double);
	void fitExpDecay3(const QString& name, double firstTime, double secondTime,
						double thirdTime, double start, double yOffset);
	void fitExpGrowth(const QString&, double, double, double);
	void fitGauss();
	void fitLorentz();
	void fitPolynomial(const QString&,int,int, double, double, bool);
	void fitCurve(const QString& curve,const QString& formula,
				 const QStringList& params,const QStringList& paramsInit,
				 double from,double to,int points, int solver, double tolerance);

	void fitBuiltInFunction(const QString& curve, const QString& function, const QStringList& initialValues,
							double from,double to,int iterations, int solver, double tolerance);
				 
	//calculus
	void integrate(int curve, int order, int iter, double tol, double low, double up);
	void differentiate();
	void fft_for();
	void fft_inv();
	void analysis(const QString& whichFit);
	void analyzeCurve(const QString& curve);
	void analyzeCurve(const QString& whichFit, long curveKey, int points);
	void showAnalysisDialog();

	void addErrorBars();
	void defineErrorBars(const QString& name,int type,const QString& percent,int direction);
	void defineErrorBars(const QString& curveName,const QString& errColumnName, int direction);
	void movePoints();
	void removePoints();

	// event handlers 
	void closeEvent( QCloseEvent*);
	void showEvent ( QShowEvent *);
	void hideEvent ( QHideEvent *);
	void timerEvent ( QTimerEvent *e);

	//dialogs
	void showPlotDialog();
	void showAxisDialog();
	genDialog* showAxisDialog(int axisPos);
	void showGeneralPlotDialog();
	void showResults();
	void showTextDialog();
	void showLineDialog();
	void showTitleDialog();
	void showExportASCIIDialog();
	void showCurvesDialog();
	void showPlotAssociations(int curve);

	void showXAxisTitleDialog();
	void showYAxisTitleDialog();
	void showRightAxisTitleDialog();
	void showTopAxisTitleDialog();
	void showColumnOptionsDialog();
	void showRowsDialog();
	void showColsDialog();
	void showRowMenu(int r);
	void showColMenu(int c);
	void showColumnValuesDialog();	
	void showGraphContextMenu();
	void showWindowContextMenu();
	void showWindowPopupMenu(QListViewItem *it, const QPoint &p, int);
	void showMoreWindows();
	void showImportDialog();
	void showMarkerPopupMenu();
	void showHelp();
	void chooseHelpFolder();
	void showPieDialog();
	void showPlotWizard();
	void showFitPolynomDialog();
	void showIntDialog();
	void showInterpolationDialog();
	void showExpGrowthDialog();
	void showExpDecayDialog();
	void showTwoExpDecayDialog();	
	void showExpDecay3Dialog();
	void showRowStatistics();
	void showColStatistics();
	void showFitDialog();
	plot3DDialog* showPlot3dDialog();
	void showImageDialog();
	void showPlotGeometryDialog();
	void showLayerDialog();
	void showConfigureDialog();
	void showMatrixDialog();
	void showMatrixSizeDialog();
	void showMatrixValuesDialog();

	void updateConfirmOptions(bool askTables, bool askMatrixes, bool askPlots2D, bool askPlots3D);
	void updateHistogram(const QString& curveName, int curveIndex, 
										bool automatic, double binSize, double begin, double end);
	void showHistogramStats(const QString& curveName, int curveIndex);

	void updateVectorsLayout(int curve, int colorIndex, int width, int arrowLength, int arrowAngle, 
									 bool filled, const QString& xEndColName, const QString& yEndColName);

	void showAxis(int axis, int type, const QString& labelsColName, bool axisOn, int ticksType,
		bool labelsOn, const QColor& c, int format, int prec, int rotation, int baselineDist);
	//plot3D tools
	void update3DToolBar();
	
	void setFramed3DPlot();
	void setBoxed3DPlot();
	void removeAxes3DPlot();
	void removeGrid3DPlot();
	void setHiddenLineGrid3DPlot();
	void setLineGrid3DPlot();
	void setPoints3DPlot();
	void setCrosses3DPlot();
	void setCones3DPlot();
	void setBars3DPlot();
	void setFilledMesh3DPlot();
	void setEmptyFloor3DPlot();
	void setFloorData3DPlot();
	void setFloorIso3DPlot();
	
	void setFloorGrid3DPlot(bool on);
	void setCeilGrid3DPlot(bool on);

	void setRightGrid3DPlot(bool on);
	void setLeftGrid3DPlot(bool on);
	void setFrontGrid3DPlot(bool on);
	void setBackGrid3DPlot(bool on);
	
	void pickPlotStyle( QAction* action );
	void pickCoordSystem( QAction* action);
	void pickFloorStyle( QAction* action);
	void custom3DActions(QWidget *w);
	void custom3DGrids(int grids);
	
signals:
	void windowClosed(const QString&);
	void modified();
	
public:
	int fileVersion;
	int majVersion, minVersion, patchVersion;
	QColor workspaceColor, panelsColor, panelsTextColor;
	QString appStyle;
	bool confirmCloseTable, confirmCloseMatrix, confirmClosePlot2D, confirmClosePlot3D;
	bool canvasFrameOn, titleOn, autoSave, drawBackbones, allAxesOn;
	int ticksStyle, legendFrameStyle, autoSaveTime, axesLineWidth, canvasFrameWidth;
	int majTicksLength, minTicksLength, defaultPlotMargin;
	int defaultCurveStyle, defaultCurveLineWidth, defaultSymbolSize;
	QFont appFont, plot3DTitleFont, plot3DNumbersFont, plot3DAxesFont;
	QFont tableTextFont, tableHeaderFont, plotAxesFont, plotLegendFont, plotNumbersFont, plotTitleFont;
	QColor tableBkgdColor, tableTextColor, tableHeaderColor;
	QString projectname,separator, helpFilePath, analysisType, configFilePath, logInfo;
	int logID,asciiID,closeID;
	int exportID, printAllID;
	int graphs,tables, matrixes, fitNumber, ignoredLines, timerId, savingTimerId;
	bool renameColumns, listViewOn, copiedLayer;
	QStringList plotWindows,tableWindows, recentProjects, plot3DWindows, matrixWindows;
	bool resultsOn,saved, showPlot3DProjection, showPlot3DLegend;
	int plot3DResolution;
	QStringList plot3DColors;
	QStringList functions; //user-defined functions;
	QStringList xFunctions, yFunctions, rFunctions,tetaFunctions; // user functions for parametric and polar plots
	QStringList fitFunctions; //user-defined fit functions;
	QStringList surfaceFunc; //user-defined surface functions;
	QWidget *aw, *wmax; //active window and maximized window when loading a .qti file
	Graph::MarkerType copiedMarkerType;
	
//variables used when user copy/paste markers
	QString auxMrkText;
	QFont auxMrkFont;
	QColor auxMrkColor, auxMrkBkgColor;
	QPoint auxMrkStart,auxMrkEnd;
	Qt::PenStyle auxMrkStyle;
	QString auxMrkFileName;
	int auxMrkBkg,auxMrkWidth;
	bool startArrowOn, endArrowOn, fillArrowHead;
	int arrowHeadLength, arrowHeadAngle;
	int matrixMenuID, plot3DMenuID, analysisMenuID, graphMenuID, windowsMenuID;
	int editMenuID, tableDataMenuID, formatMenuID, plotDataMenuID, tableMenuID;

    QAction *actionNewProject, *actionNewTable, *actionNewFunctionPlot, *actionNewSurfacePlot, *actionNewMatrix;
    QAction *actionOpen, *actionLoadImage, *actionSaveProject, *actionSaveProjectAs, *actionImportImage;
    QAction *actionLoad, *actionLoadMultiple, *actionUndo, *actionRedo;
    QAction *actionCopyWindow;
    QAction *actionCutSelection, *actionCopySelection, *actionPasteSelection, *actionClearSelection;
    QAction *actionShowList, *actionShowResult, *actionAddLayer, *actionShowLayerDialog;

    QAction *actionExportGraph, *actionExportAllGraphs, *actionPrint, *actionPrintAllPlots, *actionShowExportASCIIDialog;
    QAction *actionShowImportDialog;
    QAction *actionCloseAllWindows, *actionClearLogInfo, *actionShowPlotWizard, *actionShowConfigureDialog;
    QAction *actionShowCurvesDialog, *actionAddErrorBars, *actionAddFunctionCurve, *actionUnzoom, *actionNewLegend, *actionAddImage, *actionAddText;
    QAction *actionPlotL, *actionPlotP, *actionPlotLP, *actionPlotVerticalDropLines, *actionPlotSpline, *actionPlotSteps, *actionPlotVerticalBars, *actionPlotHorizontalBars, *actionPlotArea, *actionPlotPie, *actionPlotVectXYXY;
    QAction *actionPlotHistogram, *actionPlotStackedHistograms, *actionPlot2VerticalLayers, *actionPlot2HorizontalLayers, *actionPlot4Layers, *actionPlotStackedLayers;
    QAction *actionPlot3DRibbon, *actionPlot3DBars, *actionPlot3DScatter, *actionPlot3DTrajectory;
    QAction *actionShowColStatistics, *actionShowRowStatistics, *actionShowIntDialog;
    QAction *actionInterpolate, *actionFFTFor, *actionFFTInv;
    QAction *actionDifferentiate, *actionFitLinear, *actionShowFitPolynomDialog;
    QAction *actionShowExpDecayDialog, *actionShowTwoExpDecayDialog, *actionShowExpDecay3Dialog;
    QAction *actionFitExpGrowth, *actionFitGauss, *actionFitLorentz, *actionShowFitDialog;
    QAction *actionShowLayoutDialog, *actionShowAxisDialog, *actionShowTitleDialog;
    QAction *actionShowColumnOptionsDialog, *actionShowColumnValuesDialog, *actionShowColsDialog, *actionShowRowsDialog;
    QAction *actionAbout, *actionShowHelp, *actionChooseHelpFolder;

    QAction *actionRename, *actionClose, *actionConvertTable;

    QAction *actionAddColToTable, *actionDeleteLayer;
    QAction *actionPrintHelp, *actionResizeActiveWindow, *actionHideActiveWindow;
    QAction *actionShowMoreWindows, *actionPixelLineProfile, *actionIntensityTable;

    QAction *actionShowLineDialog, *actionShowImageDialog, *actionShowTextDialog;
    QAction *actionActivateWindow, *actionMinimizeWindow, *actionMaximizeWindow, *actionHideWindow, *actionResizeWindow, *actionPrintWindow;
    QAction *actionShowPlotGeometryDialog;
    QAction *actionRemove3DPlot, *actionAddSurfacePlot, *actionAdd3DData;

	QAction *actionMatrixDeterminant, *actionSetMatrixProperties;
	QAction *actionSetMatrixDimensions, *actionConvertMatrix, *actionSetMatrixValues, *actionTransposeMatrix, *actionInvertMatrix;
	QAction *actionPlot3DWireFrame, *actionPlot3DHiddenLine, *actionPlot3DPolygons, *actionPlot3DWireSurface;
	QAction *actionDeleteFitTables;
};

#endif
