#include "configDialog.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qfont.h>
#include <qfontdialog.h> 
#include <qcolordialog.h>
#include <qwidgetstack.h>
#include <qwidget.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qstylefactory.h>
#include <qlistbox.h>
#include <qvbox.h>

#include "graph.h"
#include "colorButton.h"

static const char * dropLines_xpm[] = {
"16 16 3 1",
" 	c None",
".	c #000000",
"+	c #CC3737",
"                ",
"  .     +++     ",
"  .     +++     ",
" ..     +++     ",
"  .      .      ",
"  . +++  .      ",
" .. +++  .  +++ ",
"  . +++  .  +++ ",
"  .  .   .  +++ ",
" ..  .   .   .  ",
"  .  .   .   .  ",
"  .  .   .   .  ",
" ..  .   .   .  ",
"  .  .   .   .  ",
"  ..............",
"    .  .  .  .  "};

static const char * spline_xpm[] = {
"17 13 2 1",
" 	c None",
".	c #000000",
"                 ",
"           ..    ",
"            ..   ",
"          .   .  ",
"  ..      .   .  ",
"   .     .    .  ",
"  .      ..    . ",
"  .      .    .. ",
"  .     .        ",
"   .    .        ",
"   ..  ..        ",
"     ..          ",
"                 "};

static const char * steps_xpm[] = {
"16 16 2 1",
" 	c None",
".	c #050404",
"                ",
"                ",
"     ....       ",
"     .  .     ..",
" ..  .  .     . ",
"  .  .  . ... . ",
"  .  .  . . . . ",
"  .  .  . . . . ",
"  ....  . . ... ",
"        . .     ",
"        . .     ",
"        . .     ",
"        . .     ",
"        ...     ",
"                ",
"                "};

static const char *pPlot_xpm[]={
"13 13 2 1",
". c None",
"# c #000000",
"..........###",
"..........###",
"..........###",
".............",
".............",
".....###.....",
".....###.....",
".....###.....",
".............",
".............",
"###..........",
"###..........",
"###.........."};


static const char *lpPlot_xpm[]={
"13 13 2 1",
". c None",
"# c #000000",
"..........###",
"..........###",
"..........###",
".........#...",
"........#....",
".....###.....",
".....###.....",
".....###.....",
"....#........",
"...#.........",
"###..........",
"###..........",
"###.........."};

static const char *lPlot_xpm[]={
"14 14 2 1",
". c None",
"# c #000000",
".............#",
"............#.",
"...........#..",
"..........#...",
".........#....",
"........#.....",
".......#......",
"......#.......",
".....#........",
"....#.........",
"...#..........",
"..#...........",
".#............",
"#............."};

static const char * area_xpm[] = {
"16 15 4 1",
" 	c None",
".	c #00FF00",
"+	c #FF0000",
"@	c #000000",
"   .        .   ",
"  ...      ...  ",
" .....    ..... ",
" ......  .......",
"................",
"++.....++++....+",
"+++...++++++..++",
"+++++++++@@+++++",
"+++@@+++@@@@++++",
"++@@@@+@@@@@@+++",
"+@@@@@@@@@@@@+++",
"@@@@@@@@@@@@@@@@",
"@@@@@@@@@@@@@@@@",
"@@@@@@@@@@@@@@@@",
"@@@@@@@@@@@@@@@@"};

static const char * hBars_xpm[] = {
"15 15 3 1",
" 	c None",
".	c #000000",
"+	c #FF0000",
"...........    ",
"++++++++++.    ",
"++++++++++.    ",
"++++++++++.    ",
"...........    ",
"...............",
"++++++++++++++.",
"++++++++++++++.",
"++++++++++++++.",
"...............",
"...........    ",
"++++++++++.    ",
"++++++++++.    ",
"++++++++++.    ",
"...........    "};

static const char * vertBars_xpm[] = {
"15 15 3 1",
" 	c None",
".	c #000000",
"+	c #FF0000",
"     .....     ",
"     .+++.     ",
"     .+++.     ",
"     .+++.     ",
"......+++......",
".+++..+++..+++.",
".+++..+++..+++.",
".+++..+++..+++.",
".+++..+++..+++.",
".+++..+++..+++.",
".+++..+++..+++.",
".+++..+++..+++.",
".+++..+++..+++.",
".+++..+++..+++.",
".+++..+++..+++."};

configDialog::configDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "configDialog" );
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, sizePolicy().hasHeightForWidth() ) );
    setMouseTracking( TRUE );
    setSizeGripEnabled( FALSE );

	QHBox *box = new QHBox (this, "box"); 
	box->setSpacing (5);
	box->setMargin(5);

	itemsList = new QListBox (box, "itemsList");
	itemsList->insertItem(  tr( "Application" ) );
    itemsList->insertItem( tr( "Confirmations" ) );
    itemsList->insertItem(  tr( "Tables" ) );
	itemsList->insertItem(  tr( "3D Plots" ) );
    itemsList->insertItem(  tr( "2D Plots" ) );
	itemsList->insertItem(  tr( "2D Curves" ) );

	generalDialog = new QWidgetStack(box, "generalDialog" );
	
	initPlots3DPage();
	initPlotsPage();
	initAppPage();
	initCurvesPage();
	initTablesPage();

	confirm = new QWidget( generalDialog, "confirm" );
	
	QButtonGroup *GroupBox6 = new QButtonGroup( 1,QGroupBox::Horizontal,tr("Prompt on closing"),confirm,"GroupBox6" );
    boxTables= new QCheckBox(tr("Tables"),GroupBox6, "boxTables" );
	boxMatrixes= new QCheckBox(tr("Matrixes"),GroupBox6, "boxMatrixes" );
    boxPlots2D= new QCheckBox(tr("2D Plots"),GroupBox6, "boxPlots2D" );
	boxPlots3D= new QCheckBox(tr("3D Plots"),GroupBox6, "boxPlots3D" );
	
	QHBoxLayout* hlayout3 = new QHBoxLayout(confirm,0,5, "hlayout3");
    hlayout3->addWidget(GroupBox6);
	
	generalDialog->addWidget(application, 0);
	generalDialog->addWidget(confirm, 1);
	generalDialog->addWidget(tables, 2);
	generalDialog->addWidget(plots3D, 3);
	generalDialog->addWidget(plots, 4);
	generalDialog->addWidget(curves, 5);

	GroupBox2 = new QButtonGroup( 3,QGroupBox::Horizontal,tr(""),this,"GroupBox2" );
	GroupBox2->setFlat(TRUE);
	GroupBox2->setLineWidth(0);
	
	buttonApply = new QPushButton(GroupBox2, "buttonApply" );
	
	buttonOk = new QPushButton(GroupBox2, "buttonOk" );
    buttonOk->setDefault( TRUE );
   
    buttonCancel = new QPushButton(GroupBox2, "buttonCancel" );
	
	QVBoxLayout* hlayout = new QVBoxLayout(this,5,5, "hlayout");
    hlayout->addWidget(box);
	hlayout->addWidget(GroupBox2);

    languageChange();
   
    // signals and slots connections
	connect( itemsList, SIGNAL(highlighted(int)), 
			generalDialog, SLOT(raiseWidget(int)));

    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( buttonApply, SIGNAL( clicked() ), this, SLOT( update() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( buttonBackground, SIGNAL( clicked() ), this, SLOT( pickBgColor() ) );
	connect( buttonText, SIGNAL( clicked() ), this, SLOT( pickTextColor() ) );
	connect( buttonHeader, SIGNAL( clicked() ), this, SLOT( pickHeaderColor() ) );
	connect( buttonTextFont, SIGNAL( clicked() ), this, SLOT( pickTextFont() ) );
	connect( buttonHeaderFont, SIGNAL( clicked() ), this, SLOT( pickHeaderFont() ) );
	}

void configDialog::initTablesPage()
{
tables = new QWidget( generalDialog, "tables" );
	
GroupBox1 = new QButtonGroup(2,QGroupBox::Horizontal,tr("Colors"),tables,"GroupBox1" );

new QLabel( tr( "Background" ), GroupBox1, "TextLabel69111",0 ); 
buttonBackground= new ColorButton(GroupBox1);

new QLabel( tr( "Text" ), GroupBox1, "TextLabel69111",0 ); 
buttonText= new ColorButton(GroupBox1);

new QLabel( tr( "Labels" ), GroupBox1, "TextLabel69111",0 ); 
buttonHeader= new ColorButton(GroupBox1);
	
GroupBox3 = new QButtonGroup(2,QGroupBox::Horizontal,tr("Fonts"),tables,"GroupBox3" );

buttonTextFont= new QPushButton(GroupBox3, "buttonTextFont" );
buttonHeaderFont= new QPushButton(GroupBox3, "buttonHeaderFont" );
	
QVBoxLayout* hlayout1 = new QVBoxLayout(tables, 20, 45, "hlayout1");
hlayout1->addWidget(GroupBox1);
hlayout1->addWidget(GroupBox3);
}

void configDialog::initPlotsPage()
{
	plots = new QWidget( generalDialog, "plots" );
	
	QHBox  *hbox1=new QHBox (plots, "hbox1"); 
	hbox1->setSpacing(5);

	QButtonGroup *GroupBox4 = new QButtonGroup( 2,QGroupBox::Horizontal,tr("Options"),hbox1,"GroupBox4" );
	
    boxTitle= new QCheckBox("Show title",GroupBox4,"title");
	boxAllAxes= new QCheckBox("Show all axes", GroupBox4, "boxAllAxes");

	boxFrame= new QCheckBox("Canvas frame",GroupBox4,"frame");
	boxBackbones= new QCheckBox("Axes backbones",GroupBox4,"boxBackbones");

	labelFrameWidth = new QLabel( tr( "Frame width" ), GroupBox4, "TextLabel69111",0 ); 
	boxFrameWidth= new QSpinBox(1, 100, 1,GroupBox4,"boxFrameWidth");	

	new QLabel( tr( "Legend frame" ), GroupBox4, "TextLabel689",0 ); 
	boxLegend = new QComboBox(GroupBox4,"legend");
	boxLegend->insertItem( tr( "None" ) );
	boxLegend->insertItem( tr( "Rectangle" ) );
    boxLegend->insertItem( tr( "Shadow" ) );
    boxLegend->insertItem( tr( "White out" ) );
    boxLegend->insertItem( tr( "Black out" ) );

	new QLabel( tr( "Margin" ), GroupBox4, "TextLabel691111",0 ); 
	boxMargin= new QSpinBox(0, 1000, 5, GroupBox4, "boxMargin");	

	QButtonGroup *GroupBox5 = new QButtonGroup( 1,QGroupBox::Horizontal,tr("Fonts"),hbox1,"GroupBox5" );
	buttonTitleFont= new QPushButton(GroupBox5, "buttonTitleFont" );  
	buttonLegendFont= new QPushButton(GroupBox5, "buttonLegendFont" );	 
    buttonAxesFont= new QPushButton(GroupBox5, "buttonAxesFont" );   
	buttonNumbersFont= new QPushButton(GroupBox5, "buttonNumbersFont" );

	QButtonGroup *GroupBox6 = new QButtonGroup(4, QGroupBox::Horizontal,tr(""),plots,"GroupBox5" );

	new QLabel( tr( "Ticks" ), GroupBox6, "TextLabel489",0 ); 
	boxTicks = new QComboBox(GroupBox6,"ticks");
	boxTicks->insertItem(tr("In"));
	boxTicks->insertItem(tr("Out"));
	boxTicks->insertItem(tr("Both"));
	boxTicks->insertItem(tr("None"));

	new QLabel( tr( "Major ticks length" ), GroupBox6, "TextLabel691",0 );  
	boxMajorTicks= new QSpinBox(0, 100, 1,GroupBox6,"boxMajorTicks");
	
	new QLabel( tr( "Axes linewidth" ), GroupBox6, "TextLabel69",0 );  
	boxLinewidth= new QSpinBox(0, 100, 1,GroupBox6,"boxLinewidth");

	new QLabel( tr( "Minor ticks length" ), GroupBox6, "TextLabel6911",0 );  
	boxMinorTicks= new QSpinBox(0, 100, 1,GroupBox6,"boxMinorTicks");

	QVBoxLayout* hlayout2 = new QVBoxLayout(plots,0,5, "hlayout1");
    hlayout2->addWidget(hbox1);
	hlayout2->addWidget(GroupBox6);

	connect( boxFrame, SIGNAL( toggled(bool) ), this, SLOT( showFrameWidth(bool) ) );

	connect( buttonAxesFont, SIGNAL( clicked() ), this, SLOT( pickAxesFont() ) );
	connect( buttonNumbersFont, SIGNAL( clicked() ), this, SLOT( pickNumbersFont() ) );
	connect( buttonLegendFont, SIGNAL( clicked() ), this, SLOT( pickLegendFont() ) );
	connect( buttonTitleFont, SIGNAL( clicked() ), this, SLOT( pickTitleFont() ) );
}

void configDialog::showFrameWidth(bool ok)
{
if (!ok)
	{
	boxFrameWidth->hide();
	labelFrameWidth->hide();
	}
else
	{
	boxFrameWidth->show();
	labelFrameWidth->show();
	}	
}

void configDialog::initPlots3DPage()
{
	plots3D = new QWidget( generalDialog, "plots3D" );

	QVBox  *box=new QVBox (plots3D, "box"); 
	box->setSpacing (5);

	QButtonGroup *GroupBox77 = new QButtonGroup(2,QGroupBox::Horizontal,tr( "" ), box, "GroupBox7" );

	boxShowLegend = new QCheckBox(GroupBox77, "boxShowLegend" );
	boxShowLegend->setText(tr( "&Show Legend" ));

	boxShowProjection = new QCheckBox(GroupBox77, "boxShowProjection" );
	boxShowProjection->setText(tr( "Show &Projection" ));

	new QLabel( tr( "Resolution" ), GroupBox77, "TextLabel690",0 ); 
	boxResolution = new QSpinBox(1, 100, 1, GroupBox77, "boxResolution" );
	boxResolution->setSpecialValueText( "1 (all data shown)" );

	QButtonGroup *GroupBox7 = new QButtonGroup(4,QGroupBox::Horizontal,tr( "Colors" ),box, "GroupBox7" );

	btnFromColor = new QPushButton( GroupBox7, "btnFrom" );
    btnFromColor->setText( tr( "&Data Max" ) );

	btnLabels = new QPushButton( GroupBox7, "btnLabels" );
    btnLabels->setText( tr( "Lab&els" ) );

	btnMesh = new QPushButton( GroupBox7, "btnMesh" );
    btnMesh->setText( tr( "Mesh &Line" ) );
	
	btnGrid = new QPushButton( GroupBox7, "btnGrid" );
    btnGrid->setText( tr( "&Grid" ) );

	btnToColor = new QPushButton( GroupBox7, "btnTo" );
    btnToColor->setText( tr( "Data &Min" ) );

	btnNumbers = new QPushButton( GroupBox7, "btnNumbers" );
    btnNumbers->setText( tr( "&Numbers" ) );

	btnAxes = new QPushButton( GroupBox7, "btnAxes" );
    btnAxes->setText( tr( "A&xes" ) );

	btnBackground3D = new QPushButton( GroupBox7, "btnBackground" );
    btnBackground3D->setText( tr( "&Background" ) );

	QButtonGroup *GroupBox3 = new QButtonGroup(3,QGroupBox::Horizontal,tr( "Fonts" ),box, "GroupBox7" );

	btnTitleFnt = new QPushButton( GroupBox3, "btnTitleFnt" );
    btnTitleFnt->setText( tr( "&Title" ) );

	btnLabelsFnt = new QPushButton( GroupBox3, "btnLabelsFnt" );
    btnLabelsFnt->setText( tr( "&Axes Labels" ) );

	btnNumFnt = new QPushButton( GroupBox3, "btnNumFnt" );
    btnNumFnt->setText( tr( "&Numbers" ) );
   	
	
	QVBoxLayout* hlayout2 = new QVBoxLayout(plots3D, 5, 5, "hlayout2");
	hlayout2->addWidget(box);
	
	connect( btnAxes, SIGNAL( clicked() ), this, SLOT(pick3DAxesColor() ) );
	connect( btnLabels, SIGNAL( clicked() ), this, SLOT(pick3DLabelsColor() ) );
	connect( btnNumbers, SIGNAL( clicked() ), this, SLOT(pick3DNumbersColor() ) );
	connect( btnBackground3D, SIGNAL( clicked() ), this, SLOT(pick3DBackgroundColor()));
	connect( btnFromColor, SIGNAL( clicked() ), this, SLOT(pickDataMinColor() ) );
	connect( btnToColor, SIGNAL( clicked() ), this, SLOT(pickDataMaxColor() ) );
	connect( btnGrid, SIGNAL( clicked() ), this, SLOT(pickGridColor() ) );
	connect( btnMesh, SIGNAL( clicked() ), this, SLOT(pickMeshColor() ) );

	connect( btnNumFnt, SIGNAL( clicked() ), this, SLOT(pick3DNumbersFont() ) );
	connect( btnTitleFnt, SIGNAL( clicked() ), this, SLOT(pick3DTitleFont() ) );
	connect( btnLabelsFnt, SIGNAL( clicked() ), this, SLOT(pick3DAxesFont() ) );
}

void configDialog::initAppPage()
{
application = new QWidget( generalDialog, "application" );
QButtonGroup *GroupBox7 = new QButtonGroup( 2,QGroupBox::Horizontal,tr("General"),application,"GroupBox7" );
    
	new QLabel( tr( "Style" ), GroupBox7, "TextLabel690",0 ); 
	boxStyle = new QComboBox(GroupBox7,"style");
	QStringList styles = QStyleFactory::keys();
	styles.sort();
	boxStyle->insertStringList(styles);
   
    new QLabel( tr( "Fonts" ), GroupBox7, "TextLabel699",0 ); 
	fontsBtn= new QPushButton(tr("Choose &font"),GroupBox7, "fontsBtn" );
	
	boxSave= new QCheckBox(tr("Save every"),GroupBox7, "boxSave" );
	boxMinutes = new QSpinBox(1,100,1,GroupBox7, "minutes");
	boxMinutes->setSuffix(" minutes");
	
	QButtonGroup *GroupBox8 = new QButtonGroup(2,QGroupBox::Horizontal, tr("Colors"),application,"GroupBox7" );
 	
	new QLabel( tr( "Workspace" ), GroupBox8, "TextLabel69111",0 ); 
	btnWorkspace = new ColorButton(GroupBox8);

	new QLabel( tr( "Panels" ), GroupBox8, "TextLabel69111",0 ); 
	btnPanels = new ColorButton(GroupBox8);

	new QLabel( tr( "Panels text" ), GroupBox8, "TextLabel69111",0 ); 
	btnPanelsText = new ColorButton(GroupBox8);
	
	QVBoxLayout* hlayout4 = new QVBoxLayout(application,0,5, "hlayout4");
    hlayout4->addWidget(GroupBox7);
	hlayout4->addWidget(GroupBox8);

	connect( fontsBtn, SIGNAL( clicked() ), this, SLOT( pickApplicationFont() ) );
	connect( boxSave, SIGNAL( toggled(bool) ), boxMinutes, SLOT( setEnabled(bool) ) );
	connect( btnWorkspace, SIGNAL( clicked() ), this, SLOT( pickWorkspaceColor() ) );
	connect( btnPanels, SIGNAL( clicked() ), this, SLOT( pickPanelsColor() ) );
	connect( btnPanelsText, SIGNAL( clicked() ), this, SLOT( pickPanelsTextColor() ) );
}

void configDialog::initCurvesPage()
{
curves = new QWidget( generalDialog, "curves" );
QButtonGroup *GroupBox77 = new QButtonGroup( 2,QGroupBox::Horizontal,tr(""),curves,"GroupBox77" );
    
new QLabel( tr( "Default curve style" ), GroupBox77, "TextLabel690",0 ); 	
boxCurveStyle = new QComboBox(GroupBox77,"boxCurveStyle");
boxCurveStyle->insertItem( QPixmap(lPlot_xpm), tr( " Line" ) );
boxCurveStyle->insertItem( QPixmap(pPlot_xpm), tr( " Scatter" ) );
boxCurveStyle->insertItem( QPixmap(lpPlot_xpm), tr( " Line + Symbol" ) );
boxCurveStyle->insertItem( QPixmap(dropLines_xpm), tr( " Vertical drop lines" ) );
boxCurveStyle->insertItem( QPixmap(spline_xpm), tr( " Spline" ) );
boxCurveStyle->insertItem( QPixmap(steps_xpm), tr( " Vertical steps" ) );
boxCurveStyle->insertItem( QPixmap(area_xpm), tr( " Area" ) );
boxCurveStyle->insertItem( QPixmap(vertBars_xpm), tr( " Vertical Bars" ) );
boxCurveStyle->insertItem( QPixmap(hBars_xpm), tr( " Horizontal Bars" ) );

new QLabel( tr( "Line width" ), GroupBox77, "TextLabel6969",0 ); 
boxCurveLineWidth = new QSpinBox(1,100,1,GroupBox77, "boxCurveLineWidth");

new QLabel( tr( "Symbol size" ), GroupBox77, "TextLabel69269",0 ); 
boxSymbolSize = new QSpinBox(1,100,1,GroupBox77, "boxSymbolSize");
	
QHBoxLayout* hlayout5 = new QHBoxLayout(curves,0,5, "hlayout5");
hlayout5->addWidget(GroupBox77);
}

configDialog::~configDialog()
{
}

void configDialog::languageChange()
{
    setCaption( tr( "QtiPlot - Choose default settings" ) );
    buttonOk->setText( tr( "&OK" ) );
	buttonCancel->setText( tr( "&Cancel" ) );
	buttonApply->setText( tr( "&Apply" ) );

	buttonText->setText( tr( "Te&xt" ) );
	buttonHeader->setText( tr( "Lab&els" ) );
	buttonBackground->setText( tr( "&Background" ) );

	buttonTextFont->setText( tr( "&Text Font" ) );
	buttonHeaderFont->setText( tr( "&Labels Font" ) );
	buttonAxesFont->setText( tr( "A&xes Legend" ) );
	buttonNumbersFont->setText( tr( "Axes &Numbers" ) );
	buttonLegendFont->setText( tr( "&Legend" ) );
	buttonTitleFont->setText( tr( "T&itle" ) );
	btnWorkspace->setText( tr( "&Workspace" ) );
	btnPanels->setText( tr( "Pa&nels" ) );
	btnPanelsText->setText( tr( "Panels Te&xt" ) );
}

void configDialog::accept()
{
update();
close();
}

void configDialog::update()
{
if (generalDialog->visibleWidget()==(QWidget*)tables)
	emit updateTables(buttonBackground->color(), buttonText->color(), 
					   buttonHeader->color(), textFont, headerFont);
else if (generalDialog->visibleWidget()==(QWidget*)plots)
	{
	emit setGraphDefaultSettings(ticksType(), boxLegend->currentItem(), 
					boxTitle->isChecked(), boxAllAxes->isChecked(), boxFrame->isChecked(), 
					boxFrameWidth->value(), boxBackbones->isChecked(), boxLinewidth->value(),
					boxMajorTicks->value(), boxMinorTicks->value(), boxMargin->value(),
					axesFont, numbersFont, legendFont, titleFont);
	}
else if (generalDialog->visibleWidget()==(QWidget*)application)
	{
	emit changeAppFont(appFont);
	emit changeAppStyle(boxStyle->currentText());
	emit setSaveSettings(boxSave->isChecked(), boxMinutes->value());
	emit setAppColors(btnWorkspace->color(), btnPanels->color(), btnPanelsText->color());
	}
else if (generalDialog->visibleWidget()==(QWidget*)confirm)
	updateConfirmOptions(boxTables->isChecked(), boxMatrixes->isChecked(),
						 boxPlots2D->isChecked(), boxPlots3D->isChecked());
else if (generalDialog->visibleWidget()==(QWidget*)plots3D)
	{
	emit setPlot3DOptions(plot3DColors, boxShowLegend->isChecked(), 
						  boxShowProjection->isChecked(), boxResolution->value(),
						  plot3DTitleFont, plot3DNumbersFont, plot3DAxesFont);
	}
else
	setCurvesSettings(curveStyle(), boxCurveLineWidth->value(), boxSymbolSize->value());
}

int configDialog::curveStyle()
{
int style = 0;
switch (boxCurveStyle->currentItem())
	{
	case 0:
		style = Graph::Line;
	break;
	case 1:
		style = Graph::Scatter;
	break;
	case 2:
		style = Graph::LineSymbols;
	break;
	case 3:
		style = Graph::VerticalDropLines;
	break;
	case 4:
		style = Graph::Spline;
	break;
	case 5:
		style = Graph::Steps;
	break;
	case 6:
		style = Graph::Area;
	break;
	case 7:
		style = Graph::VerticalBars;
	break;
	case 8:
		style = Graph::HorizontalBars;
	break;
	}
return style;
}

void configDialog::pickBgColor()
{
QColor c = QColorDialog::getColor(buttonBackground->color(), this);
if ( !c.isValid() || c == buttonBackground->color())
		return;

buttonBackground->setColor(c);
}

void configDialog::pickTextColor()
{
QColor c = QColorDialog::getColor(buttonText->color(), this);
if ( !c.isValid() || c == buttonText->color())
		return;

buttonText->setColor(c);
}

void configDialog::pickHeaderColor()
{
QColor c = QColorDialog::getColor(buttonHeader->color(), this);
if ( !c.isValid() || c == buttonHeader->color())
		return;

buttonHeader->setColor(c);
}

void configDialog::pickTextFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,textFont,this);
    if ( ok ) {
        textFont = font;
    } else {
     return;
    }
}

void configDialog::pickHeaderFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,headerFont,this);
    if ( ok ) {
        headerFont = font;
    } else {
     return;
    }
}

void configDialog::initColors(const QColor& bgCol, const QColor& textCol, const QColor& headerCol)
{
buttonBackground->setColor(bgCol);
buttonText->setColor(textCol);
buttonHeader->setColor(headerCol);
}

void configDialog::initFonts(const QFont& textFnt, const QFont& headerFnt)
{
textFont=textFnt;
headerFont=headerFnt;	
}

void configDialog::initPlotOptions(int ticksStyle, int legendStyle, bool titleOn, bool allAxes,
								   bool frameOn, int frameWidth, bool backbones, 
								   int axeLineWidth, int minLength, int majLength, int margin)
{
switch (ticksStyle)
	{
	case -1:
		boxTicks->setCurrentItem(0);
	break;

	case 1:
		boxTicks->setCurrentItem(1);
	break;

	case 2:
		boxTicks->setCurrentItem(2);
	break;

	case 0:
		boxTicks->setCurrentItem(3);
	break;	
	}
boxMinorTicks->setValue (minLength);
boxMajorTicks->setValue (majLength);

boxLegend->setCurrentItem(legendStyle);
boxTitle->setChecked(titleOn);	

boxFrame->setChecked(frameOn);
boxFrameWidth->setValue(frameWidth);

if (!frameOn)
	{
	labelFrameWidth->hide();
	boxFrameWidth->hide();
	}

boxAllAxes->setChecked (allAxes);
boxBackbones->setChecked(backbones);
boxLinewidth->setValue(axeLineWidth);	
boxMargin->setValue(margin);
}

int configDialog::ticksType()
{
int ticks=-1;
switch (boxTicks->currentItem())
	{
	case 0:
	break;

	case 1:
		ticks=1;
	break;

	case 2:
		ticks=2;
	break;

	case 3:
		ticks=0;
	break;	
	}
return ticks;
}

void configDialog::pickLegendFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,legendFont,this);
    if ( ok ) {
        legendFont = font;
    } else {
     return;
    }
}

void configDialog::pickAxesFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,axesFont,this);
    if ( ok ) {
        axesFont = font;
    } else {
     return;
    }
}

void configDialog::pickNumbersFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,numbersFont,this);
    if ( ok ) {
        numbersFont = font;
    } else {
     return;
    }
}

void configDialog::pickTitleFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,titleFont,this);
    if ( ok ) 
		titleFont = font;
    else 
     	return;
}

void configDialog::pickApplicationFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,appFont,this);
    if ( ok ) 
		appFont = font;
    else 
     	return;
}

void configDialog::initSaveSettings(bool autoSave, int min)
{
boxSave->setChecked(autoSave);
boxMinutes->setValue(min);
boxMinutes->setEnabled(autoSave);
}

void configDialog::initAppFonts(const QFont& fnt)
{
appFont = fnt;	
}

void configDialog::initAppStyle(const QString& s)
{
boxStyle->setCurrentText(s);
}

void configDialog::initGraphFonts(const QFont& axesFnt, const QFont& numbersFnt, 
																const QFont& legendFnt, const QFont& titleFnt)
{
axesFont=axesFnt;
numbersFont=numbersFnt;
legendFont=legendFnt;	
titleFont=titleFnt;
}

void configDialog::initCurvesOptions(int style, int width, int symbolSize)
{
if (style == Graph::Line)
	boxCurveStyle->setCurrentItem(0);
else if (style == Graph::Scatter)
	boxCurveStyle->setCurrentItem(1);
else if (style == Graph::LineSymbols)
	boxCurveStyle->setCurrentItem(2);
else if (style == Graph::VerticalDropLines)
	boxCurveStyle->setCurrentItem(3);
else if (style == Graph::Spline)
	boxCurveStyle->setCurrentItem(4);
else if (style == Graph::Steps)
	boxCurveStyle->setCurrentItem(5);
else if (style == Graph::Area)
	boxCurveStyle->setCurrentItem(6);
else if (style == Graph::VerticalBars)
	boxCurveStyle->setCurrentItem(7);
else if (style == Graph::HorizontalBars)
	boxCurveStyle->setCurrentItem(8);

boxCurveLineWidth->setValue(width);
boxSymbolSize->setValue(symbolSize);
}

void configDialog::initConfirmOptions(bool askTables, bool askMatrixes,
									  bool askPlots2D, bool askPlots3D)
{
boxTables->setChecked(askTables);
boxMatrixes->setChecked(askMatrixes);
boxPlots2D->setChecked(askPlots2D);
boxPlots3D->setChecked(askPlots3D);
}

void configDialog::pickPanelsTextColor()
{
QColor c = QColorDialog::getColor(btnPanelsText->color(), this);
if ( !c.isValid() || c == btnPanelsText->color())
		return;

btnPanelsText->setColor(c);
}

void configDialog::pickPanelsColor()
{
QColor c = QColorDialog::getColor(btnPanels->color(), this);
if ( !c.isValid() || c == btnPanels->color())
		return;

btnPanels->setColor(c);
}

void configDialog::pickWorkspaceColor()
{
QColor c = QColorDialog::getColor(btnWorkspace->color(), this);
if ( !c.isValid() || c == btnWorkspace->color())
		return;

btnWorkspace->setColor(c);
}

void configDialog::initAppColors(const QColor& wCol, const QColor& pCol, const QColor& tCol)
{
btnWorkspace->setColor(wCol);
btnPanels->setColor(pCol);
btnPanelsText->setColor(tCol);
}

void configDialog::initPlot3DOptions(const QStringList& lst, bool legend, 
								     bool projection, int resolution, 
									 const QFont& titleFnt,const QFont& numbersFnt, 
									 const QFont& axesFnt)
{
plot3DColors = lst;
boxShowLegend->setChecked(legend);
boxShowProjection->setChecked(projection);
boxResolution->setValue(resolution);

plot3DTitleFont = titleFnt;
plot3DNumbersFont = numbersFnt;
plot3DAxesFont = axesFnt;
}

void configDialog::pickDataMaxColor()
{
QColor c = QColorDialog::getColor(QColor(plot3DColors[0]), this );
if ( !c.isValid() )
		return;

plot3DColors[0] = c.name();
}

void configDialog::pickDataMinColor()
{
QColor c = QColorDialog::getColor(QColor(plot3DColors[4]), this );
if ( !c.isValid() )
		return;

plot3DColors[4] = c.name();
}

void configDialog::pick3DBackgroundColor()
{
QColor c = QColorDialog::getColor(QColor(plot3DColors[7]), this );
if ( !c.isValid() )
		return;

plot3DColors[7] = c.name();
}

void configDialog::pickMeshColor()
{
QColor c = QColorDialog::getColor(QColor(plot3DColors[2]), this );
if ( !c.isValid() )
		return;

plot3DColors[2] = c.name();
}

void configDialog::pickGridColor()
{
QColor c = QColorDialog::getColor(QColor(plot3DColors[3]), this );
if ( !c.isValid() )
		return;

plot3DColors[3] = c.name();
}

void configDialog::pick3DAxesColor()
{
QColor c = QColorDialog::getColor(QColor(plot3DColors[6]), this );
if ( !c.isValid() )
		return;

plot3DColors[6] = c.name();
}

void configDialog::pick3DNumbersColor()
{
QColor c = QColorDialog::getColor(QColor(plot3DColors[5]), this );
if ( !c.isValid() )
		return;

plot3DColors[5] = c.name();
}

void configDialog::pick3DLabelsColor()
{
QColor c = QColorDialog::getColor(QColor(plot3DColors[1]), this );
if ( !c.isValid() )
		return;

plot3DColors[1] = c.name();
}

void configDialog::pick3DTitleFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok, plot3DTitleFont,this);
if ( ok ) 
	plot3DTitleFont = font;
else 
	return;
}

void configDialog::pick3DNumbersFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok, plot3DNumbersFont,this);
if ( ok ) 
	plot3DNumbersFont = font;
else 
	return;
}

void configDialog::pick3DAxesFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok, plot3DAxesFont,this);
if ( ok ) 
	plot3DAxesFont = font;
else 
	return;
}
