#include "exportDialog.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qregexp.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qmessagebox.h>

exportDialog::exportDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
		setName( "exportDialog" );

	QVBox *GroupBox1 = new QVBox(this, "GroupBox1" );
	GroupBox1->setSpacing (5);
	GroupBox1->setMargin (5);

	QHBox *hbox1 = new QHBox(GroupBox1, "hbox1");
	hbox1->setSpacing (5);

	new QLabel( tr( "Table" ), hbox1, "TextLabel1", 0 );
	boxTable = new QComboBox( FALSE, hbox1, "boxTable" );
	boxTable->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));

	boxAllTables = new QCheckBox(hbox1, "boxAllTables" );
    boxAllTables->setChecked(false);

    boxNames = new QCheckBox(GroupBox1, "boxNames" );
    boxNames->setChecked( TRUE );
	
    boxSelection = new QCheckBox(GroupBox1, "boxSelection" );
    boxSelection->setChecked( FALSE );
	
	QHBox *hbox2 = new QHBox(GroupBox1, "hbox2");
	hbox2->setSpacing (5);

    new QLabel( tr( "Separator" ), hbox2, "TextLabel3",0 );
    boxSeparator = new QComboBox( true, hbox2, "boxSeparator" );
	boxSeparator->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));

	QHBox *hbox3 = new QHBox(GroupBox1, "hbox3" );
	hbox3->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
	hbox3->setSpacing(10);
	hbox3->setMargin (10);
	
	buttonOk = new QPushButton(hbox3, "buttonOk" );
    buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );
   
    buttonCancel = new QPushButton(hbox3, "buttonCancel" );
    buttonCancel->setAutoDefault( TRUE );
	
	QVBoxLayout* hlayout = new QVBoxLayout(this, 5, 5, "hlayout");
    hlayout->addWidget(GroupBox1);
	hlayout->addWidget(hbox3);

	setFixedSize(minimumSize());

    languageChange();
   
    // signals and slots connections
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( boxAllTables, SIGNAL( toggled(bool) ), this, SLOT( enableTableName(bool) ) );
}

void exportDialog::languageChange()
{
    setCaption( tr( "QtiPlot - Export ASCII" ) );
    buttonOk->setText( tr( "&OK" ) );
	buttonCancel->setText( tr( "&Cancel" ) );
    boxNames->setText( tr( "Include Column &Names" ) );
    boxSelection->setText( tr( "Export &Selection" ) );
	boxAllTables->setText( tr( "&All" ) );
    boxSeparator->clear();
    boxSeparator->insertItem( tr( "TAB" ) );
    boxSeparator->insertItem( tr( "SPACE" ) );
	boxSeparator->insertItem( tr( ",TAB" ) );
	boxSeparator->insertItem( tr( ";TAB" ) );
	boxSeparator->insertItem( tr( ",SPACE" ) );
	boxSeparator->insertItem( tr( ";SPACE" ) );
    boxSeparator->insertItem( tr( ";" ) );
    boxSeparator->insertItem( tr( "," ) );
}

void exportDialog::setTableNames(const QStringList& names)
{
boxTable->insertStringList (names,-1);
}

void exportDialog::setActiveTableName(const QString& name)
{
boxTable->setCurrentText(name);
}

void exportDialog::enableTableName(bool ok)
{
boxTable->setEnabled(!ok);
}

void exportDialog::accept()
{
QString sep = boxSeparator->currentText().lower();
sep.replace("tab", "\t", false);
sep.replace("space", " ");

if (sep.contains(QRegExp("[0-9.eE+-]"))!=0)
	{
	QMessageBox::warning(0,"QtiPlot - Import options error",
				"The separator must not contain the following characters: 0-9eE.+-");
	return;
	}

if (boxAllTables->isChecked())
	emit exportAllTables(sep, boxNames->isChecked(), boxSelection->isChecked());
else
	emit exportTable(boxTable->currentText(), sep, 
					 boxNames->isChecked(), boxSelection->isChecked());
close();
}

exportDialog::~exportDialog()
{
}
