#include "fitDialog.h"
#include "parser.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qregexp.h> 
#include <qhbox.h> 
#include <qwidgetstack.h>
#include <qvbox.h>
#include <qlistbox.h>
#include <qtextedit.h>
#include <qcheckbox.h>
#include <qregexp.h>

#include <stdio.h> 

extern "C" {
double *vector(long nl, long nh);
void free_vector(double *v, long nl, long nh);}

fitDialog::fitDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
if ( !name )
	setName( "fitDialog" );
setCaption(tr("QtiPlot - Non-linear curve fit"));
setSizeGripEnabled( true );
	
tw = new QWidgetStack( this, "tw" );
tw->setSizePolicy(QSizePolicy (QSizePolicy::Preferred, QSizePolicy::Maximum, 2, 0, FALSE ));
initEditPage();
initFitPage();

tw->addWidget(editPage, 0);
tw->addWidget(fitPage, 1);

QVBoxLayout* hlayout = new QVBoxLayout(this, 5, 5, "hlayout");
hlayout->addWidget(tw);

setMaximumHeight(tw->height());

setBuiltInFunctionNames();
setBuiltInFunctions();

categoryBox->setCurrentItem(2);
funcBox->setCurrentItem(0);
}

void fitDialog::initFitPage()
{
fitPage = new QWidget( tw, "fitPage" );
QButtonGroup *GroupBox1 = new QButtonGroup( 2,QGroupBox::Horizontal,tr(""), fitPage,"GroupBox1" );
	
new QLabel( tr("Curve"), GroupBox1, "TextLabel22",0 );
boxCurve = new QComboBox(GroupBox1, "boxCurve" );

new QLabel( tr("Function"), GroupBox1, "TextLabel2",0 );
boxFunction = new QTextEdit(GroupBox1, "boxOrder" );
boxFunction->setMaximumHeight(80);

new QLabel( tr("Type:"), GroupBox1, "TextLabel2",0 );
boxFunctionType = new QLabel(GroupBox1, "boxOrder" );
boxFunctionType->setFrameStyle(QFrame::StyledPanel);
boxFunctionType->setFrameShadow(QFrame::Sunken);

new QLabel( tr("Initial guesses"), GroupBox1, "TextLabel23",0 );
boxParams = new QLineEdit(GroupBox1, "boxParams" );
boxParams->setText("a=1 b=1");

new QLabel( tr("Algorithm"), GroupBox1, "TextLabel44",0 );
boxSolver = new QComboBox(GroupBox1, "boxSolver" );
boxSolver->insertItem(tr("Scaled Levenberg-Marquardt"));
boxSolver->insertItem(tr("Unscaled Levenberg-Marquardt"));
	
QHBox *hbox=new QHBox(fitPage,"hbox");
hbox->setSpacing(5);
	
QButtonGroup* GroupBox4 = new QButtonGroup(2,QGroupBox::Horizontal,tr( "" ),hbox, "GroupBox4" );
	
new QLabel( tr("From x="), GroupBox4, "TextLabel3",0 );
boxFrom = new QLineEdit(GroupBox4, "boxPoints" );
	
new QLabel( tr("To x="), GroupBox4, "TextLabel5",0 );
boxTo = new QLineEdit(GroupBox4, "boxOrder" );
	
QButtonGroup *GroupBox3 = new QButtonGroup( 2,QGroupBox::Horizontal,tr(""),hbox,"GroupBox3" );
			
new QLabel( tr("Iterations"), GroupBox3, "TextLabel4",0 );
boxPoints = new QLineEdit(GroupBox3, "boxStart" );
boxPoints->setText("100");
	
new QLabel( tr("Tolerance"), GroupBox3, "TextLabel41",0 );
boxTolerance = new QLineEdit(GroupBox3, "boxTolerance" );
boxTolerance->setText("1e-4");

QButtonGroup *GroupBox2 = new QButtonGroup(4,QGroupBox::Horizontal,tr(""),fitPage,"GroupBox2" );
GroupBox2->setFlat (TRUE);
GroupBox2->setLineWidth (0);

buttonEdit = new QPushButton(GroupBox2, "buttonOk" );
buttonEdit->setText( tr( "<< &Edit function" ) );
	
buttonOk = new QPushButton(GroupBox2, "buttonOk" );
buttonOk->setText( tr( "&Fit" ) );
buttonOk->setAutoDefault( TRUE );
buttonOk->setDefault( TRUE );
  
buttonCancel = new QPushButton(GroupBox2, "buttonCancel" );
buttonCancel->setText( tr( "&Cancel" ) );

QVBoxLayout* hlayout = new QVBoxLayout(fitPage, 5, 5, "hlayout");
hlayout->addWidget(GroupBox1);
hlayout->addWidget(hbox);
hlayout->addWidget(GroupBox2);

// signals and slots connections
connect( buttonOk, SIGNAL( clicked() ), this, SLOT(accept()));
connect( buttonCancel, SIGNAL( clicked() ), this, SLOT(close()));
connect( buttonEdit, SIGNAL( clicked() ), this, SLOT(showEditPage()));
	
setFocusProxy(boxFunction);
}

void fitDialog::initEditPage()
{
editPage = new QWidget( tw, "editPage" );

QHBox *hbox1=new QHBox(editPage,"hbox1");
hbox1->setSpacing(5);

QVBox *vbox1=new QVBox(hbox1,"vbox1");
vbox1->setSpacing(5);
new QLabel( tr("Category"), vbox1, "TextLabel41",0 );
categoryBox = new QListBox( vbox1, "categoryBox" );
categoryBox->insertItem(tr("User defined"));
categoryBox->insertItem(tr("Built-in"));
categoryBox->insertItem(tr("Basic"));
categoryBox->setMaximumHeight(100);
categoryBox->setSizePolicy(QSizePolicy (QSizePolicy::Fixed, QSizePolicy::Expanding, 2, 0, FALSE ));

QVBox *vbox2=new QVBox(hbox1,"vbox2");
vbox2->setSpacing(5);
new QLabel( tr("Function"), vbox2, "TextLabel41",0 );
funcBox = new QListBox( vbox2, "funcBox" );
funcBox->setMaximumHeight(100);

QVBox *vbox3=new QVBox(hbox1,"vbox3");
vbox3->setSpacing(5);
new QLabel( tr("Expresion"), vbox3, "TextLabel41",0 );
explainBox = new QTextEdit( vbox3, "explainBox" );
explainBox->setReadOnly(true);
explainBox->setMaximumHeight(100);

boxUseBuiltIn = new QCheckBox(editPage,"boxUseBuiltIn");
boxUseBuiltIn->setText(tr("Fit with &built-in function"));
boxUseBuiltIn->hide();

QButtonGroup *GroupBox1 = new QButtonGroup( 3,QGroupBox::Horizontal,tr(""),editPage,"GroupBox3" );

new QLabel( tr("Name"), GroupBox1, "TextLabel41",0 );
boxName = new QLineEdit(GroupBox1, "boxName" );
boxName->setText("user1");

btnAddFunc = new QPushButton(GroupBox1, "btnAddFunc" );
btnAddFunc->setText( tr( "&Save" ) );

new QLabel( tr("Parameters"), GroupBox1, "TextLabel41",0 );
boxParam = new QLineEdit(GroupBox1, "boxParam" );
boxParam->setText("a, b");
	
btnDelFunc = new QPushButton(GroupBox1, "btnDelFunc" );
btnDelFunc->setText( tr( "&Remove" ) );

QHBox *hbox2=new QHBox(editPage,"hbox2");
hbox2->setSpacing(5);

editBox = new QTextEdit( hbox2, "editBox" );
editBox->setTextFormat(Qt::PlainText);
editBox->setFocus();

QVBox *vbox4=new QVBox(hbox2,"vbox4");
vbox4->setSpacing(5);

btnAddTxt = new QPushButton(vbox4, "btnAddTxt" );
btnAddTxt->setText( tr( "Add &expresion" ) );

btnAddName = new QPushButton(vbox4, "btnClose" );
btnAddName->setText( tr( "Add &name" ) );

buttonClear = new QPushButton(vbox4, "buttonClear" );
buttonClear->setText( tr( "Clear user &list" ) );

btnContinue = new QPushButton(vbox4, "btnContinue" );
btnContinue->setText( tr( "&Fit >>" ) );
	
QVBoxLayout* hlayout = new QVBoxLayout(editPage, 5, 5, "hlayout");
hlayout->addWidget(hbox1);
hlayout->addWidget(boxUseBuiltIn);
hlayout->addWidget(GroupBox1);
hlayout->addWidget(hbox2);

connect( buttonClear, SIGNAL( clicked() ), this, SLOT(clearList()));
connect( categoryBox, SIGNAL(highlighted(int)), this, SLOT(showFunctionsList(int) ) );
connect( funcBox, SIGNAL(highlighted(int)), this, SLOT(showExpresion(int)));
connect( boxUseBuiltIn, SIGNAL(toggled(bool)), this, SLOT(setBuiltInFunction(bool) ) );
connect( btnAddName, SIGNAL(clicked()), this, SLOT(addFunctionName() ) );
connect( btnAddTxt, SIGNAL(clicked()), this, SLOT(addFunction() ) );
connect( btnContinue, SIGNAL(clicked()), this, SLOT(showFitPage() ) );
connect( btnAddFunc, SIGNAL(clicked()), this, SLOT(saveUserFunction()));
connect( btnDelFunc, SIGNAL(clicked()), this, SLOT(removeUserFunction()));
}

void fitDialog::saveUserFunction()
{
if (editBox->text().isEmpty())
	{
	QMessageBox::critical(0, tr("QtiPlot - Input function error"),
				tr("Please enter a valid function!"));
	editBox->setFocus();
	return;
	}
else if (boxName->text().isEmpty())
	{
	QMessageBox::critical(0, tr("QtiPlot - Input function error"),
				tr("Please enter a function name!"));
	boxName->setFocus();
	return;
	}	
else if (boxParam->text().remove(QRegExp("[,;\\s]")).isEmpty())
	{
	QMessageBox::critical(0, tr("QtiPlot - Input function error"),
				tr("Please enter at least one parameter name!"));
	boxParam->setFocus();
	return;
	}

if (builtInFunctionNames.contains(boxName->text()))
	{
	QMessageBox::critical(0, tr("QtiPlot - Error: function name"),
				"<p><b>"+boxName->text()+ "</b>"+tr(" is a built-in function name"
				"<p>You must choose another name for your function!"));
	editBox->setFocus();
	return;
	}
if (editBox->text().contains(boxName->text()))
	{
	QMessageBox::critical(0, tr("QtiPlot - Input function error"),
				tr("You can't define functions recursevely!"));
	editBox->setFocus();
	return;
	}
QString name = boxName->text();
QString f = name +"(x, " + boxParam->text() + ")="+editBox->text().remove("\n");
if (userFunctionNames.contains(name))
	{
	int index = userFunctionNames.findIndex(name);
	userFunctions[index] = f;
	if (funcBox->currentText() == name)
		showExpresion(index);
	}
else
	{
	userFunctionNames << name;
	userFunctions << f;

	if (categoryBox->currentItem() == 0)
		{
		funcBox->insertItem(name, -1);
		funcBox->setCurrentItem(funcBox->numRows()-1);
		}
	}

emit saveFunctionsList(userFunctions);
}

void fitDialog::removeUserFunction()
{
QString name = funcBox->currentText();
if (userFunctionNames.contains(name))
	{
	explainBox->setText("");

	int index = userFunctionNames.findIndex(name);
	userFunctionNames.remove(name);

	QString f = userFunctions[index];
	userFunctions.remove(f);

	funcBox->clear();
	funcBox->insertStringList(userFunctionNames, -1);
	funcBox->setCurrentItem(0);

	emit saveFunctionsList(userFunctions);
	}
}

void fitDialog::showFitPage()
{
QString name = boxName->text();
QString f = name +"(x, " + boxParam->text() + ")="+editBox->text().remove("\n");

QString parameters = boxParam->text().replace(QRegExp("[,;]*[\\s]+"), "=1 ");
parameters.replace(QRegExp("[,;]+[\\s]*"), "=1 ");
boxParams->setText(parameters + "=1");

if (boxUseBuiltIn->isChecked())
	{
	boxFunction->setReadOnly(true);
	boxFunctionType->setText(tr("Built-in function: \nthe order of the parameters matters!"));
	}
else
	{
	boxFunction->setReadOnly(false);
	boxFunctionType->setText(tr("User defined function"));
	}

boxFunction->setText(f);
tw->raiseWidget(fitPage);
}

void fitDialog::showEditPage()
{
tw->raiseWidget(editPage);
}

void fitDialog::setBuiltInFunction(bool ok)
{
if (ok)
	{
	boxName->setText(funcBox->currentText());
	editBox->setEnabled(false);
	editBox->setText(explainBox->text());
	switch(funcBox->currentItem())
		{
		case 0:
			boxParam->setText("a, t, y0");
		break;
		case 1:
			boxParam->setText("a, t, y0");
		break;
		case 2:
			boxParam->setText("a1, t1, a2, t2, y0");
		break;
		case 3:
			boxParam->setText("a1, t1, a2, t2, a3, t3, y0");
		break;
		case 4:
			boxParam->setText("a, xc, w, y0");
		break;
		case 5:
			boxParam->setText("a, xc, w, y0");
		break;
		}
	}
else
	{
	boxName->setText("user1");
	editBox->setEnabled(true);
	}
}

void fitDialog::setLimits(const QStringList& limits)
{
boxFrom->setText (limits[0]);
boxTo->setText (limits[1]);
}

void fitDialog::setCurveNames(const QStringList& names)
{
boxCurve->insertStringList (names,-1);
}

void fitDialog::clearList()
{
userFunctions.clear();
userFunctionNames.clear();
if (categoryBox->currentItem() == 0)
	{
	funcBox->clear();
	explainBox->clear();
	}
emit clearFunctionsList();
}

void fitDialog::insertFunctionsList(const QStringList& list)
{
userFunctions = list;

for (int i = 0; i<(int)list.count(); i++)
	{
	QString s = list[i];
	int pos = s.find("(",0);
	userFunctionNames << s.left(pos);
	}
}

void fitDialog::showFunctionsList(int category)
{
boxUseBuiltIn->setChecked(false);
boxUseBuiltIn->hide();
funcBox->clear();

switch (category)
	{
	case 0:
		showUserFunctions();
	break;

	case 1:
		boxUseBuiltIn->show();
		funcBox->insertStringList(builtInFunctionNames, -1);
	break;

	case 2:
		showParseFunctions();
	break;
	}

funcBox->setCurrentItem(0);
showExpresion(0);
}

void fitDialog::showUserFunctions()
{
funcBox->insertStringList (userFunctionNames, 1);
}

void fitDialog::setBuiltInFunctionNames()
{
builtInFunctionNames << "ExpGrowth" << "ExpDecay1" << "ExpDecay2" << "ExpDecay3" 
<< "Gauss" << "Lorentz";
}

void fitDialog::setBuiltInFunctions()
{
builtInFunctions << "y0+a*exp(x/t)";
builtInFunctions << "y0+a*exp(-x/t)";
builtInFunctions << "y0+a1*exp(-x/t1)+a2*exp(-x/t2)";
builtInFunctions << "y0+a1*exp(-x/t1)+a2*exp(-x/t2)+a3*exp(-x/t3)";
builtInFunctions << "y0+a*exp(-(x-xc)^2/2*w^2)";
builtInFunctions << "y0+2*a/pi*w/(4*(x-xc)*(x-xc)+w*w)";
}

void fitDialog::showParseFunctions()
{
funcBox->insertItem("abs",0);
funcBox->insertItem("acos",1);
funcBox->insertItem("acosh",2);
funcBox->insertItem("asin",3);
funcBox->insertItem("asinh",4);
funcBox->insertItem("atan",5);
funcBox->insertItem("atanh",6);
funcBox->insertItem("cos",7);
funcBox->insertItem("cosh",8);
funcBox->insertItem("exp",9);
funcBox->insertItem("ln",10);
funcBox->insertItem("log",11);
funcBox->insertItem("log2",12);
funcBox->insertItem("rint",13);
funcBox->insertItem("sign",14);
funcBox->insertItem("sin",15);
funcBox->insertItem("sinh",16);
funcBox->insertItem("sqrt",17);
funcBox->insertItem("tan",18);
funcBox->insertItem("tanh",19);
}

void fitDialog::showExpresion(int function)
{
if (categoryBox->currentItem() == 2)
	{
	switch (function)
		{
		case 0:
			explainBox->setText("abs(x)");
		break;
		case 1:
			explainBox->setText("acos(x)");
		break;
		case 2:
			explainBox->setText("acosh(x)");
		break;
		case 3:
			explainBox->setText("asin(x)");
		break;
		case 4:
			explainBox->setText("asinh(x)");
		break;
		case 5:
			explainBox->setText("atan(x)");
		break;
		case 6:
			explainBox->setText("atanh(x)");
		break;
		case 7:
			explainBox->setText("cos(x)");
		break;
		case 8:
			explainBox->setText("cosh(x)");
		break;
		case 9:
			explainBox->setText("exp(x)");
		break;
		case 10:
			explainBox->setText("ln(x)");
		break;
		case 11:
			explainBox->setText("log(x)");
		break;
		case 12:
			explainBox->setText("log2(x)");
		break;
		case 13:
			explainBox->setText("rint(x)");
		break;
		case 14:
			explainBox->setText("sign(x)");
		break;
		case 15:
			explainBox->setText("sin(x)");
		break;
		case 16:
			explainBox->setText("sinh(x)");
		break;
		case 17:
			explainBox->setText("sqrt(x)");
		break;
		case 18:
			explainBox->setText("tan(x)");
		break;
		case 19:
			explainBox->setText("tanh(x)");
		break;
		}
	}
else if (categoryBox->currentItem() == 1)
	{
	explainBox->setText(builtInFunctions[function]);

	if (boxUseBuiltIn->isChecked())
		setBuiltInFunction(true);
	}
else
	{
	QStringList l = QStringList::split("=", userFunctions[function], true);
	explainBox->setText(l[1]);
	}
}

void fitDialog::addFunction()
{
QString f = explainBox->text();
if (editBox->hasSelectedText())
	{	
	f=f.remove(")");
	QString markedText=editBox->selectedText();
	editBox->insert(f+markedText+")");
	}
else
	editBox->insert(f);

editBox->setFocus();
}

void fitDialog::addFunctionName()
{
editBox->insert(funcBox->currentText());
editBox->setFocus();
}

void fitDialog::accept()
{
QString from=boxFrom->text().lower();
QString to=boxTo->text().lower();
QString points=boxPoints->text().lower();
QString tolerance=boxTolerance->text().lower();

QStringList equation=QStringList::split("=", boxFunction->text(), FALSE);
if (int(equation.count()) < 2)
	{
	QMessageBox::critical(0, tr("QtiPlot - Input function error"),
				tr("Please enter a valid function!"));
	boxFunction->setFocus();
	return;
	}	
	
if (int(equation.count()) > 2)
	{
	QMessageBox::critical(0, tr("QtiPlot - Input function error"),
				tr("Please use exactly one '=' symbol!"));
	boxFunction->setFocus();
	return;
	}

if (boxParams->text().isEmpty())
	{
	QMessageBox::critical(0, tr("QtiPlot - Input error"),
				tr("Please enter initial guesses for your parameters!"));
	boxParams->setFocus();
	return;
	}
	
double start,end,eps;
try
	{
	myParser parser;
	parser.SetExpr(from.ascii());
	start=parser.Eval();
	}
catch(mu::ParserError &e)
	{
	QMessageBox::critical(0, tr("QtiPlot - Start limit error"),e.GetMsg());
	boxFrom->setFocus();
	return;
	}	

try
	{
	myParser parser;	
	parser.SetExpr(to.ascii());
	end=parser.Eval();
	}
catch(mu::ParserError &e)
	{
	QMessageBox::critical(0, tr("QtiPlot - End limit error"),e.GetMsg());
	boxTo->setFocus();
	return;
	}	

if (start>=end)
	{
	QMessageBox::critical(0, tr("QtiPlot - Input error"),
				tr("Please enter x limits that satisfy: from < end!"));
	boxTo->setFocus();
	return;
	}
	
try
	{
	myParser parser;
	parser.SetExpr(tolerance.ascii());
	eps=parser.Eval();
	}
catch(mu::ParserError &e)
	{
	QMessageBox::critical(0, tr("QtiPlot - Tolerance input error"),e.GetMsg());
	boxTolerance->setFocus();
	return;
	}	
	
if (eps<0 || eps>=1)
	{
	QMessageBox::critical(0, tr("QtiPlot - Tolerance input error"),
				tr("The tolerance value must be positive and less than 1!"));
	boxTolerance->setFocus();
	return;
	}
	
QRegExp autoSep= QRegExp("[,;\\s]+", FALSE, FALSE);
QStringList paramsList=QStringList::split(autoSep,boxParams->text(),false);
	
int n=int(paramsList.count());
			
QStringList parameters, initialValues;
myParser parser;
	
double x, *paramsInit;
paramsInit=vector(0,n-1);

bool error=FALSE;
QString formula = equation[1];
try
	{
	int i;
	for (i=0;i<n;i++)
		{
		QStringList p=QStringList::split("=",paramsList[i],FALSE);
		paramsInit[i]=p[1].toDouble();
		parser.DefineVar(p[0].ascii(), &paramsInit[i]);
		parameters<<p[0];
		initialValues<<p[1];
		}

	bool withNames = containsUserFunctionName(formula);
	while(withNames)
		{
		for (i=0; i<(int)userFunctionNames.count(); i++)
			{
			if (formula.contains(userFunctionNames[i]))
				{
				QStringList l = QStringList::split("=", userFunctions[i], true);
				formula.replace(userFunctionNames[i], "(" + l[1] + ")");
				}
			}
		withNames = containsUserFunctionName(formula);
		}

	for (i=0; i<(int)builtInFunctionNames.count(); i++)
		{
		if (formula.contains(builtInFunctionNames[i]))
			formula.replace(builtInFunctionNames[i], "(" + builtInFunctions[i] + ")");
		}	
	parser.SetExpr(formula.ascii());		
	x=start;
	parser.DefineVar("x", &x);	
	parser.Eval();
	}
catch(mu::ParserError &e)
	{
	QString errorMsg = boxFunction->text() + " = " + formula + "\n" + e.GetMsg() + "\n" +
					tr("Please verify that you have initialized all the parameters!");

	QMessageBox::critical(0, tr("QtiPlot - Input function error"), errorMsg);
	boxFunction->setFocus();
	error=TRUE;	
	}

free_vector(paramsInit,0,n-1);

if (!error)
	{
	if (boxUseBuiltIn->isChecked())
		{
		emit fitBuiltInFunction(boxCurve->currentText(),funcBox->currentText(),initialValues,
								start,end,points.toInt(),boxSolver->currentItem(),eps);
		}
	else
		{emit fit(boxCurve->currentText(), equation[0]+"="+formula, parameters, initialValues,
					start, end, points.toInt(), boxSolver->currentItem(), eps);
		}
	}
}

bool fitDialog::containsUserFunctionName(const QString& s)
{
bool contains = false;
for (int i=0; i<(int)userFunctionNames.count(); i++)
	{
	if (s.contains(userFunctionNames[i]))
		{
		contains = true;
		break;
		}
	}
return contains;
}

fitDialog::~fitDialog()
{
}
