#include "importDialog.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qmessagebox.h>
#include <qregexp.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qvbox.h>

importDialog::importDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
		setName( "importDialog" );
    setSizeGripEnabled( true );

	QVBox  *box=new QVBox (this, "box2"); 
	box->setMargin (5);
	box->setSpacing (5);

	QHBox  *hbox1=new QHBox (box, "hbox1"); 
	hbox1->setSpacing (5);

	new QLabel( tr( "Separator" ), hbox1, "TextLabel1",0 );
    boxSeparator = new QComboBox(true, hbox1, "boxSeparator" );
	boxSeparator->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));

	boxSeparator->insertItem( tr( "TAB" ) );
    boxSeparator->insertItem( tr( "SPACE" ) );
	boxSeparator->insertItem( tr( ";TAB" ) );
	boxSeparator->insertItem( tr( ",TAB" ) );
	boxSeparator->insertItem( tr( ";SPACE" ) );
	boxSeparator->insertItem( tr( ",SPACE" ) );
    boxSeparator->insertItem( tr( ";" ) );
    boxSeparator->insertItem( tr( "," ) );

	QHBox  *hbox2=new QHBox (box, "hbox2");
	hbox2->setSpacing (5);

	new QLabel( tr( "Ignore first" ), hbox2, "TextLabel2",0 );
    boxLines = new QSpinBox(0,10000, 1, hbox2, "boxLines" );
	boxLines->setSuffix(" lines");
	
    boxRenameCols = new QCheckBox(tr( "Use first row to &name columns" ),
														box, "boxRenameCols" );
	
	GroupBox2 = new QButtonGroup( 2,QGroupBox::Horizontal,tr(""),this,"GroupBox2" );
	GroupBox2->setFlat(TRUE);
	GroupBox2->setLineWidth(0);
	GroupBox2->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));

	buttonOk = new QPushButton(GroupBox2, "buttonOk" );
    buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );
   
    buttonCancel = new QPushButton(GroupBox2, "buttonCancel" );
    buttonCancel->setAutoDefault( TRUE );
	
	QVBoxLayout* hlayout = new QVBoxLayout(this,5,5, "hlayout");
    hlayout->addWidget(box);
	hlayout->addWidget(GroupBox2);

    languageChange();
   
    // signals and slots connections
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
}

importDialog::~importDialog()
{
}

void importDialog::languageChange()
{
    setCaption( tr( "QtiPlot - ASCII Import Options" ) );
    buttonOk->setText( tr( "&OK" ) );
	buttonCancel->setText( tr( "&Cancel" ) );	
}

void importDialog::renameCols(bool rename)
{
boxRenameCols->setChecked(rename);	
}

void importDialog::setLines(int lines)
{
boxLines->setValue(lines);
}

void importDialog::setSeparator(const QString& sep)
{
if (sep=="\t")
	boxSeparator->setCurrentItem(0);
else if (sep==" ")
	boxSeparator->setCurrentItem(1);
else if (sep==";\t")
	boxSeparator->setCurrentItem(2);
else if (sep==",\t")
	boxSeparator->setCurrentItem(3);
else if (sep=="; ")
	boxSeparator->setCurrentItem(4);
else if (sep==", ")
	boxSeparator->setCurrentItem(5);
else if (sep==";")
	boxSeparator->setCurrentItem(6);
else if (sep==",")
	boxSeparator->setCurrentItem(7);
else
	boxSeparator->setCurrentText(sep);
}

void importDialog::accept()
{
QString sep = boxSeparator->currentText().lower();
sep.replace("tab", "\t", false);
sep.replace("space", " ");

if (sep.contains(QRegExp("[0-9.eE+-]"))!=0)
	{
	QMessageBox::warning(0,"QtiPlot - Import options error",
				"The separator must not contain the following characters: 0-9eE.+-");
	return;
	}

emit options(sep, boxLines->value(), boxRenameCols->isChecked());
close();
}
