#include "intDialog.h"
#include "parser.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qmessagebox.h>
#include <qhbuttongroup.h>
#include <qregexp.h> 

intDialog::intDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "polynomFitDialog" );
	setCaption(tr("QtiPlot - Integration Options"));
    setMinimumSize( QSize( 310, 140 ) );
	setMaximumSize( QSize( 310, 140 ) );
    setMouseTracking( TRUE );
    setSizeGripEnabled( FALSE );
	
	GroupBox1 = new QButtonGroup( 2,QGroupBox::Horizontal,tr(""),this,"GroupBox1" );

	new QLabel( tr("Integration of"), GroupBox1, "TextLabel1",0 );
	boxName = new QComboBox(GroupBox1, "boxShow" );
	
	new QLabel( tr("Order (1 - 5, 1 = Trapezoid Rule)"), GroupBox1, "TextLabel2",0 );
	boxOrder = new QSpinBox(1,5,1,GroupBox1, "boxOrder" );
	
	new QLabel( tr("Number of iterations (Max=40)"), GroupBox1, "TextLabel3",0 );
	boxSteps = new QSpinBox(2,40,1,GroupBox1, "boxPoints" );
	boxSteps->setValue(40);

	new QLabel( tr("Tolerance"),GroupBox1, "TextLabel4",0);
	boxTol = new QLineEdit(GroupBox1,"boxTol");
	boxTol->setText("0.01");
	
	new QLabel( tr("Lower limit"), GroupBox1, "TextLabel5",0 );
	boxStart = new QLineEdit(GroupBox1, "boxStart" );
	
	new QLabel( tr("Upper limit"), GroupBox1, "TextLabel6",0 );
	boxEnd = new QLineEdit(GroupBox1, "boxEnd" );

	GroupBox2 = new QHButtonGroup(this,"GroupBox2" );
	GroupBox2->setFlat (TRUE);
	GroupBox2->setLineWidth (0);
	
	buttonOk = new QPushButton(GroupBox2, "buttonOk" );
    buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );
   
    buttonCancel = new QPushButton(GroupBox2, "buttonCancel" );
    buttonCancel->setAutoDefault( TRUE );
    
    buttonHelp = new QPushButton(GroupBox2, "buttonHelp" );
    buttonHelp->setAutoDefault( TRUE );
    buttonHelp->setText("Help");
    
	QVBoxLayout* vlayout = new QVBoxLayout(this,5,5, "vlayout");
    vlayout->addWidget(GroupBox1);
	vlayout->addWidget(GroupBox2);

    languageChange();
   
    // signals and slots connections
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( buttonHelp, SIGNAL(clicked()),this, SLOT(help()));
    connect( boxName, SIGNAL( activated(int) ), this, SLOT(changeCurve(int)));
}

intDialog::~intDialog()
{
}


void intDialog::languageChange()
{
    buttonOk->setText( tr( "&OK" ) );
	buttonCancel->setText( tr( "&Cancel" ) );
}

void intDialog::accept()
{
    int curve=boxName->currentItem();
	
	if (points[curve]<2)
	{
	QString s="You can not integrate curve: ";
	s+="<p><b>'"+boxName->text(curve)+"'</b>";
	s+="<p>because it has less than 2 points!";
	QMessageBox::warning(0,"QtiPlot",s);	
	while(points[curve]<2)
		{//Search for the next curve with more than two points
		curve++;
		if(curve>=(int)points.count()) curve=0; //Restart from the beginning
		}
	boxName->setCurrentItem(curve);
	boxStart->setText(QString::number(Xmin[curve]));
	boxEnd->setText(QString::number(Xmax[curve]));
	return;
	}
	
	double start,stop;    
    double minx=Xmin[curve];
    double maxx=Xmax[curve];
    
    QString from=boxStart->text();
    QString end=boxEnd->text();
    QString tol=boxTol->text();
    
    QRegExp nonDigit("\\D");
    // Remove White spaces
    
    from=from.simplifyWhiteSpace();
    end=end.simplifyWhiteSpace();
    tol=tol.simplifyWhiteSpace();
    
    from=from.remove(".");
    end=end.remove(".");
    tol=tol.remove(".");

    int pos=from.find("-",0);
	if(pos==0)
		from=from.replace(pos,1,"");

	pos=end.find("-",0);
	if(pos==0)
		end=end.replace(pos,1,"");
    
    if(tol.isEmpty())
  	 	{
   		 QMessageBox::warning(0, "QtiPlot - Input error","Please give a number for the tolerance");
		boxTol->setFocus();
		return;
    	}
    
    if (tol.contains(nonDigit)>0)
    	{
		try
			{
			mu::Parser parser;
			parser.SetExpr(tol.ascii());
			parser.Eval();
			}
		catch(mu::ParserError &e)
			{
			QMessageBox::critical(0,"QtiPlot - Tolerance value error",e.GetMsg());
			boxTol->clear();
			boxTol->setFocus();
			return;
			}	
		}

// Check the Xmin 
if (from.isEmpty())
	{
	QMessageBox::warning(0, "QtiPlot - Input error","Please give a number for the lower limit ");
	boxStart->setFocus();
	return;
	}

if(from.contains(nonDigit)>0)
    {
	from=from.lower();
	if(from=="min")
		{
		boxStart->setText(QString::number(minx));
		return;
		}
	else if(from=="max")
		{
		boxStart->setText(QString::number(maxx));
		return;
		}
	else
		{
		try
			{
			myParser parser;			
			parser.SetExpr((boxStart->text()).ascii());
			start=parser.Eval();
			
			if(start<minx)
				{
				QMessageBox::warning(0, "QtiPlot - Input error","Please give a number larger or equal to the minimum value of X, for the lower limit.\n If you do not know that value, type min in the box.");
				boxStart->clear();
				boxStart->setFocus();
				return;
				}
			if(start > maxx)
				{
				QMessageBox::warning(0, "QtiPlot - Input error","Please give a number smaller or equal to the maximum value of X, for the lower limit.\n If you do not know that value, type max in the box.");
				boxStart->clear();
				boxStart->setFocus();
				return;
				}
			}
		catch(mu::ParserError &e)
			{
			QMessageBox::critical(0,"QtiPlot - Start limit error",e.GetMsg());
			boxStart->clear();
			boxStart->setFocus();
			return;
			}	
    	}
	}
else 
	{
	start=boxStart->text().toDouble();
	if(start<minx)
		{
		QMessageBox::warning(0, "QtiPlot - Input error","Please give a number larger or equal to the minimum value of X, for the lower limit.\n If you do not know that value, type min in the box.");
		boxStart->clear();
		boxStart->setFocus();
		return;
		}
	if(start>maxx )
		{
		QMessageBox::warning(0, "QtiPlot - Input error","Please give a number smaller or equal to the maximum value of X, for the lower limit.\n If you do not know that value, type max in the box.");
		boxStart->clear();
		boxStart->setFocus();
		return;
		}
	}
	
// Check Xmax
if (end.isEmpty())
 {
 QMessageBox::warning(0, "QtiPlot - Input error","Please give a number for the upper limit ");
 boxEnd->setFocus();
 return;
 }

if (end.contains(nonDigit)>0)
    {
    end=end.lower();
	if(end=="min")
		{
		boxEnd->setText(QString::number(minx));
		return;
		}
	else if(end=="max")
		{
		boxEnd->setText(QString::number(maxx));
		return;
		}
	else
		{
		try
			{
			myParser parser;
			parser.SetExpr((boxEnd->text()).ascii());
			stop=parser.Eval();
				
			if(stop>maxx)
				{
				QMessageBox::warning(0, "QtiPlot - Input error","Please give a number smaller or equal to the maximum value of X, for the upper limit.\n If you do not know that value, type max in the box.");
				boxEnd->clear();
				boxEnd->setFocus();
				return;
				}
			if(stop<minx)
				{
				QMessageBox::warning(0, "QtiPlot - Input error","Please give a number larger or equal to the minimum value of X, for the upper limit.\n If you do not know that value, type min in the box.");
				boxEnd->clear();
				boxEnd->setFocus();
				return;
				}
			}
		catch(mu::ParserError &e)
			{
			QMessageBox::critical(0,"QtiPlot - End limit error",e.GetMsg());
			boxEnd->clear();
			boxEnd->setFocus();
			return;
			}	
		}
    }
else 
{
	stop=boxEnd->text().toDouble();
	if(stop>maxx )
		{
		QMessageBox::warning(0, "QtiPlot - Input error","Please give a number smaller or equal to the maximum value of X, for the upper limit.\n If you do not know that value, type max in the box.");
		boxEnd->clear();
		boxEnd->setFocus();
		return;
		}
	if(stop<minx)
		{
		QMessageBox::warning(0, "QtiPlot - Input error","Please give a number larger or equal to the minimum value of X, for the upper limit.\n If you do not know that value, type min in the box.");
		boxEnd->clear();
		boxEnd->setFocus();
		return;
		}
}

start=boxStart->text().toDouble();
stop=boxEnd->text().toDouble();
emit options(curve, boxOrder->value(), boxSteps->value(),boxTol->text().toDouble(), start,stop);
close();
}

void intDialog::setCurveNames(const QStringList& names)
{
boxName->insertStringList (names,-1);
}

void intDialog::getMinMax(double *m,double *mm,int curves, int *p)
{
for (int i=0;i<curves;i++)
	{
	Xmin.append(*(m+i));
	Xmax.append(*(mm+i));
	points.append(*(p+i));
	}
	
int curve=0;
if(points[curve]<2)
	{
	curve++;
	while(points[curve]<2){
		curve++;}
	boxName->setCurrentItem(curve);
	}
boxStart->setText(QString::number(Xmin[curve]));
boxEnd->setText(QString::number(Xmax[curve]));
}

void intDialog::changeCurve(int curve)
{
if(points[curve]<2)
	{
	QString s="You can not integrate curve: ";
	s+="<p><b>'"+boxName->text(curve)+"'</b>";
	s+="<p>because it has less than 2 points!";
	QMessageBox::warning(0,"QtiPlot",s);
	while(points[curve]<2)
		{
		curve++;
		if(curve>=(int)points.count()) curve=0; //Restart from the beginning
		}
	}
	
boxName->setCurrentItem(curve);
boxStart->setText(QString::number(Xmin[curve]));
boxEnd->setText(QString::number(Xmax[curve]));
}

void intDialog::setCurveLimits(int curve, double xMin, double xMax)
{
boxName->setCurrentItem(curve);
boxStart->setText(QString::number(xMin));
boxEnd->setText(QString::number(xMax));
}

void intDialog::help()
{
QMessageBox::about(0,"QtiPlot - Help for Integration","The integration of a curve consists of the following five steps:\n 1) Choose which curve you want to integrate\n 2) Set the order of the integration. The higher it is the more accurate the calculation is\n 3) Choose the number of iterations \n 4) Choose the tollerance \n 5) Choose the lower and the upper limit.\n The code integrates the curve with an iterative algorithm. The tolerance determines the termination criteria for the solver.\n Because, sometimes we ask for too much accuracy, the number of iterations makes sure that the solver will not work for ever.\n IMPORTANT \nThe limits must be within the range of x; If you do not know the maximum (minimum) value of x, type max (min) in the boxes. ");
}
