#ifndef MATRIX_H
#define MATRIX_H

#include <qwidget.h>
#include <qtable.h>

#include "widget.h"
	
class Matrix: public myWidget
{
    Q_OBJECT

public:

	Matrix(int r, int c, const QString& label, QWidget* parent=0, const char* name=0, WFlags f=0);
	~Matrix(){};
	
	int numRows();
	int numCols();
	
	void init(int rows, int cols);

	bool isEmptyRow(int row);
	void addDataRow(const QString& s, int cols);

	//event handlers
	bool eventFilter(QObject *object, QEvent *e);
	void closeEvent( QCloseEvent *e );
	void askOnCloseEvent(bool ask){askOnClose = ask; };
	void resizeEvent(QResizeEvent *);
	void contextMenuEvent(QContextMenuEvent *e);

	QString birthDate(){return birthdate;};

public slots:
	void cellEdited(int,int);
	void moveCurrentCell();

	int columnsWidth();
	void setColumnsWidth(int width);

	void setMatrixDimensions(int rows, int cols);
	void transpose();
	void invert();
	double determinant();

	void setValues (const QString& txt, const QString& formula,
					const QStringList& rowIndexes, const QStringList& colIndexes,
					int startRow, int endRow, int startCol, int endCol);

	QString text (int row, int col);
	void setText (int row, int col, const QString & text );

	QChar textFormat(){return txt_format;};
	int precision(){return num_precision;};
	void setTextFormat(const QChar &format, int precision);
	void setNumericFormat(const QChar & f, int prec);

	QString formula(){return formula_str;};
	void setFormula(const QString &s);

	QString saveToString(const QString &info);
	QString saveText();

	// selection operations 
	void cutSelection();
	void copySelection();
	void clearSelection();
	void pasteSelection();
	void selectAll();

	void storeCellsToMemory();
	void freeMemory();

signals:
	void modifiedMatrix (QWidget *);
	void closedMatrix(QWidget*);
	void hiddenMatrix(QWidget*);
	void resizedMatrix(QWidget*);
	void showContextMenu();

private:
	QTable *table;
	QString birthdate, formula_str;
	QChar txt_format;
	int selectedCol, lastSelectedCol, num_precision;
	bool LeftButton, askOnClose;
	double **dMatrix;
};
   
#endif
