#include "patternBox.h"

#include <algorithm>

const Qt::BrushStyle PatternBox::patterns[] = {
  Qt::SolidPattern,
  Qt::HorPattern,
  Qt::VerPattern,
  Qt::CrossPattern,
  Qt::BDiagPattern,
  Qt::FDiagPattern,
  Qt::DiagCrossPattern,
  Qt::Dense1Pattern,
  Qt::Dense2Pattern,
  Qt::Dense3Pattern,
  Qt::Dense4Pattern,
  Qt::Dense5Pattern,
  Qt::Dense6Pattern,
  Qt::Dense7Pattern,
};

PatternBox::PatternBox(bool rw, QWidget *parent) : QComboBox(rw, parent)
{
  init();
}

PatternBox::PatternBox(QWidget *parent) : QComboBox(parent)
{
  init();
}

void PatternBox::init()
{
  this->insertItem(tr( "Solid" ) );
  this->insertItem(tr( "Horizontal" ) );
  this->insertItem(tr( "Vertical" ) );
  this->insertItem(tr( "Cross" ) );
  this->insertItem(tr( "BDiagonal" ) );
  this->insertItem(tr( "FDiagonal" ) );
  this->insertItem(tr( "DiagCross" ) );

  this->insertItem(tr( "Dense1" ) );
  this->insertItem(tr( "Dense2" ) );
  this->insertItem(tr( "Dense3" ) );
  this->insertItem(tr( "Dense4" ) );
  this->insertItem(tr( "Dense5" ) );
  this->insertItem(tr( "Dense6" ) );
  this->insertItem(tr( "Dense7" ) );
}

void PatternBox::setPattern(const Qt::BrushStyle& style)
{
  const Qt::BrushStyle*ite = std::find(patterns, patterns + sizeof(patterns), style);
  if (ite == patterns + sizeof(patterns))
    this->setCurrentItem(0); // default pattern is solid.
  else
    this->setCurrentItem(ite - patterns);
}

Qt::BrushStyle PatternBox::getSelectedPattern() const
{
  size_t i = this->currentItem();
  if (i < sizeof(patterns))
    return patterns[this->currentItem()];
  else
    return Qt::SolidPattern; // default patterns is solid. 
}
