#include "pie.h"

#include <qwt_plot.h>
#include <qwt_plot_layout.h>
#include <qwt_curve.h>
#include <qwt_painter.h>
#include <qwt_scale.h>
#include <qwt_layout_metrics.h>

#include <qpainter.h>
#include <qrect.h>
#include <qcolor.h>
#include <qpaintdevicemetrics.h>

#include <stdlib.h>

QwtPieCurve::QwtPieCurve(QwtPlot *parent, const char *name):
    QwtPlotCurve(parent,name)
{
pieRay=100;
piePen=QPen(QColor(Qt::black),1,Qt::SolidLine);
brushStyle=Qt::SolidPattern;
firstColor=0;
}

void QwtPieCurve::draw(QPainter *painter,
    const QwtDiMap &xMap, const QwtDiMap &yMap, int from, int to)
{
    if ( !painter || dataSize() <= 0 )
        return;

    if (to < 0)
        to = dataSize() - 1;

    if ( verifyRange(from, to) > 0 )
		drawPie(painter, xMap, yMap, from, to);
}

void QwtPieCurve::drawPie(QPainter *painter,
    const QwtDiMap &xMap, const QwtDiMap &yMap, int from, int to)
{   	
	int i, d = pieRay*2;
	if (painter->device()->isExtDev()) //draw on printer
		{
		QwtPlot *plot = (QwtPlot *)parentPlot();
		QwtPlotLayout *layout=plot->plotLayout();
		QRect rect=layout->canvasRect();

		QwtDiMap xMap = plot->canvasMap(QwtPlot::xBottom);
		QwtDiMap yMap = plot->canvasMap(QwtPlot::yLeft);
			
		double w0=xMap.invTransform(int((rect.width()-d)*0.5));
		double w1=xMap.invTransform(int((rect.width()+d)*0.5));
		double h0=yMap.invTransform(int((rect.height()-d)*0.5));
		double h1=yMap.invTransform(int((rect.height()+d)*0.5));
			
		QwtDiMap map=scaleMap(painter,QwtPlot::xBottom);
		int	x0=map.transform(w0);	
		int	x1=map.transform(w1);

		map=scaleMap(painter,QwtPlot::yLeft);
		int	y0=map.transform(h0);	
		int	y1=map.transform(h1);

		d=QMIN(abs(x1-x0),abs(y1-y0));
		}
	
	const int x0 = xMap.transform(x(from));
	const int x1 = xMap.transform(x(to));
	const int y0 = yMap.transform(y(from));
	const int y1 = yMap.transform(y(to));

	QRect pieRect;
	pieRect.setX(x0 + (x1 - x0 - d)/2);
	pieRect.setY(y0 + (y1 - y0 - d)/2);	
	pieRect.setHeight(d);
	pieRect.setWidth(d);	
				
	double sum=0.0;
	for (i = from; i <= to; i++)
		{
		const double yi = y(i);
		sum+=yi;
		}
	
	int angle = (int)(5760 * 0.75);
	for (i = from; i <= to; i++)
		{
		const double yi = y(i);			
		const int value = (int)(yi/sum*5760);
			
		painter->save();
		painter->setPen(piePen);
		painter->setBrush(QBrush(color(i), brushStyle));
		painter->drawPie(pieRect, -angle, -value);
		painter->restore();

		angle += value;
		}
}

QColor QwtPieCurve::color(int i)
{
QColor c;
int index=(firstColor+i)%16;
switch (index)
	{
case 0:
	c=QColor(Qt::black);
break;

case 1:
	c=QColor(Qt::red);
break;

case 2:
	c=QColor(Qt::green);
break;

case 3:
	c=QColor(Qt::blue);
break;

case 4:
	c=QColor(Qt::cyan);
break;

case 5:
	c=QColor(Qt::magenta);
break;

case 6:
	c=QColor(Qt::yellow);
break;

case 7:
	c=QColor(Qt::darkYellow);
break;

case 8:
	c=QColor(Qt::darkBlue);
break;

case 9:
	c=QColor(Qt::darkMagenta);
break;

case 10:
	c=QColor(Qt::darkRed);
break;

case 11:
	c=QColor(Qt::darkGreen);
break;

case 12:
	c=QColor(Qt::darkCyan);
break;

case 13:
	c=QColor(Qt::white);
break;

case 14:
	c=QColor(Qt::lightGray);
break;

case 15:
	c=QColor(Qt::darkGray);
break;
	}
return c;
}

void QwtPieCurve::setFirstColor(int index)
{
firstColor=index;	
}

void QwtPieCurve::setRay(int size)
{
pieRay=size;	
}

void QwtPieCurve::setBrushStyle(const Qt::BrushStyle& style)
{
if (brushStyle == style)
	return;

brushStyle =style;
}

Qt::BrushStyle QwtPieCurve::pattern()
{
return brushStyle;
}

void QwtPieCurve::setPen(const QPen& pen)
{
if (piePen == pen)
	return;

piePen=pen;
}

QwtDiMap QwtPieCurve::scaleMap(QPainter *p,int axis) 
{
QwtPlot *plot = (QwtPlot *)parentPlot();
QwtPlotLayout *pl=plot->plotLayout ();
 
QPaintDeviceMetrics pdmFrom(plot);
QPaintDeviceMetrics pdmTo(p->device());	
	
QwtMetricsMap metricsMap;
metricsMap.setMetrics(pdmFrom,pdmTo);
			
QwtDiMap map;
double from,to;	
const int margin = pl->canvasMargin(axis);		
const QRect &canvasRect = pl->canvasRect();
if ( axis == QwtPlot::yLeft )
	{
	QwtDiMap yMap = plot->canvasMap(QwtPlot::yLeft);
	map.setDblRange(yMap.d1(),yMap.d2(),yMap.logarithmic());
			 
    from = metricsMap.layoutToDeviceX(canvasRect.bottom() - margin);
    to = metricsMap.layoutToDeviceX(canvasRect.top() + margin);
	}
else
    {
	QwtDiMap xMap = plot->canvasMap(QwtPlot::xBottom);
	map.setDblRange(xMap.d1(),xMap.d2(),xMap.logarithmic());
			
    from = metricsMap.layoutToDeviceY(canvasRect.left() + margin);
    to = metricsMap.layoutToDeviceY(canvasRect.right() - margin);
    }
map.setIntRange(qwtInt(from), qwtInt(to));	
return map;	
}
