#ifndef MYDIALOG_H
#define MYDIALOG_H

#include <qvariant.h>
#include <qdialog.h>

class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class QCheckBox;
class QComboBox;
class QListBox;
class QListBoxItem;
class QPushButton;
class QSpinBox;
class QTabWidget;
class QWidget;
class QCheckBox;
class QGroupBox;
class QButtonGroup;
class QLabel;
class QLineEdit;
class ColorBox;
class PatternBox;
class ColorButton;

typedef struct{ 
  int plotType;
  int lCol;
  int lWidth;
  int lStyle;
  int filledArea;
  int aCol;
  int aStyle;
  int symCol;
  int fillCol;
  int sSize; 
  int sType;  
  int connectType;
  int barGap;
  int autoBinning;
  double binSize;
  double histBegin;
  double histEnd;
  int vectColor;
  int vectWidth;
  int headLength;
  int headAngle;
  int filledHead;
}  curveLayout;

class plotDialog : public QDialog
{ 
    Q_OBJECT

public:
    plotDialog( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~plotDialog();

    QListBox* listBox;
    QPushButton* buttonApply, *btnWorksheet, *btnAssociations;
    QPushButton* buttonOk;
    QPushButton* buttonCancel;
    QComboBox* boxPlotType;
    QWidget* linePage;
    QComboBox* boxConnect;
    QComboBox* boxLineStyle;
    QSpinBox* boxLineWidth;
    ColorBox* boxLineColor, *boxAreaColor;
    QWidget* symbolPage;
    QSpinBox* boxSymbolSize;
    ColorBox* boxSymbolColor,*boxFillColor;
    QComboBox* boxSymbolStyle;
    PatternBox *boxPattern;
	QTabWidget* privateTabWidget;
	QCheckBox* boxFill;

	QWidget* errorsPage;
	QButtonGroup* GroupBox0;
	QButtonGroup* GroupBox1;
	QButtonGroup* GroupBox2;
	QButtonGroup* GroupBox3;
	QButtonGroup* GroupBox4;
    QCheckBox* plusBox;
    QCheckBox* minusBox;
    QCheckBox* xBox;
    ColorButton* colorBox;
	ColorBox* vectColorBox;
    QComboBox* widthBox;
    QComboBox* capBox;
    QCheckBox* throughBox;
	QLabel *TextLabel4_3, *TextLabel4_4;

	QButtonGroup* GroupBoxH;
	QWidget *histogramPage, *spacingPage;
	QLineEdit *binSizeBox, *histogramBeginBox, *histogramEndBox;
	QCheckBox *automaticBox;
	QPushButton* buttonStatistics;
	QSpinBox* gapBox;

	QWidget *vectPage;
	QComboBox *xEndBox, *yEndBox;
	QSpinBox* headAngleBox, *headLengthBox, *vectWidthBox;
	QCheckBox *filledHeadBox;

public slots:
	void showStatistics();
	void clearTabWidget();
	void initLinePage();
	void initSymbolsPage();
	void initHistogramPage();
	void initErrorsPage();
	void initSpacingPage();
	void initVectPage();

	void insertCurvesList(const QStringList& names);
	void insertColumnsList(const QStringList& names){columnNames = names;};
	void updateEndPointColumns(const QString& text);
	
	bool acceptParams();
	void showWorksheet();
	void quit();

	curveLayout curveParams(int index);
	void setPlotType(int index);
	void changePlotType(int plotType);
	void setCurveParams(int curveIndex);
	void changeCurveLayout(int);

	void setAllCurvesLayout(const QArray<curveLayout>& l);
	void setSelectedCurve(int index);
	void updateTabWindow(int);
	void showAreaColor(bool show);

	curveLayout getLayout();

	void showPopupMenu(QListBoxItem *it, const QPoint &point);
	void removeCurve();
	void removeSelectedCurve();
	
	/******* error bars options **************/

	void pickErrorBarsColor();
	void changeErrorBarsWidth(int);
	void changeErrorBarsCap(int);
	void changeErrorBarsType(bool);
	void changeErrorBarsPlus();
	void changeErrorBarsMinus();
	void changeErrorBarsThrough();
	void setErrorBarsLayout(QStringList err);
	
	void setAutomaticBinning();
	bool validInput();
	void showPlotAssociations();
	void showPlotAssociations(QListBoxItem *item);

signals:
	void showPlotAssociations(int);
	void updateBarsGap(int, int);
	void updateHistogram(const QString&, int, bool, double, double, double);
	void showHistogramStats(const QString&, int);
	void worksheet(const QString&);
	void createWorksheet(const QString&);
	void setLayout(int,const curveLayout&);
	void updateErrorBars(int,bool,int,int,const QColor&,bool,bool,bool);
	void curveToRemove(int);
	void updateVectorsLayout(int, int, int, int, int, bool, const QString&, const QString&);
	
protected:
	QStringList errorBars, errorBarsLayout, columnNames;
	int selectedCurve;
	QArray<curveLayout> layout;
};

#endif // MYDIALOG_H
