#include "polynomFitDialog.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qmessagebox.h>

polynomFitDialog::polynomFitDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "polynomFitDialog" );
	setCaption(tr("QtiPlot - Polynomial Fit Options"));
    setMinimumSize( QSize( 310, 140 ) );
	setMaximumSize( QSize( 310, 140 ) );
    setMouseTracking( TRUE );
    setSizeGripEnabled( FALSE );
	
	GroupBox1 = new QButtonGroup( 2,QGroupBox::Horizontal,tr(""),this,"GroupBox1" );

	new QLabel( tr("Polynomial Fit of"), GroupBox1, "TextLabel1",0 );
	boxName = new QComboBox(GroupBox1, "boxShow" );
	
	new QLabel( tr("Order (1 - 9, 1 = linear)"), GroupBox1, "TextLabel2",0 );
	boxOrder = new QSpinBox(1,9,1,GroupBox1, "boxOrder" );
	boxOrder->setValue(2);
	
	new QLabel( tr("Fit curve # pts"), GroupBox1, "TextLabel3",0 );
	boxPoints = new QSpinBox(2,200,1,GroupBox1, "boxPoints" );
	
	new QLabel( tr("Fit curve Xmin"), GroupBox1, "TextLabel4",0 );
	boxStart = new QLineEdit(GroupBox1, "boxStart" );
	boxStart->setText(tr("0"));
	
	new QLabel( tr("Fit curve Xmax"), GroupBox1, "TextLabel5",0 );
	boxEnd = new QLineEdit(GroupBox1, "boxEnd" );

	new QLabel( tr( "Show Formula on Graph?" ), GroupBox1, "TextLabel6",0 );
    boxShowFormula = new QCheckBox(GroupBox1, "boxShow" );
    boxShowFormula->setChecked( FALSE );
	
	GroupBox2 = new QButtonGroup(1,QGroupBox::Horizontal,tr(""),this,"GroupBox2" );
	GroupBox2->setFlat (TRUE);
	GroupBox2->setLineWidth (0);
	
	buttonFit = new QPushButton(GroupBox2, "buttonFit" );
    buttonFit->setAutoDefault( TRUE );
    buttonFit->setDefault( TRUE );

	buttonOk = new QPushButton(GroupBox2, "buttonOk" );
    buttonOk->setAutoDefault( TRUE );
   
    buttonCancel = new QPushButton(GroupBox2, "buttonCancel" );
    buttonCancel->setAutoDefault( TRUE );
	
	QHBoxLayout* hlayout = new QHBoxLayout(this,5,5, "hlayout");
    hlayout->addWidget(GroupBox1);
	hlayout->addWidget(GroupBox2);

    languageChange();
   
    // signals and slots connections
	connect( buttonFit, SIGNAL( clicked() ), this, SLOT( fit() ) );
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( boxName, SIGNAL( activated(int) ), this, SLOT(changeCurve(int)));
}

polynomFitDialog::~polynomFitDialog()
{
}


void polynomFitDialog::languageChange()
{
	buttonFit->setText( tr( "&Fit" ) );
    buttonOk->setText( tr( "&OK" ) );
	buttonCancel->setText( tr( "&Cancel" ) );
}

void polynomFitDialog::fit()
{
emit options(boxName->currentText(),boxOrder->value(),
	boxPoints->value(), boxStart->text().toDouble(),
	boxEnd->text().toDouble(), boxShowFormula->isChecked());
}

void polynomFitDialog::accept()
{
fit();
close();
}

void polynomFitDialog::setCurveNames(const QStringList& names)
{
boxName->insertStringList (names,-1);
}

void polynomFitDialog::setXLimits(double *m,double *mm,int curves, int *p)
{
for (int i=0;i<curves;i++)
	{
	Xmin.append(*(m+i));
	Xmax.append(*(mm+i));
	points.append(*(p+i));
	}
	
int curve=0;
if(points[curve]<2)
	{
	curve++;
	while(points[curve]<2){
		curve++;}
	boxName->setCurrentItem(curve);
	}
boxStart->setText(QString::number(Xmin[curve]));
boxEnd->setText(QString::number(Xmax[curve]));
boxPoints->setValue(points[curve]);
}

void polynomFitDialog::setCurveLimits(int curve, double xMin, double xMax)
{
boxName->setCurrentItem(curve);
boxStart->setText(QString::number(xMin));
boxEnd->setText(QString::number(xMax));
boxPoints->setValue(points[curve]);
}

void polynomFitDialog::changeCurve(int curve)
{
if(points[curve]<2)
	{
	QString s="You can not fit curve: ";
	s+="<p><b>'"+boxName->text(curve)+"'</b>";
	s+="<p>because it has less than 2 points!";
	QMessageBox::warning(0,"QtiPlot - Warning",s);
	while(points[curve]<2)
		{
		curve++;
		if(curve>=(int)points.count()) curve=0; //Restart from the beginning
		}
	}
	
boxName->setCurrentItem(curve);
boxStart->setText(QString::number(Xmin[curve]));
boxEnd->setText(QString::number(Xmax[curve]));
boxPoints->setValue(points[curve]);
}
