#include "textDialog.h"
#include "colorButton.h"
#include "txt_icons.h"

#include "symbolDialog.h"

#include <qlabel.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qfont.h>
#include <qfontdialog.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <qcolordialog.h>

textDialog::textDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "textDialog" );
    resize(330, 300 ); 
    setCaption( tr( "QtiPlot - Text Dialog" ) );
    setSizeGripEnabled( FALSE );
	
	GroupBox1 = new QButtonGroup(3,QGroupBox::Horizontal,tr( "" ),this, "GroupBox1" );
    
	TextLabel2 = new QLabel(GroupBox1, "TextLabel2" ); 
    TextLabel2->setText( tr( "Title Color" ) );

	boxColor = new ColorButton(GroupBox1);
	boxColor->setText(tr("Co&lor"));

	buttonOk = new QPushButton( GroupBox1, "buttonOk" );
    buttonOk->setText( tr( "&OK" ) );
    buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );
	
	TextLabel1 = new QLabel( GroupBox1, "TextLabel1" );
    TextLabel1->setText( tr( "Title Font" ) );

    buttonFont = new QPushButton( GroupBox1, "buttonFont" );
    buttonFont->setText( tr( "&Font" ) );
    buttonFont->setAutoDefault( TRUE );
	
	buttonApply = new QPushButton( GroupBox1, "buttonApply" );
    buttonApply->setText( tr( "&Apply" ) );
    buttonApply->setAutoDefault( TRUE );
	
	new QLabel(tr( "Alignement" ),GroupBox1, "TextLabel1_22",0);
	
	alignementBox = new QComboBox( FALSE, GroupBox1, "alignementBox" );
    alignementBox->insertItem( tr( "Center" ) );
	alignementBox->insertItem( tr( "Left" ) );
    alignementBox->insertItem( tr( "Right" ) );
	
    buttonCancel = new QPushButton( GroupBox1, "buttonCancel" );
    buttonCancel->setText( tr( "&Cancel" ) );
    buttonCancel->setAutoDefault( TRUE );
	
	GroupBox2= new QButtonGroup(7,QGroupBox::Horizontal,tr( "" ),this, "GroupBox2" );
	
	buttonIndice = new QPushButton( GroupBox2, "buttonIndice" ); 
    buttonIndice->setPixmap (QPixmap(index_xpm));
	buttonIndice->setMaximumWidth(40);

    buttonExp = new QPushButton( GroupBox2, "buttonExp" );
    buttonExp->setPixmap (QPixmap(exp_xpm));
	buttonExp->setMaximumWidth(40);

    buttonMinGreek = new QPushButton(QChar(0x3B1), GroupBox2, "buttonMinGreek" ); 
	buttonMinGreek->setMaximumWidth(40);

	buttonMajGreek = new QPushButton(QChar(0x393), GroupBox2, "buttonMajGreek" ); 
	buttonMajGreek->setMaximumWidth(40);

	QFont font = this->font();
	font.setBold(true);

    buttonB = new QPushButton("B", GroupBox2, "buttonB" ); 
    buttonB->setFont(font);
	buttonB->setMaximumWidth(40);

	font = this->font();
	font.setItalic(true);
    buttonI = new QPushButton("It", GroupBox2, "buttonI" );
	buttonI->setFont(font);
	buttonI->setMaximumWidth(40);

	font = this->font();
	font.setUnderline(true);

    buttonU = new QPushButton("U", GroupBox2, "buttonU" );
	buttonU->setFont(font);
	buttonU->setMaximumWidth(40);

	LineEdit = new QMultiLineEdit( this, "LineEdit" );
	LineEdit->setFocus();
	
	QVBoxLayout* vlayout = new QVBoxLayout(this,5,5, "vlayout");
    vlayout->addWidget(GroupBox1);
	vlayout->addWidget(GroupBox2);
	vlayout->addWidget(LineEdit);

    // signals and slots connections
	connect( boxColor, SIGNAL( clicked() ), this, SLOT(pickTextColor() ) );
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( buttonApply, SIGNAL( clicked() ), this, SLOT( apply() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( buttonFont, SIGNAL( clicked() ), this, SLOT(customFont() ) );
	
	connect( buttonExp, SIGNAL(clicked()), this, SLOT(addExp()));
	connect( buttonIndice, SIGNAL(clicked()), this, SLOT(addIndex()));

	connect( buttonU, SIGNAL(clicked()), this, SLOT(addUnderline()));
	connect( buttonI, SIGNAL(clicked()), this, SLOT(addItalic()));
	connect( buttonB, SIGNAL(clicked()), this, SLOT(addBold()));

	connect(buttonMinGreek, SIGNAL(clicked()), this, SLOT(showMinGreek()));
	connect(buttonMajGreek, SIGNAL(clicked()), this, SLOT(showMajGreek()));
}

void textDialog::showMinGreek()
{
symbolDialog *greekLetters = new symbolDialog(symbolDialog::minGreek, this,"greekLetters",
											  false, WStyle_Tool|WDestructiveClose);
connect(greekLetters, SIGNAL(addLetter(const QString&)), this, SLOT(addSymbol(const QString&)));
greekLetters->show();
greekLetters->setActiveWindow();
}

void textDialog::showMajGreek()
{
symbolDialog *greekLetters = new symbolDialog(symbolDialog::majGreek, this, "greekLetters",
											  false, WStyle_Tool|WDestructiveClose);
connect(greekLetters, SIGNAL(addLetter(const QString&)), this, SLOT(addSymbol(const QString&)));
greekLetters->show();
greekLetters->setActiveWindow();
}

bool textDialog::rejected()
{
if (result()==0)
	{
	reject();
	return TRUE;
	}	
else 
return FALSE;
}

void textDialog::apply()
{	
emit changeAlignment(alignment());
emit changeText(LineEdit->text());
emit changeColor(boxColor->color());
}

void textDialog::accept()
{
apply();
close();
}

int textDialog::alignment()
{
int align=-1;
switch (alignementBox->currentItem())
	{
	case 0:
		align=Qt::AlignHCenter;
	break;

	case 1:
		align=Qt::AlignLeft;
	break;
	
	case 2:
		align=Qt::AlignRight;
	break;
	}
return align;
}

void textDialog::setAlignment(int align)
{	
if (align==Qt::AlignHCenter)
	alignementBox->setCurrentItem(0);
else if (align==Qt::AlignLeft)
	alignementBox->setCurrentItem(1);
else if (align==Qt::AlignRight)
	alignementBox->setCurrentItem(2);
else
	return;
}

void textDialog::customFont()
{
bool okF;
QFont fnt = QFontDialog::getFont( &okF,f,this);
if (okF)
f=fnt;
emit changeFont (fnt);
}

QFont textDialog::font()
{
return f;
}

void textDialog::setFont(const QFont& fnt)
{
f=fnt;
}

void textDialog::setText(const QString & t)
{
LineEdit->insertAt (t, 0, 0, TRUE );
}

void textDialog::setColor(QColor c)
{
  boxColor->setColor(c);
}

void textDialog::pickTextColor()
{
QColor c = QColorDialog::getColor( boxColor->color(), this);
if ( !c.isValid() || c ==  boxColor->color() )
	return;

boxColor->setColor ( c ) ;
emit changeColor(c);
}

QString textDialog::getText()
{
return LineEdit->text();
}

void textDialog::addUnderline()
{
int line=0, col=0;
if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<u>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</u>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insert("<u></u>");
	LineEdit->setCursorPosition (line,col+3,FALSE);
	}
}

void textDialog::addItalic()
{
int line=0, col=0;

if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<i>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</i>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insert("<i></i>");
	LineEdit->setCursorPosition (line,col+3,FALSE);
	}
}

void textDialog::addBold()
{
int line=0, col=0;
	
if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<b>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</b>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insert("<b></b>");
	LineEdit->setCursorPosition (line,col+3,FALSE);
	}
}

void textDialog::addIndex()
{
int line=0, col=0;
LineEdit->getCursorPosition (&line,&col);
	
if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<sub>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</sub>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->insert("<sub></sub>");
	LineEdit->setCursorPosition (line,col+5,FALSE);
	}
}

void textDialog::addExp()
{
int line, col;
LineEdit->getCursorPosition (&line,&col);

if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<sup>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</sup>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->insert("<sup></sup>");
	LineEdit->setCursorPosition (line,col+5,FALSE);
	}
}

void textDialog::addSymbol(const QString& letter)
{
LineEdit->insert(letter);
}

textDialog::~textDialog()
{
}
