#include "worksheet.h"
#include "sortDialog.h"
#include "parser.h"
#include "widget.h"

#include <qpopupmenu.h>
#include <qfiledialog.h>
#include <qinputdialog.h>
#include <qheader.h>
#include <qmessagebox.h>
#include <qarray.h>
#include <qfileinfo.h>
#include <qfile.h>
#include <qregexp.h>
#include <qtextstream.h>
#include <qclipboard.h> 
#include <qapplication.h> 
#include <qcursor.h>
#include <qdatetime.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qdragobject.h>
#include <qlayout.h>
#include <qaccel.h>
#include <qprogressbar.h>

#include <stdlib.h>
#include <math.h>
#include <stdio.h>

#include <gsl/gsl_vector.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_sort_vector.h>
#include <gsl/gsl_statistics.h>

extern "C" {
double **matrix(long nrl, long nrh, long ncl, long nch);
void free_matrix(double **m, long nrl, long nrh, long ncl, long nch);
double *vector(long nl, long nh);
void free_vector(double *v, long nl, long nh);
size_t *ivector(long nl, long nh);
void free_ivector(size_t *v, long nl, long nh);
}

Table::Table(const QString &fname,const QString &sep, int ignoredLines, bool renameCols,
				  const QString& label, QWidget* parent, const char* name, WFlags f)
        : myWidget(label, parent,name,f)
{
QApplication::setOverrideCursor(waitCursor);
importASCII(fname, sep, ignoredLines, renameCols);
QApplication::restoreOverrideCursor();
}

Table::Table(int r, int c, const QString& label, QWidget* parent, const char* name, WFlags f)
        : myWidget(label,parent,name,f)
{
init(r,c);	
}

void Table::init(int rows, int cols)
{
selectedCol=0;clickedRow=0;xCol=0;yCol=1;
LeftButton=FALSE;
	
QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();

birthdate=dt.toString ("dd/MM/yyyy ");
birthdate+=time.toString();

worksheet= new QTable (rows,cols,this,"table");
worksheet->setFocusPolicy(QWidget::StrongFocus);
worksheet->setFocus();
worksheet->setSelectionMode (QTable::Single);

QVBoxLayout* hlayout = new QVBoxLayout(this,0,0, "hlayout1");
hlayout->addWidget(worksheet);

for (int i=0; i<cols; i++)
	{
	commandes << "col(1)";
	colTypes << Numeric;
	dateTimeFormats << "";
	comments << "";
	}

QHeader* head=(QHeader*)worksheet->horizontalHeader();
head->installEventFilter(this);
head->setMouseTracking(TRUE);
head->setLabel(0,"1[X]");
head->setLabel(1,"2[Y]");

int w=4*(worksheet->horizontalHeader())->sectionSize(0);
int h;
if (rows>11)
	h=11*(worksheet->verticalHeader())->sectionSize(0);
else
	h=(rows+1)*(worksheet->verticalHeader())->sectionSize(0);
setGeometry(50,50,w + 45, h);

QAccel *accel = new QAccel(this);
accel->connectItem( accel->insertItem( Key_Tab ),
                            this, SLOT(moveCurrentCell()) );
accel->connectItem( accel->insertItem( CTRL+Key_A ),
                            this, SLOT(selectAllTable()) );

connect(worksheet->verticalHeader(), SIGNAL(clicked(int)), this, SIGNAL(rowMenu(int)));
connect(worksheet, SIGNAL(valueChanged(int,int)),this, SLOT(cellEdited(int,int)));
}

void Table::setBackgroundColor(const QColor& col)
{
worksheet->setPaletteBackgroundColor ( col );
}

void Table::setTextColor(const QColor& col)
{
worksheet->setPaletteForegroundColor (col);
}

void Table::setTextFont(const QFont& fnt)
{
worksheet->setFont (fnt);
QFontMetrics fm(fnt);
int lm = fm.width( QString::number(10*worksheet->numRows()));
worksheet->setLeftMargin( lm );
}

void Table::setHeaderColor(const QColor& col)
{
worksheet->horizontalHeader()->setPaletteForegroundColor (col);
}

void Table::setHeaderFont(const QFont& fnt)
{
worksheet->horizontalHeader()->setFont (fnt);
}

void Table::print()
{
QPrinter printer;
printer.setColorMode (QPrinter::GrayScale);
if (printer.setup()) 
	{
        printer.setFullPage( TRUE );
        QPainter p;
        if ( !p.begin(&printer ) )
            return; // paint on printer
        QPaintDeviceMetrics metrics( p.device() );
        int dpiy = metrics.logicalDpiY();
        int margin = (int) ( (2/2.54)*dpiy ); // 2 cm margins
		
		int rows=worksheet->numRows();
		int cols=worksheet->numCols();
		int height=margin;
		int right=margin;
		int i,width=0;
		
		// print header
		QStringList header=colNames();	
		p.setFont(QFont("Arial",12, QFont::Normal, FALSE));
		QRect br=p.boundingRect(br,Qt::AlignCenter,	header[0],-1,0);
		p.drawLine(right,height,right,height+br.height());
		QRect tr(br);	
		
		for (i=0;i<cols;i++)
			width+=worksheet->columnWidth (i);
		
		if (width >= metrics.width()-2*margin )
			printer.setOrientation(QPrinter::Landscape);
		
		for (i=0;i<cols;i++)
		{	
			int width=worksheet->columnWidth (i);
			tr.setTopLeft(QPoint(right,height));
			tr.setWidth(width);	
			tr.setHeight(br.height());
			p.fillRect (tr,QBrush(QColor(lightGray)));
			p.drawText(tr,Qt::AlignCenter,header[i],-1);
			right+=width;
			p.drawLine(right,height,right,height+tr.height());
			
			if (right >= metrics.width()-2*margin )
				break;
		}
		p.drawLine(margin,height,right,height);
		height+=tr.height();	
		p.drawLine(margin,height,right,height);		
		
		// print table values
		for (i=0;i<rows;i++)
		{
			right=margin;
			QString text=worksheet->text(i,0)+"\t";
			tr=p.boundingRect(tr,Qt::AlignCenter,text,-1,0);
			p.drawLine(right,height,right,height+tr.height());
			for (int j=0;j<cols;j++)
			{
			int width=worksheet->columnWidth (j);
			text=worksheet->text(i,j)+"\t";
			tr=p.boundingRect(tr,Qt::AlignCenter,text,-1,0);
			br.setTopLeft(QPoint(right,height));	
			br.setWidth(width);	
			br.setHeight(tr.height());
			p.drawText(br,Qt::AlignCenter,text,-1);
			right+=width;
			p.drawLine(right,height,right,height+tr.height());
				
			if (right >= metrics.width()-2*margin )
				break;
			}
			height+=br.height();
			p.drawLine(margin,height,right,height);	
			
			if (height >= metrics.height()-margin )
				{
            	printer.newPage();
				height=margin;
				p.drawLine(margin,height,right,height);
				}
		}	
    }
}

void Table::cellEdited(int,int col)
{
QString name=colName(col);
emit modifiedData(name);
emit modifiedTable(this);
}

int Table::colX()
{
return xCol;
}

int Table::colY()
{
return yCol;
}

bool Table::setXCol()
{
if (selectedCol == xCol)
	return true;
	
QHeader *head=worksheet->horizontalHeader();

if (selectedCol == yCol)
	{
	QString yColName=(head->label(yCol)).remove("[Y]");
	head->setLabel(yCol, yColName);
	yCol = -1;	
	}

QString xColName=(head->label(xCol)).remove("[X]");
head->setLabel(xCol,xColName);
xCol=selectedCol;
xColName=head->label(xCol)+"[X]";
worksheet->horizontalHeader()->setLabel(xCol,xColName);
	
emit modifiedTable(this);
return true;
}

void Table::disregardCol()
{
QHeader *head=worksheet->horizontalHeader();
if (selectedCol == xCol)
	{
	QString xColName=(head->label(xCol)).remove("[X]");
	head->setLabel(xCol,xColName);
	xCol=-1;
	emit modifiedTable(this);
	}
else if (selectedCol == yCol)
	{
	QString yColName=(head->label(yCol)).remove("[Y]");
	head->setLabel(yCol,yColName);
	yCol=-1;
	emit modifiedTable(this);
	}
else
  return;
}

void Table::setXCol(int i)
{
xCol=i;
}

bool Table::setYCol()
{
if (selectedCol == yCol)
	return true;
	
QHeader *head=worksheet->horizontalHeader();

if (selectedCol == xCol)
	{
	QString xColName=(head->label(xCol)).remove("[X]");
	head->setLabel(xCol, xColName);
	xCol = -1;
	}

QString yColName=(head->label(yCol)).remove("[Y]");
head->setLabel(yCol,yColName);
yCol=selectedCol;
yColName=head->label(yCol)+"[Y]";
worksheet->horizontalHeader()->setLabel(yCol,yColName);
emit modifiedTable(this);
return true;
}

void Table::setYCol(int i)
{
yCol=i;
}

int Table::columnWidth(int col)
{
return worksheet->columnWidth(col);
}

QStringList Table::columnWidths()
{
QStringList widths;
for (int i=0;i<worksheet->numCols();i++)
	widths<<QString::number(worksheet->columnWidth(i));
	
return widths;
}

void Table::setColWidths(const QStringList& widths)
{
for (int i=0;i<(int)widths.count();i++)
	worksheet->setColumnWidth (i, widths[i].toInt() );
}

void Table::setColumnTypes(const QStringList& ctl)
{
for (int i=0; i<(int)ctl.count(); i++)
	{
	QStringList l= QStringList::split(";", ctl[i], true);
	colTypes[i] = l[0].toInt();
		
	if ((int)l.count() > 0)
		dateTimeFormats[i] = l[1];
	}
}

QString Table::saveColumnWidths()
{
QString s="ColWidth\t";
for (int i=0;i<worksheet->numCols();i++)
	s+=QString::number(worksheet->columnWidth (i))+"\t";
	
return s+"\n";
}

QString Table::saveColumnTypes()
{
QString s="ColType\t";
for (int i=0; i<worksheet->numCols(); i++)
	{
	s+=QString::number(colTypes[i]);
	if (colTypes[i] == Time || colTypes[i] == Date)
		s+= ";"+dateTimeFormats[i];
	
	s+="\t";	
	}
return s+"\n";
}

QString Table::saveCommandes()
{
QString s="com\t";
s+=commandes.join ("\t")+"\n";
return s;
}

QString Table::saveComments()
{
QString s="Comments\t";
for (int i=0; i<worksheet->numCols(); i++)
	s+= comments[i] + "\t";
return s + "\n";
}

QString Table::saveToString(const QString& geometry, const QString& birth)
{
QString s="<table>\n";
s+=QString(name())+"\t";
s+=QString::number(worksheet->numRows())+"\t";
s+=QString::number(worksheet->numCols())+"\t";
s+=birth+"\t";
s+=QString::number(colX())+"\t";
s+=":\t";
s+=QString::number(colY())+"\n";	//starting from 0.5.0 version
s+= geometry;
s+=saveHeader();
s+=saveColumnWidths();
s+=saveCommandes();
s+=saveColumnTypes();
s+=saveComments();
s+="WindowLabel\t" + windowLabel() + "\t" + QString::number(captionPolicy()) + "\n";
s+=saveText();
return s+="</table>\n";
}

QString Table::saveHeader()
{
QString s="header\t";
for (int j=0; j<worksheet->numCols(); j++)
	s+= colLabel(j) + "\t";

return s+="\n";
}

QStringList Table::getCommandes()
{
return commandes;
}

void Table::setCommandes(const QStringList& com)
{
commandes=com;
}

void Table::setCommandes(const QString& com)
{
QStringList list=QStringList::split ("\t",com,FALSE);
for (int i=0;i<int(list.count());i++)
	{
	commandes[i]=list[i+1];
	}
}

QString Table::birthDate()
{
return birthdate;
}

void Table::enumerateRightCols(bool checked)
{	
if (!checked)
	return;

QString oldLabel=colLabel(selectedCol);
QStringList oldLabels=colNames();
QString caption=QString(this->name())+"_";
int n=1;
for (int i=selectedCol; i<(int)worksheet->numCols(); i++)
	{
	QString newLabel=oldLabel+QString::number(n);
	commandes.gres("col("+colLabel(i)+")", "col("+newLabel+")", true);
	setColName(i, newLabel);
	emit changedColHeader(caption+oldLabels[i],colName(i));
	n++;
	}	
emit modifiedTable(this);
}

void Table::setColComment(const QString& s)
{
if (comments[selectedCol] == s)
	return;

comments[selectedCol] = s;
}

void Table::changeColWidth(int width, bool allCols)
{
int cols=worksheet->numCols();
if (allCols)
	{
	for (int i=0;i<cols;i++)
		worksheet->setColumnWidth (i, width);
	
	emit modifiedTable(this);
	}
else
	{
	if (worksheet->columnWidth(selectedCol) == width)
		return;
	
	worksheet->setColumnWidth (selectedCol, width);
	emit modifiedTable(this);
	}
}

void Table::changeColName(const QString& text)
{
QString caption = this->name();
QString oldName = colName(selectedCol);
QString newName = caption+"_"+text;

if (oldName == newName)
	return;

if (caption == text)
	{
	QMessageBox::critical(0,tr("QtiPlot - Error"),
	"The column name must be different from the table name : <b>"+caption+"</b></b>!<p>Please choose another name!");
	return;	
	}

QStringList labels=colNames();
if (labels.contains(text)>0)
	{
	QMessageBox::critical(0,tr("QtiPlot - Error"),
	"There is already a column called : <b>"+text+"</b> in table <b>"+caption+"</b>!<p>Please choose another name!");
	return;	
	}

commandes.gres("col("+colLabel(selectedCol)+")", "col("+text+")", true);	
	
setColName(selectedCol, text);	
emit changedColHeader(oldName, newName);
emit modifiedTable(this);
}

void Table::setColName(int col, const QString& text)
{
if (col<0 || col >=worksheet->numCols())
	return; 

if (col == xCol)
	worksheet->horizontalHeader()->setLabel(col,text+"[X]",-1);
else if (col == yCol)
	worksheet->horizontalHeader()->setLabel(col,text+"[Y]",-1);
else
	worksheet->horizontalHeader()->setLabel(col,text,-1);
}

QStringList Table::selectedColumns()
{
int i;
QStringList names;
for (i=0;i<worksheet->numCols();i++)
	{
	if(worksheet->isColumnSelected (i,TRUE))
		{
		if (i==xCol)
			names<<QString(name())+"_"+(worksheet->horizontalHeader()->label(i)).remove("[X]");
		else if (i==yCol)
			names<<QString(name())+"_"+(worksheet->horizontalHeader()->label(i)).remove("[Y]");
		else
			names<<QString(name())+"_"+worksheet->horizontalHeader()->label(i);
		}
	}
return names;
}

int Table::firstSelectedColumn()
{
for (int i=0;i<worksheet->numCols();i++)
	{
	if(worksheet->isColumnSelected (i,TRUE))
		return i;
	}
	
return -1;
}

int Table::selectedRows()
{
int r=0;
for (int i=0;i<worksheet->numRows();i++)
	{
	if(worksheet->isRowSelected (i,TRUE))
		r++;
	}
return r;
}

int Table::selectedColsNumber()
{
int c=0;
for (int i=0;i<worksheet->numCols(); i++)
	{
	if(worksheet->isColumnSelected (i,TRUE))
		c++;
	}
return c;
}

QStringList Table::colNames()
{
QStringList names;
for (int i=0;i<worksheet->numCols();i++)
	{
	if (i==xCol)
		names<<(worksheet->horizontalHeader()->label(i)).remove("[X]");
	else if (i==yCol)
		names<<(worksheet->horizontalHeader()->label(i)).remove("[Y]");
	else
		names<<worksheet->horizontalHeader()->label(i);
	}
return names;
}

QString Table::colLabel(int col)
{//returns the horizontal header text
QString name=worksheet->horizontalHeader()->label(col);
return name.remove("[X]").remove("[Y]");
}

QString Table::colName(int col)
{//returns the table name + horizontal header text
if (col<0 || col >=worksheet->numCols())
	return "";

QString name=QString(this->name())+"_"+worksheet->horizontalHeader()->label(col);
return name.remove("[X]").remove("[Y]");
}

int Table::colIndex(const QString& name)
{
int cols=worksheet->numCols();
int index=-1;
int pos=name.find("_",FALSE);
QString label=name.right(name.length()-pos-1);
for (int i=0; i<cols; i++)
	{
	if (colLabel(i) == label)
		{
		return i;
		}
	}
return index;
}

QArray<double> Table::col(int ycol)
{
int i;
int rows=worksheet->numRows();
int cols=worksheet->numCols();
QArray<double> Y(rows);

if (ycol<=cols)
	{
	for (i=0;i<rows;i++)
		Y[i]=worksheet->text(i,ycol).toDouble();
	}
return Y;
}

void Table::insertCols(int start, int count)
{	
int i, index, max=0;
int cols=worksheet->numCols();
QRegExp letters=QRegExp ("\\D");
QString xColLabel, yColLabel;
for (i=0; i<cols; i++)
	{
	QString label=colLabel(i);
	if (i == xCol)
		xColLabel=label;
	else if (i == yCol)
		yColLabel=label;
	
	if (label.contains(letters)==0)
		{
		index=label.toInt();
		if (index>max) 
			max=index;
		}		
	}
	
QStringList::Iterator it=commandes.at(start);
QStringList::Iterator dt_it=dateTimeFormats.at(start);
QStringList::Iterator c_it=comments.at(start);
QValueListIterator<int> itt=colTypes.at(start);
start++;
	
QStringList labels;
for (i=start; i<cols; i++)
	labels<<worksheet->horizontalHeader()->label(i);

worksheet->insertColumns (start, count);

int offset = start+count;
for (i=0; i<(int)labels.count(); i++)		
	worksheet->horizontalHeader()->setLabel(offset+i, labels[i], -1);
	
max++;
for (i=0; i<count; i++)
	{
	commandes.insert(++it, "col(" +xColLabel+")");
	dateTimeFormats.insert(++dt_it, "");
	comments.insert(++c_it, "");
	colTypes.insert(++itt, Numeric);
	worksheet->horizontalHeader()->setLabel(start+i, QString::number(max+i), -1);
	}
	
xCol=colIndex(xColLabel);
yCol=colIndex(yColLabel);
}

void Table::insertCol()
{
insertCols(selectedCol,1);
emit modifiedTable(this);
}

void Table::insertRow()
{
worksheet->insertRows(clickedRow,1 );
emit modifiedTable(this);
}

void Table::addCol()
{
worksheet->clearSelection();
int cols=worksheet->numCols();
QRegExp letters=QRegExp ("\\D");
int index,max=0,i;
QString xColLabel;
for (i=0; i<cols; i++)
	{
	QString label=colLabel(i);
	if (i == xCol)
		xColLabel=label;

	if (label.contains(letters)==0)
		{
		index=label.toInt();
		if (index>max) max=index;
		}		
	}
worksheet->insertColumns(cols,1);
worksheet->horizontalHeader()->setLabel (cols,QString::number(max+1),-1);
worksheet->ensureCellVisible ( 0, cols );
	
emit modifiedTable(this);
commandes<<"col("+xColLabel+")";
colTypes<<Numeric;
dateTimeFormats<<"";
}

void Table::clearCol()
{
int rows=worksheet->numRows();
for (int i=0;i<rows;i++)
	{
	worksheet->setText(i,selectedCol, "");
	}
QString name=colName(selectedCol);
emit modifiedData(name);
emit modifiedTable(this);
}

void Table::clearCell(int row, int col)
{
worksheet->setText(row, col, "");

QString name=colName(col);
emit modifiedData(name);
emit modifiedTable(this);
}

int Table::atRow(int col, double value)
{
if (colTypes[col] == Numeric)
	{
	int row = -1;
	for (int i=0; i<worksheet->numRows(); i++)
		{
		if (worksheet->text(i,col).toDouble() == value)
			return i;
		}
	return row;
	}
else
	return  -1;
}

void Table::deleteRow()
{
int cols=worksheet->numCols();
worksheet->removeRow(clickedRow);
for (int i=0;i<cols;i++)
	{
	QString name=colName(i);
	emit modifiedData(name);
	}
emit modifiedTable(this);
}

void Table::deleteSelectedRows()
{
QTableSelection sel=worksheet->selection(worksheet->currentSelection());
int top=sel.topRow();
int bottom=sel.bottomRow();

int i;
for (i=top; i<=bottom; i++)
	worksheet->removeRow(top);

int cols=worksheet->numCols();
for (i=0; i<cols; i++)
	{
	QString name=colName(i);
	emit modifiedData(name);
	}
emit modifiedTable(this);
}

void Table::cutSelection()
{
copySelection();
clearSelection();
}

bool Table::singleCellSelected()
{
bool single=TRUE;
QTableSelection sel=worksheet->selection(worksheet->currentSelection());
int top=sel.topRow();
int bottom=sel.bottomRow();
int left=sel.leftCol();
int right=sel.rightCol();
	
if (top != bottom || left != right)
	single=FALSE;
return single;
}

bool Table::multipleRowsSelected()
{
bool selected = true;
QTableSelection sel=worksheet->selection(worksheet->currentSelection());
int top=sel.topRow();
int bottom=sel.bottomRow();
	
if (top == bottom)
	return FALSE;

for (int i=top; i<=bottom; i++)
	{
	if (!worksheet->isRowSelected (i, true))
		return FALSE;
	}
return selected;
}

void Table::selectAllTable()
{	
worksheet->addSelection (QTableSelection( 0, 0, worksheet->numRows(), worksheet->numCols() ));
}

void Table::deselect()
{	
worksheet->clearSelection();
}

void Table::clearSelection()
{	
QStringList list=selectedColumns();
int i,n=int(list.count());

if (n>0)
{
for (i=0;i<n;i++)
	{
	QString name=list[i];
	int id=colIndex(name);
	selectedCol=id;
	clearCol();
	}
}
else
{
QTableSelection sel=worksheet->selection(worksheet->currentSelection());
int top=sel.topRow();
int bottom=sel.bottomRow();
int left=sel.leftCol();
int right=sel.rightCol();
	
if (sel.isEmpty ())
	{
	int row=worksheet->currentRow ();
	int col=worksheet->currentColumn ();
	worksheet->setText(row,col, "");
		
	QString name=colName(col);
	emit modifiedData(name);		
	}
else
	{	
	for (i=top;i<=bottom;i++)
		{
		for (int j=left;j<=right;j++)
			{
			worksheet->setText(i, j, "");
			}
		}

	for (i=left;i<=right;i++)
		{
		QString name=colName(i);
		emit modifiedData(name);
		}
	}
}
emit modifiedTable(this);
}

void Table::copySelection()
{
QString text;
int i,j;
int rows=worksheet->numRows();
int cols=worksheet->numCols();

QArray<int> selection(1);
int c=0;	
for (i=0;i<cols;i++)
	{
	if (worksheet->isColumnSelected(i,TRUE))
		{
		c++;
		selection.resize(c);
		selection[c-1]=i;			
		}
	}
if (c>0)
	{	
	for (i=0; i<rows; i++)
		{
		for (j=0;j<c-1;j++)
			text+=worksheet->text(i,selection[j])+"\t";
		text+=worksheet->text(i,selection[c-1])+"\n";
		}	
	}
else
	{
	QTableSelection sel=worksheet->selection(worksheet->currentSelection());
	int top=sel.topRow();
	int bottom=sel.bottomRow();
	int left=sel.leftCol();
	int right=sel.rightCol();
	for (i=top; i<=bottom; i++)
		{
		for (j=left; j<right; j++)
			text+=worksheet->text(i,j)+"\t";
		text+=worksheet->text(i,right)+"\n";
		}
	}		
	
// Copy text into the clipboard
QApplication::clipboard()->setData(new QTextDrag(text,worksheet,0));
}

// Paste text from the clipboard
void Table::pasteSelection()
{
QApplication::setOverrideCursor(waitCursor);
	
QString text;		
if ( !QTextDrag::decode(QApplication::clipboard()->data(), text) || text.isNull())
	{
	QApplication::restoreOverrideCursor();
	return;
	}
	
QTextStream ts( &text, IO_ReadOnly );
QString s = ts.readLine(); 
QStringList cellTexts=QStringList::split ("\t", s, TRUE);
int cols=int(cellTexts.count());
int rows= 1;
while(!ts.atEnd()) 
	{
	rows++;
	s = ts.readLine(); 
	}
ts.reset();
	
int i, j, top, bottom, right, left, firstCol=firstSelectedColumn();
QTableSelection sel=worksheet->selection(worksheet->currentSelection());
if (!sel.isEmpty())
	{// not columns but only cells are selected
	top=sel.topRow();
	bottom=sel.bottomRow();
	left=sel.leftCol();
	right=sel.rightCol();
	}
else
	{
	top=0;
	bottom=worksheet->numRows() - 1;	
	left=0;
	right=worksheet->numCols() - 1;
	if (firstCol>=0)
		{// columns are selected
		left=firstCol;
		right=firstCol+selectedColsNumber()-1;
		}
	}

QTextStream ts2( &text, IO_ReadOnly );	
int r = bottom-top+1;
int c = right-left+1;

QApplication::restoreOverrideCursor();
if (rows>r || cols>c)
	{
	switch( QMessageBox::information(0,"QtiPlot",
		   "The text in the clipboard is larger than your current selection!\
	        \nDo you want to insert cells?",
				      "Yes", "No", "Cancel",
				      0,0) ) 
		{
		case 0:	
			if(cols > c )
				insertCols(left, cols - c);

			if(rows > r)
				{
				if (firstCol >= 0)
					worksheet->insertRows(top, rows - r);
				else
					worksheet->insertRows(top, rows - r + 1);
				}
		break;		
    	case 1:
			rows = r;
			cols = c;		
		break;
    	case 2:
			return;
		break;
		}
	}

QApplication::setOverrideCursor(waitCursor);	
for (i=top; i<top+rows; i++)
	{
	s = ts2.readLine();
	cellTexts=QStringList::split ("\t", s, TRUE);
	for (j=left; j<left+cols; j++)					
		worksheet->setText(i, j, cellTexts[j-left]);
	}
			
for (i=left; i<left+cols; i++)
	{
	emit modifiedData(colName(i));
	}	

emit modifiedTable(this);
QApplication::restoreOverrideCursor();
}

void Table::removeCol()
{
QStringList list=selectedColumns();
removeCol(list);
}

void Table::removeCol(const QStringList& list)
{
QApplication::setOverrideCursor(waitCursor);

QString xColName= colName(xCol);
QString yColName = colName(yCol);

int n = (int) list.count();
QMemArray<int> columns(n);	
for (int i=0; i<n; i++)
	{
	QString name=list[i];
	int id=colIndex(name);
	columns[i]=id;

	QStringList::Iterator it=commandes.at(id);
	if ( it != commandes.end() )
		commandes.remove (it);
	
	it=dateTimeFormats.at(id);
	if ( it != dateTimeFormats.end() )
		dateTimeFormats.remove (it);
	
	it=comments.at(id);
	if ( it != comments.end() )
		comments.remove (it);

	QValueListIterator<int> itt=colTypes.at(id);
	if ( itt != colTypes.end() )
		colTypes.remove (itt);
	
	emit removedCol(name);
	}

columns.sort();
worksheet->removeColumns(columns);

xCol = colIndex(xColName);	
yCol = colIndex(yColName);
	
emit modifiedTable(this);
QApplication::restoreOverrideCursor();
}

void Table::normalizeSelection()
{
QStringList s=selectedColumns();
	
for (int i=0; i<(int)s.count(); i++)
	{
	selectedCol=colIndex(s[i]);
	normalizeCol();
	}
	
emit modifiedTable(this);	
}

void Table::normalizeTable()
{
for (int i=0; i<worksheet->numCols(); i++)
	{
	selectedCol = i;
	normalizeCol();
	}
	
emit modifiedTable(this);	
}

void Table::normalizeCol()
{
double max=worksheet->text(0,selectedCol).toDouble();
double aux=0.0;
int i;
QString text;
int rows=worksheet->numRows();
for (i=0; i<rows; i++)
	{
	text=worksheet->text(i,selectedCol);
	aux=text.toDouble();
	if (!text.isEmpty() && fabs(aux)>fabs(max)) 
		max=aux;
	}
	
if (max == 1.0)
	return;
	
for (i=0;i<rows;i++)
	{
	text=worksheet->text(i,selectedCol);
	aux=text.toDouble();
	if ( !text.isEmpty() )
		worksheet->setText(i,selectedCol,QString::number(aux/max));
	}

QString name=colName(selectedCol);
emit modifiedData(name);
}

void Table::sortManyColDialog()
{
QStringList s=selectedColumns();
sortDialog *sortd=new sortDialog(this,"sortDialog",TRUE,WStyle_Tool|WDestructiveClose);
connect (sortd,SIGNAL(options(const QString&, const QString&, const QString&)),
		 this,SLOT(sortManyCol(const QString&, const QString&, const QString&)));
sortd->insertColumnsList(s);
sortd->showNormal();
sortd->setActiveWindow();
}

void Table::sortTableDialog()
{
sortDialog *sortd=new sortDialog(this,"sortDialog",TRUE,WStyle_Tool|WDestructiveClose);
connect (sortd,SIGNAL(options(const QString&, const QString&, const QString&)),
		 this,SLOT(sortTable(const QString&, const QString&, const QString&)));
sortd->insertColumnsList(colNames());
sortd->showNormal();
sortd->setActiveWindow();
}

void Table::sortTable(const QString& type, const QString& order, 
																	const QString& leadCol)
{
sortManyCol(colNames(), type, order, leadCol);
}

void Table::sortManyCol(const QString& type, const QString& order, 
																	const QString& leadCol)
{
sortManyCol(selectedColumns(), type, order, leadCol);
}

void Table::sortManyCol(const QStringList&s, const QString& type, const QString& order, 
																	const QString& leadCol)
{
int cols=s.count();
if(type=="Separately")
	{
	for(int i=0;i<cols;i++)
		{
		selectedCol=colIndex(s[i]);
		if(order=="Ascending")
			sortColAsc();
		else
			sortColDesc();
		}
	}
else if (type == "Together")
	{
	int i,j, leadcol=colIndex(leadCol);
	int rows=worksheet->numRows();
	QArray<double> r(rows), rtemp(rows);
	// Find the permutation index for the lead col
	size_t *p;
	p=ivector(0,rows-1);
	for (j = 0; j <rows; j++)
			r[j]=worksheet->text(j,leadcol).toDouble();

	gsl_sort_index(p,r,1,rows);
	// Since we have the permutation index, sort all the columns
	for(i=0;i<cols;i++)
		{
		int scol=colIndex(s[i]);
		if (!isEmptyColumn(scol))
			{
			for (j = 0; j <rows; j++)
	 			r[j]=worksheet->text(j,scol).toDouble();

			for (j=0;j<rows;j++)
				{
				int aux=p[j];
				rtemp[j]=r[aux];
				}
			for (j=0;j<rows;j++)
				r[j]=rtemp[j];
		
			if(order=="Ascending")
				{
				for (j=0;j<rows;j++)
					worksheet->setText(j,scol,QString::number(r[j])); 
				}
			else
				{
				for (j=0;j<rows;j++)
					worksheet->setText(j,scol,QString::number(r[rows-j-1])); 
				}
			emit modifiedData(colName(scol));
			}
		}
	free_ivector(p,0,rows-1);
	}
emit modifiedTable(this);	
}

void Table::sortColAsc()
{
//changed from version 0.5.9
int rows=worksheet->numRows();
QArray<int> aux(rows);
QArray<double> r(rows);
int i,n=0;
for (i = 0; i <rows; i++)
	{
	QString text=worksheet->text(i,selectedCol);
	if (!text.isEmpty())
		{
		n++;
		aux[i]=i;
		r[i]=text.toDouble();
		}
	else
		{
		aux[i]=-1;
		r[i]=0.0;
		}
	}

if (!n)
	return;

gsl_vector * v = gsl_vector_alloc (n);
int index;
n=0;
for (i = 0; i <rows; i++)
	{
	index=aux[i];
	if (index>=0)
		{
   		gsl_vector_set (v, n, r[index]);
		n++;
		}	
	}

gsl_sort_vector (v);
n=0;
for (i=0;i<rows;i++)
	{
	index=aux[i];
	if (index>=0)
		{
		worksheet->setText(i,selectedCol,QString::number(gsl_vector_get (v, n))); 
		n++;
		}
	}
gsl_vector_free (v);
QString name=colName(selectedCol);
emit modifiedData(name);
emit modifiedTable(this);
}

void Table::sortColDesc()
{//changed from version 0.5.9
int rows=worksheet->numRows();
QArray<int> aux(rows);
QArray<double> r(rows);
int i,n=0;
for (i = 0; i <rows; i++)
	{
	QString text=worksheet->text(i,selectedCol);
	if (!text.isEmpty())
		{
		n++;
		aux[i]=i;
		r[i]=text.toDouble();
		}
	else
		{
		aux[i]=-1;
		r[i]=0.0;
		}
	}

if (!n)
	return;

gsl_vector * v = gsl_vector_alloc (n);
int index;
n=0;
for (i = 0; i <rows; i++)
	{
	index=aux[i];
	if (index>=0)
		{
   		gsl_vector_set (v, n, r[index]);
		n++;
		}	
	}

gsl_sort_vector (v);

for (i=0;i<rows;i++)
	{
	index=aux[i];
	if (index>=0)
		{
		n--;
		worksheet->setText(i,selectedCol,QString::number(gsl_vector_get (v, n))); 
		}
	}
gsl_vector_free (v);
QString name=colName(selectedCol);
emit modifiedData(name);
emit modifiedTable(this);
}

int Table::tableRows()
{
return worksheet->numRows();
}

int Table::tableCols()
{
return worksheet->numCols();
}

bool Table::isEmptyRow(int row)
{
bool empty=TRUE;
int cols=worksheet->numCols();
for (int i=0;i<cols;i++)
	{
	QString text=worksheet->text(row,i);
	if (!text.isEmpty())
		{
		empty=FALSE;
		break;
		}
	}	
return empty;
}

bool Table::isEmptyColumn(int col)
{
bool empty=TRUE;
int rows=worksheet->numRows();
for (int i=0;i<rows;i++)
	{
	QString text=worksheet->text(i,col);
	if (!text.isEmpty())
		{
		empty=FALSE;
		break;
		}
	}	
return empty;
}

QString Table::saveText()
{
QString text = "<data>\n";
int cols=worksheet->numCols();
int rows=worksheet->numRows();
	
for (int i=0; i<rows; i++)
	{
	if (!isEmptyRow(i))
		{
		text+=QString::number(i)+"\t";
		for (int j=0; j<cols-1; j++)
			text+=worksheet->text(i,j)+"\t";
		
		text+=worksheet->text(i,cols-1)+"\n";
		}
	}	
return text + "</data>\n";
}

void Table::addDataRow(const QString& s, int cols)
{
QStringList	list=QStringList::split("\t", s, TRUE);
int row = list[0].toInt();
for (int i=0;i<cols;i++)
	worksheet->setText(row, i, list[i+1]);
}

int Table::nonEmptyRows()
{
int r=0; 	
for (int i=0;i<worksheet->numRows();i++)
	{
	if (!isEmptyRow(i))
		r++;
	}	
return r;
}

QString Table::text(int row, int col)
{
return worksheet->text(row,col).replace(",", ".");
}

void Table::setText (int row, int col, const QString & text )
{
worksheet->setText(row, col, text); 
}

void Table::freeMatrix()
{
int rows=worksheet->numRows();
int cols=worksheet->numCols();
free_matrix(wMatrix,0,rows-1,0,cols-1);
}

void Table::freeMemory()
{
int rows=worksheet->numRows();
int cols=worksheet->numCols();
free_matrix(wMatrix,0,rows-1,0,cols-1);

cells.clear();//clear cells list
}

void Table::storeCellsToMatrix()
{
int rows=worksheet->numRows();
int cols=worksheet->numCols();
wMatrix=matrix(0,rows-1,0,cols-1);
for (int i=0;i<rows;i++)
	{
	for (int j=0;j<cols;j++)
		{
		QString s = text(i, j);
		if (!s.isEmpty())
			wMatrix[i][j]=s.toDouble();
		}
	}
}

void Table::storeCellsToMemory()
{
int rows=worksheet->numRows();
int cols=worksheet->numCols();
wMatrix=matrix(0,rows-1,0,cols-1);
for (int i=0;i<rows;i++)
	{
	for (int j=0;j<cols;j++)
		{
		QString s = text(i, j);
		cells << s;
		if (!s.isEmpty())
			wMatrix[i][j]=s.toDouble();
		}
	}
}

void Table::setTextFormat(bool applyToAll)
{
if (applyToAll)
	{
	for (int i=selectedCol; i<worksheet->numCols(); i++)
		colTypes[i] = Text;
	}
else
	colTypes[selectedCol] = Text;
}

void Table::setNumericFormat(int f, int precType, const QString& prec,bool applyToAll)
{
int cols=worksheet->numCols();
if (applyToAll)
	{
	for (int i=selectedCol; i<cols; i++)
		setColNumericFormat(f, precType, prec, i);
	}
else
	setColNumericFormat(f, precType, prec, selectedCol);

emit modifiedTable(this);
}

void Table::setColNumericFormat(int f, int precType, const QString& prec, int col)
{
colTypes[col] = Numeric;

QString t=prec;
int p=t.toInt();
int rows=worksheet->numRows();
for (int i=0;i<rows;i++)
	{
	t = worksheet->text(i, col);
	if (!t.isEmpty())
		{
		double val = wMatrix[i][col];
		if (f==0) 
			{
			if (precType)
				t=t.setNum(val, 'f', p);
			}
		else if (f==1)
			{
			if (precType)
				t=t.setNum(val,'e',p);
			}
		worksheet->setText(i, col, t);
		}		
	}
}

void Table::setDateTimeFormatInfo(const QStringList& lst)
{
if (dateTimeFormats == lst)
	return;

dateTimeFormats = lst;
}
	
void Table::setDateTimeFormat(int f, const QString& format, bool applyToAll)
{
QApplication::setOverrideCursor(waitCursor);

int cols=worksheet->numCols();
if (applyToAll)
	{
	for (int j=selectedCol; j<cols; j++)
		setDateTimeFormat(f, format, j);
	}
else
	setDateTimeFormat(f, format, selectedCol);

emit modifiedTable(this);

QApplication::restoreOverrideCursor();	
}

void Table::setDateTimeFormat(int f, const QString& format, int col)
{
switch (f)
	{
	case 2:
		setDateFormat(format, col);
	break;

	case 3:
		setTimeFormat(format, col);
	break;

	case 4:
		setMonthFormat(format, col);
	break;

	case 5:
		setDayFormat(format, col);
	break;

	default:
	break;
	}
}

void Table::setDateFormat(const QString& format, int col)
{
if (dateTimeFormats[col] == format)
	return;

colTypes[col] = Date;
dateTimeFormats[col] = format;
int rows=worksheet->numRows();
int cols=worksheet->numCols();

for (int i=0; i<rows; i++)
	{
	QString s = cells[cols*i + col];
	if (!s.isEmpty())
		{
		QDate d = QDate::fromString (s,Qt::ISODate);
		if (d.isValid())
			worksheet->setText(i, col, d.toString(format));
		else
			worksheet->setText(i, col, "-");
		}
	}
}

void Table::setTimeFormat(const QString& format, int col)
{
if (dateTimeFormats[col] == format)
	return;

int rows=worksheet->numRows();
int cols=worksheet->numCols();

colTypes[col] = Time;
dateTimeFormats[col] = format;

for (int i=0; i<rows; i++)
	{
	QString s = cells[cols*i + col];
	if (!s.isEmpty())
		{
		QTime t = QTime::fromString (s,Qt::TextDate);
		if (t.isValid())
			worksheet->setText(i, col, t.toString(format));
		else
			worksheet->setText(i, col, "-");
		}
	}
}

void Table::setMonthFormat(const QString& format, int col)
{
colTypes[col] = Month;
int rows=worksheet->numRows();
if (format == "shortMonthName")
	{
	for (int i=0;i<rows; i++)
		{
		QString s = worksheet->text(i,col);
		if (!s.isEmpty())
			{
			int month= (int(wMatrix[i][col]))%12;
			if (!month)
				month = 12;
		
			worksheet->setText(i, col, QDate::shortMonthName(month));
			}
		}
	}
else if (format == "longMonthName")
	{
	for (int i=0;i<rows; i++)
		{
		QString t = worksheet->text(i,col);
		if (!t.isEmpty())
			{
			int month= (int(wMatrix[i][col]))%12;
			if (!month)
				month = 12;

			worksheet->setText(i, col, QDate::longMonthName(month));
			}
		}
	}
}

void Table::setDayFormat(const QString& format, int col)
{
colTypes[col] = Day;
int rows=worksheet->numRows();
if (format == "shortDayName")
	{
	for (int i=0;i<rows; i++)
		{
		QString t = worksheet->text(i,col);
		if (!t.isEmpty())
			{
			int day= (int(wMatrix[i][col]))%7;
			if (!day)
				day = 7;

			worksheet->setText(i, col, QDate::shortDayName(day));
			}
		}
	}
else if (format == "longDayName")
	{
	for (int i=0;i<rows; i++)
		{
		QString t = worksheet->text(i,col);
		if (!t.isEmpty())
			{
			int day= (int(wMatrix[i][col]))%7;
			if (!day)
				day = 7;
		
			worksheet->setText(i, col, QDate::longDayName(day));
			}
		}
	}
}

void Table::setRandomValues()
{	
QApplication::setOverrideCursor(waitCursor);

int i;
double max=0.0;
int rows=worksheet->numRows();
QStringList list=selectedColumns();
QArray<double> r(rows);

for (int j=0;j<(int) list.count(); j++)
	{
	QString name=list[j];
	selectedCol=colIndex(name);

	srand(rand());

	for (i=0;i<rows;i++)
		{
		r[i]=rand();
		if (max<r[i]) max=r[i];
		}

	for (i=0;i<rows;i++) 
		{
		r[i]/=max;
		worksheet->setText(i,selectedCol,QString::number(r[i]));	
		}

	emit modifiedData(name);
	}

emit modifiedTable(this);
QApplication::restoreOverrideCursor();
}

void Table::setHeader(QStringList header)
{
QHeader *head=worksheet->horizontalHeader();
for (int i=0;i<(int)worksheet->numCols();i++) 
	{
	QString s = header[i];
	s.replace("(X)","(x)").replace("(Y)","(y)");
	s.replace("(xErr)","(xerr)").replace("(yErr)","(yerr)");
	head->setLabel(i, s.replace("_","-"), -1);
	}

QString xColName=head->label(xCol);
if (xColName.contains("[X]", TRUE) == 0)
	head->setLabel(xCol, xColName+"[X]");

QString yColName=head->label(yCol);
if (yColName.contains("[Y]", TRUE) == 0)
	head->setLabel(yCol, yColName+"[Y]");
}

void Table::setAscValues()
{
QApplication::setOverrideCursor(waitCursor);

int rows=worksheet->numRows();
QStringList list=selectedColumns();

for (int j=0;j<(int) list.count(); j++)
	{
	QString name=list[j];
	selectedCol=colIndex(name);

	for (int i=0;i<rows;i++) 
		worksheet->setText(i,selectedCol,QString::number(i+1));

	emit modifiedData(name);
	}

emit modifiedTable(this);
QApplication::restoreOverrideCursor();
}

QString Table::parseComand(int line,const QString& comand, 
						   const QStringList& colLabels, const QStringList& rowIndexes)
{
QString s,aux=comand;
QStringList labels=colLabels;
double result;

int m=(int)colLabels.count();
QMemArray<double> vars(m); 

bool error=FALSE;
myParser parser, rparser;
try
    {
	for (int i=0; i<m; i++)
		{	
		parser.DefineVar(colLabels[i].ascii(), &vars[i]);	
			
		int row=line;
		if (rowIndexes[i] != "i")
			{
			double l=(double)(line + 1);
			rparser.ClearVar();
			rparser.DefineVar("i", &l);
			rparser.SetExpr(rowIndexes[i].ascii());
	        row=qRound(rparser.Eval()) - 1;
			if (row  < 0 && row >= worksheet->numRows())
				return "";
			}
		
		QStringList list=QStringList::split("_", labels[i], FALSE);	
		int col=list[1].toInt();
		s=worksheet->text(row, col);
		if (!s.isEmpty())
			vars[i]=wMatrix[row][col]; //s.toDouble();
		else 
			return "";
		}

	double index = (double)(line+1);
	parser.DefineVar("i", &index);
	parser.SetExpr(aux.ascii());
	result=parser.Eval();
	}
catch(mu::ParserError &e)
	{
	QMessageBox::critical(0,"QtiPlot - Input function error",e.GetMsg());
	error=TRUE;	
    }

if (error)
	return "";
else
	{
	QString rs = QString::number(result);
	if (rs.contains ("#") || rs.contains ("inf"))//inf or undefined values
		rs = "";
	return rs;
	}
}

void Table::setColValues(const QString& text, const QString& com, 
						const QStringList& colLabels, const QStringList& rowIndexes,
						int startRow, int endRow)
{
int rows=worksheet->numRows();
if (endRow>rows) 
	{
	rows=endRow;
	worksheet->setNumRows(rows);
	}

storeCellsToMatrix();
//we store the table values in memory and we use them instead of the table 
//cells texts which are changing during the following for loop

for (int i=startRow-1; i<endRow; i++) 
	worksheet->setText(i, selectedCol, parseComand(i, com, colLabels, rowIndexes));

//we free the memory	
freeMatrix();

commandes[selectedCol]=text;

QString name=colName(selectedCol);
emit modifiedData(name);
emit modifiedTable(this);
}

void Table::setSelectedRow(int row)
{
clickedRow=row;
}


void Table::plotL()
{	
if (!validate2DPlot())
	return;
	
QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this, s,Graph::Line);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void Table::plotP()
{
if (!validate2DPlot())
	return;

QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this, s,Graph::Scatter);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void Table::plotLP()
{
if (!validate2DPlot())
	return;

QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this, s, Graph::LineSymbols);
else
	QMessageBox::warning(0,"QtiPlot - Plot Error","Please select a column to plot!");
}

void Table::plotVB()
{
if (!validate2DPlot())
	return;

QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this, s,Graph::VerticalBars);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void Table::plotHB()
{
if (!validate2DPlot())
	return;

QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this, s,Graph::HorizontalBars);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void Table::plotArea()
{
if (!validate2DPlot())
	return;

QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this, s,Graph::Area);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void Table::plotPie()
{
if (xCol < 0)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error", "Please set a default X column for this table, first!");
	return;
	}
	
QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this,s, Graph::Pie);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void Table::plotVerticalDropLines()
{
if (!validate2DPlot())
	return;
	
QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this,s,Graph::VerticalDropLines);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void Table::plotSpline()
{
if (!validate2DPlot())
	return;
	
QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this,s,Graph::Spline);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void Table::plotSteps()
{
if (!validate2DPlot())
	return;
	
QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this,s,Graph::Steps);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void Table::plotHistogram()
{
if (!validate2DPlot())
	return;

QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this,s,Graph::Histogram);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void Table::plotVectXYXY()
{
if (!validate2DPlot())
	return;

QStringList s=selectedColumns();
int cols=int(s.count());
if (cols == 2 || cols == 4)
	emit plotCol(this, s,Graph::VectXYXY);
else
	QMessageBox::warning(0,"QtiPlot","Please select two or four columns for this operation!");
}

bool Table::validate2DPlot()
{
if (worksheet->numCols()<2)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need at least two columns for this operation!");
	return false;
	}

if (xCol < 0)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error", "Please set a default X column for this table, first!");
	return false;
	}
	
return true;
}

void Table::plot3DRibbon()
{
if (!validate3DPlot())
	return;
	
if (selectedCol>=0)
	{
	QString s=QString(name())+"_"+worksheet->horizontalHeader()->label(selectedCol);
	emit plot3DRibbon(this,s);
	}
}

void Table::plot3DBars()
{
if (!validate3DPlot())
	return;
if (selectedCol>=0)
	{
	QString s= QString(name())+"_"+worksheet->horizontalHeader()->label(selectedCol);
	emit plotXYZ(this,s,2);
	}
}

void Table::plot3DScatter()
{
if (!validate3DPlot())
	return;
if (selectedCol>=0)
	{
	QString s= QString(name())+"_"+worksheet->horizontalHeader()->label(selectedCol);
	emit plotXYZ(this, s, 0);
	}
}

void Table::plot3DTrajectory()
{
if (!validate3DPlot())
	return;

if (selectedCol>=0)
	{
	QString s=QString(name())+"_"+worksheet->horizontalHeader()->label(selectedCol);
	emit plotXYZ(this,s,1);
	}
}

bool Table::validate3DPlot()
{
if (worksheet->numCols()<2)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need at least two columns for this operation!");
	return false;
	}
if (xCol<0)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need to define a X column first!");
	return false;
	}
if (yCol<0)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need to define a Y column first!");
	return false;
	}
return true;
}

void Table::setSelectedCol(int col)
{
selectedCol = col;
}

int Table::selectedColumn()
{
return selectedCol;
}

void Table::importASCII(const QString &fname, const QString &sep,
						int ignoredLines, bool renameCols)
{
QFile f(fname);
QTextStream t( &f );        // use a text stream

int rows = 1;
int cols = 0;

int i, j;
QString s;
QStringList line;

if ( f.open(IO_ReadOnly) )
	{
    for (i=0; i<ignoredLines; i++)
		t.readLine();

	s = t.readLine();//read first line after the ignored ones
	while ( !t.atEnd() ) 
		{
		t.readLine(); 
		rows++;
		}

	line = QStringList::split(sep, s, true);
	cols = (int)line.count();

	bool allNumbers = true;
	line.gres ( ",", ".", false ); //Qt uses decimal dot
	for (i=0; i<cols; i++)
		{//verify if the strings in the line used to rename the columns are not all numbers
		(line[i]).toDouble(&allNumbers);
		if (!allNumbers)
			break;
		}

	if (renameCols && !allNumbers)
		rows--;

	QProgressBar *pbar=new QProgressBar(this,"QtiPlot - Open file",WType_TopLevel|WStyle_StaysOnTop);
	pbar->setCenterIndicator ( TRUE );
	pbar->setPercentageVisible(TRUE);
	pbar->setCaption("Qtiplot - Reading file...");
	pbar->move((this->parentWidget()->width()-200)/2, this->parentWidget()->height()/2);
	pbar->setTotalSteps(rows);
	pbar->show();
	
	init (rows, cols);

	f.reset();
	for (i=0; i<ignoredLines; i++)
		t.readLine();
		
	if (renameCols && !allNumbers)
		{
		s = t.readLine();
		line = QStringList::split(sep, s, true );	
		setHeader(line);
		}
		
	for (i=0; i<rows; i++)
		{
		s = t.readLine(); 
		line = QStringList::split(sep, s, true );
		for (j=0; j<cols; j++)
			worksheet->setText(i, j, line[j]);
		
		pbar->setProgress(i);
		}
	f.close();
	
	for (j=0; j<cols; j++)		
		worksheet->adjustColumn (j);
	
	delete pbar;
	}
}

void Table::exportASCII(const QString& separator,bool withLabels,bool exportSelection)
{
QString selectedFilter;
QString fname = QFileDialog::getSaveFileName( QString::null,"*.txt;;*.dat;;*.DAT", 0,"file dialog",
	tr("Choose a filename to save under"),&selectedFilter,TRUE);
	
if (!fname.isEmpty() ) 
	{ // the user gave a file name	
	QFileInfo fi(fname);
	QString baseName = fi.fileName();	
	if (baseName.contains(".")==0)
			fname.append(selectedFilter.remove("*"));	
	
	if ( QFile::exists(fname) &&
        	QMessageBox::question(
            0,
            tr("QtiPlot - Overwrite File?"),
            tr("A file called: <p><b>%1</b><p>already exists. "
                "Do you want to overwrite it?")
                .arg(fname),
            tr("&Yes"), tr("&No"),
            QString::null, 0, 1 ) )
        			return ;
	else
		{	
		QApplication::setOverrideCursor(waitCursor);
		exportToASCIIFile(fname, separator, withLabels, exportSelection);
		QApplication::restoreOverrideCursor();
		}
	}
}

bool Table::exportToASCIIFile(const QString& fname, const QString& separator,
							  bool withLabels,bool exportSelection)
{
QFile f(fname);
if ( !f.open( IO_WriteOnly ) ) 
	{
	QApplication::restoreOverrideCursor();
	QMessageBox::critical(0, tr("QtiPlot - ASCII Export Error"),
		tr("Could not write to file: <br><h4>"+fname+ "</h4><p>Please verify that you have the right to write to this location!").arg(fname));
   	return false;
	}

QString text;
int i,j;
int rows=worksheet->numRows();
int cols=worksheet->numCols();
int selectedCols = 0;
int topRow = 0, bottomRow = 0;
size_t *sCols;

if (exportSelection)
		{
		for (i=0; i<cols; i++)
			{
			if (worksheet->isColumnSelected(i)) 
				selectedCols++;
			}

		sCols=ivector(1, selectedCols);
		int aux = 1;
		for (i=0; i<cols; i++)
			{
			if (worksheet->isColumnSelected(i)) 
				{
				sCols[aux] = i;
				aux++;
				}
			}

		for (i=0; i<rows; i++)
			{
			if (worksheet->isRowSelected(i)) 
				{
				topRow = i;
				break;
				}
			}

		for (i=rows - 1; i>0; i--)
			{
			if (worksheet->isRowSelected(i)) 
				{
				bottomRow = i;
				break;
				}
			}
		}

	if (withLabels)
		{
		QStringList header=colNames();
		QStringList ls=header.grep ( QRegExp ("\\D"));
		if (exportSelection)
			{
			for (i=1;i<selectedCols;i++)
				{
				if (ls.count()>0)
					text+=header[sCols[i]]+separator;
				else
					text+="C"+header[sCols[i]]+separator;
				}
					
			if (ls.count()>0)
				text+=header[sCols[selectedCols]] + "\n";
			else
				text+="C"+header[sCols[selectedCols]] + "\n";
			}
		else
			{
			if (ls.count()>0)
				{
				for (j=0; j<cols-1; j++) 
					text+=header[j]+separator;
				text+=header[cols-1]+"\n";	
				}			
			else
				{
				for (j=0; j<cols-1; j++) 
					text+="C"+header[j]+separator;
				text+="C"+header[cols-1]+"\n";	
				}	
			} 
	}// finished writting labels

	if (exportSelection)
		{
		for (i=topRow;i<=bottomRow; i++)
			{
			for (j=1;j<selectedCols;j++)
				text+=worksheet->text(i, sCols[j]) + separator;			
			text+=worksheet->text(i, sCols[selectedCols]) + "\n";
			}

		//free memory
		free_ivector(sCols,1,selectedCols);
		}
	else
		{
		for (i=0;i<rows;i++)
			{
			for (j=0;j<cols-1;j++)
				text+=worksheet->text(i,j)+separator;
			text+=worksheet->text(i,cols-1)+"\n";
			}
		}
QTextStream t( &f );
t << text;
f.close();
return true;
}

void Table::resizeEvent(QResizeEvent *)
{
emit resizedTable(this);
}

void Table::contextMenuEvent(QContextMenuEvent *e)
{
emit showContextMenu();
e->accept();
}

void Table::closeEvent( QCloseEvent *e )
{
   /* bool empty=FALSE;
    for(int i=0;i<worksheet->numCols();i++)
    	{
	if(i==0 || (i>0 && empty))
    	empty=isEmptyColumn(i);
	}
    if (!empty)*/
	if (askOnClose)
    {
    switch( QMessageBox::information(0,tr("QtiPlot"),
				      tr("Do you want to hide or delete <p><b>'"
					   +QString(name())+"'</b> ?"),
				      "Delete", "Hide", "Cancel",
				      0,2) ) 
	{
    case 0:	
	e->accept();
	emit closedTable(this);
	break;

    case 1:
	e->ignore();
	emit hiddenTable(this);
	break;

	case 2:
	e->ignore();
	break;
    } 
    }
    else 
    {
    e->accept();
    emit closedTable(this);
    }
}

void Table::moveCurrentCell()
{
int cols=worksheet->numCols();
int row=worksheet->currentRow();
int col=worksheet->currentColumn();
worksheet->clearSelection (TRUE);
	
if (col+1<cols)
	{
	worksheet->setCurrentCell (row,col+1);
	worksheet->selectCells(row,col+1,row,col+1);
	}
else
	{
	worksheet->setCurrentCell (row+1,0);
	worksheet->selectCells(row+1,0,row+1,0);	
	}
}

bool Table::eventFilter(QObject *object, QEvent *e)
{
QHeader *header = worksheet->horizontalHeader();
if (object != (QObject *)header)
	return FALSE;

int offset =header->offset();
switch(e->type())
    {
        case QEvent::MouseButtonDblClick:
			{
            const QMouseEvent *me = (const QMouseEvent *)e;
			selectedCol = header->sectionAt (me->pos().x()+offset);
			emit optionsDialog();			
			return TRUE; 
			}

		case QEvent::MouseButtonPress:
			{
			const QMouseEvent *me = (const QMouseEvent *)e;
			if (me->button() == QMouseEvent::RightButton)	
				{
				const QMouseEvent *me = (const QMouseEvent *)e;
				selectedCol=header->sectionAt (me->pos().x() + offset);
				emit colMenu(selectedCol);
				worksheet->setCurrentCell (0, selectedCol);
				}
			
			if (me->button() == QMouseEvent::LeftButton)	
				{
				LeftButton=TRUE;
				const QMouseEvent *me = (const QMouseEvent *)e;
				
				if (((const QMouseEvent *)e)->state ()==Qt::ControlButton)
					{
					int current=worksheet->currentSelection();
					QTableSelection sel=worksheet->selection(current);
					if (sel.topRow() != 0 || sel.bottomRow() != (worksheet->numRows() - 1))
						//select only full columns
						worksheet->removeSelection(sel);						
					}
				else
					worksheet->clearSelection (TRUE);
				
				selectedCol=header->sectionAt (me->pos().x()+offset);
				lastSelectedCol=selectedCol;
				worksheet->selectColumn (selectedCol);
				worksheet->setCurrentCell (0, selectedCol);
				}		
			return TRUE; 
			}
		
		case QEvent::MouseMove:
			{
			if(LeftButton)
				{
				const QMouseEvent *me = (const QMouseEvent *)e;
				selectedCol=header->sectionAt (me->pos().x() + offset);

				if(selectedCol != lastSelectedCol)
					{// This means that we are in the next column
					if(worksheet->isColumnSelected(selectedCol,TRUE))
						{//Since this column is selected, deselect it
						worksheet->removeSelection(QTableSelection (0,lastSelectedCol,worksheet->numRows()-1,lastSelectedCol));
						}
					else
						worksheet->selectColumn (selectedCol);
					}
				lastSelectedCol=selectedCol;
				worksheet->setCurrentCell (0, selectedCol);
				}
			return TRUE;
			}
			
		case QEvent::MouseButtonRelease:
			{
			LeftButton=FALSE;
			return TRUE;
			}
			
			default:
				;
    }
return QObject::eventFilter(object, e);
}

void Table::showColStatistics()
{
QString caption=QString(name())+"_stat-Statistics of "+QString(name());
	
int rows=worksheet->numRows();
QStringList columns=selectedColumns();	
int n=(int)columns.count();
int colsNonEmpty = 0;

QString text="Col Name\t";
text+="Rows\t";
text+="Mean\t";
text+="Standard Dev\t";
text+="Variance\t";	
text+="Sum\t";
text+="iMax\t";	
text+="Max\t";
text+="iMin\t";	
text+="Min\t";
text+="N\n";

for (int i=0; i<n; i++)
	{
	int index=colIndex(columns[i]);
	text+=colLabel(index)+"\t";
	text+="[1:"+QString::number(rows)+"]\t";
	
	int j, start = 0, m = 0;
	for (j=0; j<rows; j++)
		{
		if (!worksheet->text(j,index).isEmpty())
			{
			start = j;
			m++;
			break;
			}
		}
	for (j = start + 1; j<rows; j++)
		{
		if (!worksheet->text(j,index).isEmpty())
			m++;
		}

	if (m > 0 && colTypes[index] == Numeric)
		{
		double *dat = vector(0, m-1);
		gsl_vector *y = gsl_vector_alloc (m);
		
		int aux = 0, min_index = start, max_index = start;
		double val = worksheet->text(start, index).toDouble();
		gsl_vector_set (y, 0, val);
		dat[0] = val;
		double min = val, max = val;
		for (j = start + 1; j<rows; j++)
			{
			if (!worksheet->text(j,index).isEmpty())
				{
				aux++;
				val = worksheet->text(j, index).toDouble();
				gsl_vector_set (y, aux, val);
				dat[aux] = val;
				if (val < min)
					{
					min = val;
					min_index = j;
					}
				if (val > max)
					{
					max = val;
					max_index = j;
					}
				}
			}
		double mean=gsl_stats_mean (dat,1,m);	
		text+=QString::number(mean)+"\t";
		text+=QString::number(gsl_stats_sd (dat,1,m))+"\t";
		text+=QString::number(gsl_stats_variance (dat,1,m))+"\t";
		text+=QString::number(mean*m)+"\t";

		text+=QString::number(max_index + 1)+"\t";
		text+=QString::number(max)+"\t";
		
		text+=QString::number(min_index + 1)+"\t";
		text+=QString::number(min)+"\t";	
		text+=QString::number(m)+"\n";	
		
		gsl_vector_free (y);
		free_vector(dat,0,m-1);
		}
	else
		text+="\t\t\t\t\t\t\t\t\n";

	if (m > 0)
		colsNonEmpty++;
	}
emit createTable(caption,colsNonEmpty,11,text);
}

void Table::showRowStatistics()
{
QString caption=QString(name())+"-statistics";
int rows=nonEmptyRows();
int cols=worksheet->numCols();
int r=0;
	
QString text="Row\t";
text+="Cols\t";
text+="Mean\t";
text+="Standard Dev\t";
text+="Variance\t";	
text+="Sum\t";
text+="Max\t";
text+="Min\t";
text+="N\n";
	
for (int i=0;i<rows;i++)
	{
	if (worksheet->isRowSelected(i,TRUE))
		{
		r++;
		text+=QString::number(i+1)+"\t";
		text+=QString::number(cols)+"\t";
	
		int j, m = 0;		
		for (j = 0; j<cols; j++)
			{
			if (!worksheet->text(i,j).isEmpty() && colTypes[j] == Numeric)
				m++;	
			}

		if (m > 0)
			{
			double *dat = vector(0,m-1);
			gsl_vector *y = gsl_vector_alloc (m);
			int aux = 0;
			for (j = 0; j<cols; j++)
				{
				QString text = worksheet->text(i,j);
				if (!text.isEmpty() && colTypes[j] == Numeric)
					{					
					double val = text.toDouble();
					gsl_vector_set (y, aux, val);
					dat[aux] = val;
					aux++;
					}
				}
			double mean=gsl_stats_mean (dat,1,m);	
			text+=QString::number(mean)+"\t";
			text+=QString::number(gsl_stats_sd (dat,1,m))+"\t";
			text+=QString::number(gsl_stats_variance (dat,1,m))+"\t";
			text+=QString::number(mean*m)+"\t";

			double min, max;
			gsl_vector_minmax (y, &min, &max);
			text+=QString::number(max)+"\t";		
			text+=QString::number(min)+"\t";		
			text+=QString::number(m)+"\n";	
		
			gsl_vector_free (y);
			free_vector(dat,0,m-1);
			}
		}
	}
emit createTable(caption, r, 9, text);
}

QString Table::sizeToString()
{
QString s=makeSpecifications();
int size=s.length()*sizeof(QString);

if (size>1024)
	return QString::number(s.length()*sizeof(QString)/1024)+" kB";
else
	return QString::number(s.length()*sizeof(QString))+" B";
}


QString Table::makeSpecifications()
{
int cols=worksheet->numCols();
	
QString s="<table>\n";
s+=QString(name())+"\t";
s+=QString::number(worksheet->numRows())+"\t";
s+=QString::number(cols)+"\t";
s+=birthdate+"\t";	
s+=QString::number(xCol)+"\t";
s+=":\t";
s+=QString::number(yCol)+"\n";
s+="geometry\n"; //useles
s+=saveHeader();
s+=saveColumnWidths();
s+=saveCommandes();
s+=saveColumnTypes();
s+=saveComments();
s+="WindowLabel\t" + windowLabel() + "\t" + QString::number(captionPolicy()) + "\n";
s+=saveText();
s+="</table>";
return s;
}

QString& Table::getSpecifications()
{
return specifications;
}

void Table::setSpecifications(const QString& s)
{
if (specifications == s)
	return;

specifications=s;
}

void Table::setNewSpecifications()
{
newSpecifications=makeSpecifications();
}

QString& Table::getNewSpecifications()
{
return newSpecifications;
}

QString Table::oldCaption()
{
QTextStream ts( &specifications, IO_ReadOnly );
ts.readLine();
QString s=ts.readLine();
int pos=s.find("\t",0);
return s.left(pos);
}

QString Table::newCaption()
{
QTextStream ts(&newSpecifications, IO_ReadOnly );
ts.readLine();
QString s=ts.readLine();
int pos=s.find("\t",0);
return s.left(pos);
}

void Table::restore(QString& spec)
{
int i, j, row;
int cols=worksheet->numCols();
int rows=worksheet->numRows();

QTextStream t(spec, IO_ReadOnly);	
t.readLine();	//table tag
QString s= t.readLine();
QStringList list=QStringList::split ("\t",s,TRUE);

QStringList oldHeader = colNames();	
QString oldCaption = name();
QString newCaption=list[0];
if (oldCaption != newCaption)
	this->setName(newCaption);

int r=list[1].toInt();
if (rows != r)
	worksheet->setNumRows(r);
		
int c=list[2].toInt();
if (cols != c)
	worksheet->setNumCols(c);

xCol=list[4].toInt();
if ((int)list.count()>=7)
	yCol=list[6].toInt();

//clear all cells
for (i=0;i<rows;i++)
	{
	for (j=0; j<c; j++)
		worksheet->setText(i,j, "");
	} 

t.readLine();	//table geometry useless info
s= t.readLine();	//header line
	
list = QStringList::split ("\t",s,TRUE);
list.remove(list.first());
			
if (!oldHeader.isEmpty() && oldHeader != list)
	{
	list.gres("[X]","");
	list.gres("[Y]","");
	setHeader(list);

	for (j=0;j<c;j++)
		{
		if (!list.contains(oldHeader[j]))
			emit changedColHeader(newCaption + "_"+oldHeader[j], newCaption+"_"+list[j]);
		}

	if (c<cols)
		{
		for (j=0;j<c;j++)
			{
			if (!list.contains(oldHeader[j]))
				emit removedCol(oldCaption + "_" + oldHeader[j]);
			}
		}
	}			

s= t.readLine();	//colWidth line
list = QStringList::split ("\t", s,TRUE);
list.remove(list.first());
if (columnWidths() != list)
	setColWidths(list);

s = t.readLine();	//commandes line
list = QStringList::split ("\t", s,TRUE);
list.remove(list.first());
if (list != commandes)
	commandes = list;

s= t.readLine();	//colType line ?
list = QStringList::split ("\t", s,TRUE);
if (s.contains ("ColType",TRUE))
	{
	list.remove(list.first());	
	for (i=0; i<int(list.count()); i++)
		{
		QStringList l= QStringList::split(";", list[i], true);
		colTypes[i] = l[0].toInt();
		
		if ((int)l.count() > 0)
			dateTimeFormats[i] = l[1];
		}
	}	
else //if fileVersion < 65 set table values
	{
	row = list[0].toInt();
	for (j=0; j<cols; j++)
		worksheet->setText(row, j, list[j+1]);	
	}

s= t.readLine();	//comments line ?
list = QStringList::split ("\t", s,TRUE);
if (s.contains ("Comments",TRUE))
	{
	list.remove(list.first());
	comments = list;	
	}

s= t.readLine();
list = QStringList::split ("\t", s,TRUE);
if (s.contains ("WindowLabel",TRUE))
	{
	setWindowLabel(list[1]);
	setCaptionPolicy((myWidget::CaptionPolicy)list[2].toInt());
	}

s= t.readLine();	
if (s == "<data>")	
	s = t.readLine();	

while (!t.atEnd () && s != "</data>")
	{
	list = QStringList::split ("\t", s,TRUE);

	row = list[0].toInt();
	for (j=0; j<c; j++)
		worksheet->setText(row, j, list[j+1]);	
	
	s= t.readLine();	
	}	

for (j=0; j<c; j++)
	emit modifiedData(colName(j));	
}

void Table::setNumRows(int rows)
{
worksheet->setNumRows(rows);
}

void Table::setNumCols(int cols)
{
worksheet->setNumCols(cols);
}

void Table::resizeRows(int r)
{
int rows = worksheet->numRows();
if (rows == r)
	return;

if (rows > r)
	{
	QString text="Rows will be deleted from the table!";
	text+="<p>Do you really want to continue?";
	int i,cols = worksheet->numCols();
	switch( QMessageBox::information(0,"QtiPlot", text,"Yes", "Cancel", 0, 1 ) ) 
		{
    	case 0:
		QApplication::setOverrideCursor(waitCursor);
		worksheet->setNumRows(r);
		for (i=0; i<cols; i++)
			emit modifiedData(colName(i));
		
		QApplication::restoreOverrideCursor();
		break;
		
   	    case 1:
        	return;
		break;
    	}
	}
else
	{
	QApplication::setOverrideCursor(waitCursor);
	worksheet->setNumRows(r);
	QApplication::restoreOverrideCursor();
	}

emit modifiedTable(this);
}

void Table::resizeCols(int c)
{
int i, cols = worksheet->numCols();	
if (cols == c)
	return;

if (cols > c)
	{
	QString text="Columns will be deleted from the table!";
	text+="<p>Do you really want to continue?";
	QMemArray<int> columns (cols-c);
	switch( QMessageBox::information(0,"QtiPlot", text,"Yes", "Cancel", 0, 1 ) ) 
		{
    	case 0:
		QApplication::setOverrideCursor(waitCursor);
		for (i=c; i<cols; i++)
			{
			QString name = colName(i);
			emit removedCol(name);
			columns[i-c]=i;

			QStringList::Iterator it=commandes.end();
			commandes.remove (--it);
	
			QValueListIterator<int> itt=colTypes.end();
			colTypes.remove (--itt);
			}
		
		worksheet->removeColumns (columns);

		if (xCol >= c)
			xCol = -1;
				
		if (yCol >= c)
			yCol = -1;	

		QApplication::restoreOverrideCursor();
		break;
		
   	    case 1:
        	return;
		break;
    	}
	}
else
	{
	QApplication::setOverrideCursor(waitCursor);
	worksheet->setNumCols(c);
	for (i=cols; i<c; i++)
			{
			commandes<<"col("+colLabel(xCol)+")";
			colTypes<<Numeric;
			}
	QApplication::restoreOverrideCursor();
	}
emit modifiedTable(this);
}

Table::~Table()
{
}
