#ifndef WORKSHEET_H
#define WORKSHEET_H

#include <qwidget.h>
#include <qlist.h>
#include <qtable.h>
#include <qheader.h>
#include <qpopupmenu.h>

#include "graph.h"
#include "widget.h"

class Table: public myWidget
{
    Q_OBJECT

public:

	enum ColType{Numeric = 0, Text = 1, Date = 2, Time = 3, Month = 4, Day = 5};

   	Table(const QString &fname,const QString &sep, int ignoredLines, 
			bool renameCols, const QString &label, QWidget* parent=0, const char* name=0, WFlags f=0);
	Table(int r,int c, const QString &label, QWidget* parent=0, const char* name=0, WFlags f=0);
	~Table();

public slots:

	QString sizeToString();
	int tableRows();
	int tableCols();
	void setNumRows(int rows);
	void setNumCols(int cols);
	void resizeRows(int);
	void resizeCols(int);

	QString text(int row, int col);
	QStringList colNames();
	QString colName(int col);
	QString colLabel(int col);
	int colIndex(const QString& name);
	void setColName(int col,const QString& text);
	void setHeader(QStringList header);
	void setText(int row,int col,const QString & text);
	void setColValues(const QString& text, const QString& com, 
					const QStringList& colLabels, const QStringList& rowIndexes,
					int startRow, int endRow);
	void setRandomValues();
	void setAscValues();

	QStringList getCommandes();
    void setCommandes(const QStringList& com);
	void setCommandes(const QString& com);
	void cellEdited(int,int col);
	void moveCurrentCell();
	void clearCell(int row, int col);
	QString saveText();
	bool isEmptyRow(int row);
	bool isEmptyColumn(int col);
	int nonEmptyRows();
	
	void plotL();
	void plotLP();
	void plotP();
	void plotVB();
	void plotHB();
	void plotArea();
	void plotPie();	
	void plotVerticalDropLines();
	void plotSpline();
	void plotSteps();
	void plotHistogram();
	void plotVectXYXY();
	
	//3D plots
	void plot3DRibbon();
	void plot3DScatter();
	void plot3DTrajectory();
	void plot3DBars();
	
	bool validate2DPlot();
	bool validate3DPlot();
	
	void print();
	
	// event handlers 
	void closeEvent( QCloseEvent *);
	void resizeEvent(QResizeEvent *);
	bool eventFilter(QObject *object, QEvent *e);
	void contextMenuEvent(QContextMenuEvent *e);
	
	// set col values
	QString parseComand(int line, const QString& comand, 
								const QStringList& colLabels, const QStringList& rowIndexes);
	
	// column operations 
	void removeCol();
	void removeCol(const QStringList& list);
	void clearCol();
	void insertCol();
	void insertCols(int start, int count);
	void addCol();	
	void sortTableDialog();
	void sortTable(const QString& type, const QString& order, 
																	const QString& leadCol);
	void sortColAsc();
	void sortColDesc();
	void sortManyCol(const QString& type, const QString& order, 
																	const QString& leadCol);
	void sortManyCol(const QStringList&s, const QString& type, const QString& order, 
																	const QString& leadCol);

	void sortManyColDialog();
	
	//normalization
	void normalizeCol();
	void normalizeSelection();
	void normalizeTable();

	QArray<double> col(int ycol);
	int colX();
	bool setXCol();
	void setXCol(int i);
	
	int colY();
	bool setYCol();
	void setYCol(int i);

	void disregardCol();

	int atRow(int col, double value);
	void showColStatistics();
	void showRowStatistics(); 
	
	// row operations 
	void deleteRow();
	void deleteSelectedRows();
	void insertRow();
	void addDataRow(const QString& s, int cols);
	void setSelectedRow(int row);

	// selection operations 
	void cutSelection();
	void copySelection();
	void clearSelection();
	void pasteSelection();
	void selectAllTable();
	void deselect();
	bool singleCellSelected();
	bool multipleRowsSelected();

	void init(int rows, int cols);
	QStringList selectedColumns();
	int selectedColsNumber();
	void changeColName(const QString& text);
	void enumerateRightCols(bool checked);

	void changeColWidth(int width, bool allCols);	
	int columnWidth(int col);
	QStringList columnWidths();
	void setColWidths(const QStringList& widths);
	
	int selectedColumn();
	int firstSelectedColumn();
	void setSelectedCol(int col);
	int selectedRows();
	
	int columnType(int col){return colTypes[col];};

	QValueList<int> columnTypes(){return colTypes;};
	void setColumnTypes(QValueList<int> ctl){colTypes = ctl;};
	void setColumnTypes(const QStringList& ctl);

	void storeCellsToMatrix();
	void freeMatrix();
	void storeCellsToMemory();
	void freeMemory();

	QStringList getDateTimeFormats(){return dateTimeFormats;};
	void setDateTimeFormatInfo(const QStringList& lst);
	
	void setTextFormat(bool applyToAll);
	void setColNumericFormat(int f, int precType, const QString& prec, int col);
	void setDateFormat(const QString& format, int col);
	void setTimeFormat(const QString& format, int col);
	void setMonthFormat(const QString& format, int col);
	void setDayFormat(const QString& format, int col);
	void setDateTimeFormat(int f, const QString& format,int col);
	void setDateTimeFormat(int f, const QString& format,bool applyToAll);
	void setNumericFormat(int f, int precType, const QString& prec,bool applyToAll);

	QString birthDate();
	
	void exportASCII(const QString& separator,bool withLabels,bool exportSelection);
	bool exportToASCIIFile(const QString& fname, const QString& separator,
							  bool withLabels,bool exportSelection);

	void importASCII(const QString &fname,const QString &sep, 
					 int ignoredLines, bool renameCols);

	//saving
	QString saveToString(const QString& geometry, const QString& birth);
	QString saveHeader();
	QString saveComments();
	QString saveCommandes();
	QString saveColumnWidths();
	QString saveColumnTypes();

	//restoring
	QString makeSpecifications();
	void setSpecifications(const QString& s);
	QString& getSpecifications();
	void restore(QString& spec);
	QString& getNewSpecifications();
	void setNewSpecifications();
	
	/*!
	*used for restoring the table old caption stored in specifications string
	*/ 
	QString oldCaption();
	
	/*!
	*used for restoring the table caption stored in new specifications string
	*/ 
	QString newCaption();

	void setBackgroundColor(const QColor& col);
	void setTextColor(const QColor& col);
	void setHeaderColor(const QColor& col);
	void setTextFont(const QFont& fnt);
	void setHeaderFont(const QFont& fnt);
	
	void askOnCloseEvent(bool ask){askOnClose = ask; };
	
	int verticalHeaderWidth(){return worksheet->verticalHeader()->width();};

	QString colComment(int col){return comments[col];};
	void setColComment(const QString& s);

	QStringList colComments(){return comments;};
	void setColComments(const QStringList& lst){comments = lst;};
				
signals:
	void plot3DRibbon(Table *,const QString&);
	void plotXYZ(Table *,const QString&, int);

	void plotCol(Table *,const QStringList&, int);
	void closedTable(QWidget*);
	void changedColHeader(const QString&, const QString&);
	void removedCol(const QString&);
	void modifiedData(const QString&);
	void modifiedTable(QWidget*);
	void optionsDialog();
	void colValuesDialog();
	void resizedTable(QWidget*);
	void hiddenTable(QWidget*);
	void rowMenu(int);
	void colMenu(int);
	void showContextMenu();
	void createTable(const QString&,int,int,const QString&);
	
private:
	QTable *worksheet;
	QString birthdate, specifications, newSpecifications;
	QStringList commandes, cells, dateTimeFormats, comments;
	QValueList<int> colTypes;
	int selectedCol, lastSelectedCol, clickedRow,xCol,yCol;
	double **wMatrix; //global matrix used to store the values of the table cells
	bool LeftButton;
	bool askOnClose;
};

#endif
